package com.bull.cimero.pluginEditor.editors.commands;

import org.eclipse.gef.commands.Command;
import com.bull.cimero.pluginEditor.editors.model.Connection;
import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;

/**
 * This class is used to create the command.
 * to delete connection beetween two objects.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class ConnectionDeleteCommand extends Command {

	/** source parameter of the connection. */
	private GeneriqueCimeroModel source;
	/** target parameter of the connection. */
	private GeneriqueCimeroModel target;
	/** current connection. */
	private Connection connection;

	/**
	 * set the source of the connection.
	 * @param sourceParam the source given in parameter
	 */
	public final void setSource(final GeneriqueCimeroModel sourceParam) {
	   this.source = sourceParam;
	}

	/**
	 * set the target of the connection.
	 * @param targetParam the target given in parameter
	 */
	public final void setTarget(final GeneriqueCimeroModel targetParam) {
	   this.target = targetParam;
	}

	/**
	 * set the connection.
	 * @param connectionParam the connection given in parameter
	 */
	public final void setConnection(final Connection connectionParam) {
	  this.connection = connectionParam;
	}

	/**
    * function to verify the method is executable.
    * @return boolean
    */
   public final boolean canExecute() {
	   //if there is a connection -> source and target != null
      if (getTarget() != null) {
    	  if (getSource() != null) {
    		  return true;
    	  }
      }
      return false;
   }

   /**
    * execute the method.
    */
   public final void execute() {
	   this.source.removeOutput(this.connection);
	   this.target.removeInput(this.connection);
   }

   /**
    * undo the method.
    */
   public final void undo() {
	   this.source.addOutput(this.connection);
	   this.target.addInput(this.connection);
   }

   /**
    * redo the method.
    */
   public final void redo() {
	   this.source.removeOutput(this.connection);
	   this.target.removeInput(this.connection);
   }

   /**
   * get the target of the connection.
   * @return GeneriqueCimeroModel
   */
   public final GeneriqueCimeroModel getTarget() {
		return target;
	}

  /**
  * get the source of the connection.
   * @return GeneriqueCimeroModel
  */
  public final GeneriqueCimeroModel getSource() {
		return source;
	}

  /**
   * get the connection.
   * @return Connection
   */
  public final Connection getConnection() {
		return connection;
	}
}
