package com.bull.cimero.pluginEditor.editors.figure.BCFigure;

import org.eclipse.draw2d.geometry.Dimension;
import com.bull.cimero.pluginEditor.editors.figure.GeneriqueCimeroFigure;

/**
 * This class is the parent class of all BindingComponent Figure
 * It contain all the fonctionnalities which permit to change the
 * Mep mode on the figure.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public abstract class INOUTFigure extends GeneriqueCimeroFigure {

	/** boolean : true if input. */
	private boolean input;
	/** boolean : true if output. */
	private boolean output;
	/** figure path. */
	private String pathToFigure = "";
	/** figure error path. */
	private String pathToErrorFigure = "";
	/** figure icon. */
	private String pathToIcon = "";

	/**
	 * default constructor.
	 */
	public INOUTFigure() {
		super();
		setInput(false);
		setOutput(false);
	}

	/**
	 * @return Dimension : wanted dimension
	 * @see com.bull.cimero.pluginEditor
	 * .editors.figure.GeneriqueCimeroFigure
	 * #getFigureDimension()
	 */
	@Override
	public abstract Dimension getFigureDimension();

	/**
	 * get the input boolean.
	 * @return boolean : true if input
	 */
	public final boolean isInput() {
		return input;
	}

	/**
	 * set the input boolean.
	 * @param inputParam : input to set
	 */
	public final void setInput(final boolean inputParam) {
		this.input = inputParam;
	}

	/**
	 * get the output boolean.
	 * @return boolean : true if output
	 */
	public final boolean isOutput() {
		return output;
	}

	/**
	 * set the output boolean.
	 * @param outputParam : output to set
	 */
	public final void setOutput(final boolean outputParam) {
		this.output = outputParam;
	}

	/**
	 * method used to refresh the port.
	 * @param inputParam : boolean
	 * @param outputParam : boolean
	 */
	public final void refreshPort(
					final boolean inputParam, final boolean outputParam) {
		//is there is an input in model and there is no input in figure
		if ((inputParam) && (!isInput())) {
			addDefaultInput();
			//addDefaultInput();
			setInput(true);
		}
		//is there is an output in model and there is no output in figure
		if ((outputParam) && (!isOutput())) {
			addDefaultOutput();
			setOutput(true);
		}
		//is there is no input in model and there is an input in figure
		if ((!inputParam) && (isInput())) {
			removeInput("input");
			setInput(false);
		}
		//is there is no output in model and there is an output in figure
		if ((!outputParam) && (isOutput())) {
			removeOutput("output");
			setOutput(false);
		}
	}

	/**
	 * get the path to the error figure.
	 * @return Returns the pathToErrorFigure.
	 */
	@Override
	public final String getPathToErrorFigure() {
		return pathToErrorFigure;
	}

	/**
	 * set the path to the error figure.
	 * @param pathToErrorFigureP The pathToErrorFigure to set.
	 */
	@Override
	public final void setPathToErrorFigure(final String pathToErrorFigureP) {
		this.pathToErrorFigure = pathToErrorFigureP;
	}

	/**
	 * get the path to the figure.
	 * @return Returns the pathToFigure.
	 */
	@Override
	public final String getPathToFigure() {
		return pathToFigure;
	}

	/**
	 * set the path to the figure.
	 * @param pathToFigureParam The pathToFigure to set.
	 */
	@Override
	public final void setPathToFigure(final String pathToFigureParam) {
		this.pathToFigure = pathToFigureParam;
	}

	/**
	 * get the path to the icon.
	 * @return Returns the pathToIcon.
	 */
	@Override
	public final String getPathToIcon() {
		return pathToIcon;
	}

	/**
	 * set the path to the figure.
	 * @param pathToIconParam The pathToFigure to set.
	 */
	@Override
	public final void setPathToIcon(final String pathToIconParam) {
		this.pathToIcon = pathToIconParam;
	}

}
