package com.bull.cimero.pluginEditor.editors.figure.BCFigure;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.FlowLayout;
import org.eclipse.draw2d.ImageFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.swt.graphics.Image;
import cimeroEditor.CimeroEditorPlugin;

/**
 * This class permit to create a figure that contain a image.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class ImageCanal extends Figure {

	/** path of the icon. */
	private String iconePath = "";
	/** path of the error icon. */
	private String iconePathError = "";
	/** The image. */
	private Image monImage = null;
	/** The printed image. */
	private ImageFigure img;
	/** constant : size icone. */
	private static final int DEFAULT_SIZE_ICON = 50;

	/**
	 * The constructor.
	 * @param iconePathParam : path to the normal icone
	 * @param iconePathErrorParam : path to the error image
	 */
	public ImageCanal(
			final String iconePathParam
			, final String iconePathErrorParam) {
		super();
		FlowLayout layout = new FlowLayout();
		setLayoutManager(layout);
		//init variable
		this.iconePath = iconePathParam;
		this.iconePathError = iconePathErrorParam;
		monImage = CimeroEditorPlugin.getImageDescriptor(
				this.iconePath).createImage();
		img = new ImageFigure(monImage);
		add(img);
	}

	/**
	 * method used to set the error image
	 * and not the normal one.
	 */
	public final void setErrorImage() {
		remove(img);
		monImage = CimeroEditorPlugin.getImageDescriptor(
				this.iconePath).createImage();
		img = new ImageFigure(monImage);
		add(img);

	}

	/**
	 * method used to set the normal image
	 * and not the error one.
	 */
	public final void setNormalImage() {
		monImage = CimeroEditorPlugin.getImageDescriptor(
				this.iconePathError).createImage();
		remove(img);
		img = new ImageFigure(monImage);
		add(img);

	}

	/**
	 * This function return the preferred size of this image.
	 * @param wHint : width
	 * @param hHint : height
	 * @return Dimension : dimension wanted
	 */
	public final Dimension getPreferredSize(
			final int wHint, final int hHint) {
		Dimension dim = null;
		if (monImage != null) {
			dim = new Dimension(monImage);
		} else {
			dim = new Dimension(DEFAULT_SIZE_ICON, DEFAULT_SIZE_ICON);
		}
		return dim;
	}
}
