

package com.bull.cimero.pluginEditor.editors.figure;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.draw2d.geometry.Rectangle;

/**
 * Provides the base implementation for all the port figures.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class PortFigure extends Figure {

	/** prefered width. */
	private static final int WIDTHPREFERED = 15;
	/** prefered height. */
	private static final int HEIGHTPREFERED = 5;

	/**
	 * The default construtor.
	 */
	public PortFigure() {
		super();

		setOpaque(false);
		setBackgroundColor(ColorConstants.white);
	}

	/**
	 * This function is used to draw the representation of this figure.
	 * @param g : graphic
	 */
	public final void paintFigure(final Graphics g) {
		super.paintFigure(g);

		g.setForegroundColor(ColorConstants.black);

		Rectangle bounds = getBounds();

		g.setBackgroundColor(ColorConstants.black);
		PointList pts = new PointList();
		pts.addPoint(bounds.getTopLeft());
		pts.addPoint(bounds.getTopRight());
		pts.addPoint(bounds.getBottomRight());
		pts.addPoint(bounds.getBottomLeft());
		pts.addPoint(bounds.getTopLeft());
		g.fillPolygon(pts);
		g.drawPolyline(pts);
	}

	/**
	 * Return the dimension of this Figure.
	 * @param hHint : height
	 * @param wHint : width
	 * @return Dimension
	 */
	public final Dimension getPreferredSize(final int wHint, final int hHint) {
		return new Dimension(WIDTHPREFERED , HEIGHTPREFERED);
	}
}
