package com.bull.cimero.pluginEditor.editors.model.BCModel;

import java.io.IOException;
import java.io.ObjectInputStream;

import com.bull.cimero.pluginEditor.utils.log.CimeroLog;

/**
 * The routeur model class.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class FileBC extends BindingComponentModel  {

	
	private static final long serialVersionUID = 1L;
	public static final String DEFAULTNAME = "myFileBC";
	private static final String NAMEDEST = "destination service";
	public static final String DIRECTORY = "directory";
	public static final String FILE = "file";
	private static final String ID = "FILEBC";
	private static final String FILE_MARSHALLER = "marshaller";
	private static final String[] FILE_MARSHALLER_COMBO = {
		"Default file Marshaller","Binary File Marshaller"
	};
	public static final String BINARY_CONTENT_TYPE = "binaryContentType";
	public static final String DEFAULT_FILE_MARSHALLER = "defaultFileMarshaler";
	public  static final String BINARY_FILE_MARSHALLER = "binaryFileMarshaler";
	public static final String IS_DELETE = "isDelete";
	public static final String[] BOOLEAN_COMBO = {
		"TRUE","FALSE"
	};
	private static final String IS_APPEND = "isDelete";
	public static final String PERIOD = "period";

	/**.
	 * The default contructor
	 */
	public FileBC() {
		super(DEFAULTNAME);
		if (setInput(false)) {
			setOutput(true);
		}
		
		//add name properties
		setProperty(NAME, DEFAULTNAME);
		setProperty(NAMEDEST, "");
		setProperty(INOUT, "");
		setProperty(DIRECTORY, "file:");
		setProperty(FILE, "file:");
		setProperty(FILE_MARSHALLER, 0);
		setProperty(IS_DELETE, 0);
		setProperty(IS_APPEND, 1);
		setProperty(PERIOD, "5000");
		setProperty(BINARY_CONTENT_TYPE, "text/plain");
	}

	public String getFileMarshalerValue() {
		int intFileMarshaler = (Integer)getProperty(FILE_MARSHALLER);
		switch (intFileMarshaler) {
		case 0:
			return DEFAULT_FILE_MARSHALLER;
		case 1:
			return BINARY_FILE_MARSHALLER;
		default:
			return DEFAULT_FILE_MARSHALLER;
		}
	}

	/**.
	 * This function is used by the validator. It is used to determine
	 * if this class is validate or not
	 * @param showLog : true if the log must to be show.
	 * @return boolean
	 */
	public final boolean isValide(final boolean showLog) {
		String msg = "";
		boolean returnResult = true;
		if (getName().compareTo("") == 0) {
			msg += "\n- Name => the element name cannot be null";
			returnResult = false;
		}
		if (isOutput()) { 
			//mode IN
			if (getNameFirstDestination().compareTo("") == 0) {
				msg += "\n- destination => "
					+ "the destination service cannot be null";
				returnResult = false;
			}
			if (getFile().equalsIgnoreCase("")) {
				msg += "\n- file => "
					+ "the file cannot be null";
				returnResult = false;
			}
		} else {
			if (getDirectory().equalsIgnoreCase("")) {
				msg += "\n- directory => "
					+ "the directory cannot be null";
				returnResult = false;
			}
			String period = (String)getProperty(PERIOD);
			if (period == null || "".equals(period)) {
				msg += "\n- period => "
					+ "the polling period has to be valorized";
				returnResult = false;
			} else {
				try {
					Long.parseLong(period);
				} catch (NumberFormatException nfe) {
					msg += "\n- period => "
						+ "the polling period must be a long value";
					returnResult = false;
				}
			}
		}
		if (!returnResult) {
			msg = msg.substring(1);
			msg = "ELEMENT NAME : " + getName() + "\n" + msg;
			if (showLog) {
				CimeroLog.warningLog(msg);
			}
		}
		return returnResult;
	}
	
	private String getString(int indice){
		if (indice >= 10)
			return String.valueOf(indice);
		else
			return "0" + String.valueOf(indice);
	}

	/**
	 * reset all property value.
	 * @param id : the id of the element
	 * @see org.eclipse.ui.views.properties.
	 * IPropertySource#resetPropertyValue(java.lang.Object)
	 */
	public final void resetPropertyValue(final Object id) {
		//do nothing
	}
	
	public String getDirectory(){
		return (String)getProperty(DIRECTORY);
	}

	public String getFile(){
		return (String)getProperty(FILE);
	}
	/**
	 * method to create descriptors.
	 * @see com.bull.cimero.pluginEditor.editors.model
	 * .GeneriqueModel#createDescriptor()
	 */
	protected final void createDescriptor() {
		int indice = 1;
		addTextDescriptor(NAME, getString(indice++) + " - name : ");
		addComboBoxDescriptor(INOUT,
				getString(indice++) + " - mode", new String[]{"IN", "OUT"});
		 
		if (!isInput()) {
				//consumer
			addTextDescriptor(FILE, getString(indice++) + " - file ");
			String [] lst = getLstObjectDrawWithInputConnection();
			addComboBoxDescriptor(NAMEDEST, getString(indice++) + " - destination name :", lst);
				 
			addComboBoxDescriptor(IS_DELETE, getString(indice++) + " - Delete polled file",
					BOOLEAN_COMBO);
			addTextDescriptor(PERIOD, getString(indice++) + " - Polling period (ms):");
		} else {
			addTextDescriptor(DIRECTORY, getString(indice++) + " - directory ");
			addComboBoxDescriptor(IS_APPEND, getString(indice++) + " - Append file",
					BOOLEAN_COMBO);

		}
		
		addComboBoxDescriptor(FILE_MARSHALLER, getString(indice++) + " - Select the file management",
			FILE_MARSHALLER_COMBO);
		Integer one = 1;
		if (one.equals(getProperty(FILE_MARSHALLER))) {
			addTextDescriptor(BINARY_CONTENT_TYPE, getString(indice++) + " - Content type");
		}

	}
	
	/**
	 * Abstract method. Permit to set the property value.
	 * @param id : the id of the property
	 * @param value : the value of this property
	 */
	public final void setPropertyValue(final Object id, final Object value) {
		
		if (id == INOUT) {
			System.out.print("Called setPropertyValue -> [" + value +"] isInput["+isInput()+"] isOutput ["+isOutput()+"]\n");
		
			if (isInput()) {
				if (setInput(false)) {
					setOutput(true);
				}
			} else {
				if (setOutput(false)) {
					setInput(true);
				}
			}
		} else if (id == NAMEDEST) {
			//we retreive the destination name
			int selecNum = Integer.parseInt(value.toString());
			//retreive the list af all the objects name
			String [] lstStr = getLstObjectDrawWithInputConnection();
			//call the function to create a connection beetween
			//this object and the selected object
			if (!refreshDestinationConnection(lstStr[selecNum])) {
				CimeroLog.errorLog("problem when created connection",
						new Exception());
			}
		}
		super.setPropertyValue(id, value);
	}

	/**
	 * Return the value of a property.
	 * @param id : the id of the property
	 * @return Object
	 */
	public final Object getPropertyValue(final Object id) {
		if (INOUT.equals(id)) {
			System.out.print("Called getPropertyValue isInput() [" + isInput()+"] isOutput ["+isOutput()+"]\n");
			if (!isInput()) {
				return 0;
			} else {
				return 1;
			}
		} else if (NAMEDEST.equals(id)) {
			//we retreive the name of the destination object
			String destinationName = getNameFirstDestination();
			//return the num of the name  of the object
			String [] lstStr = getLstObjectDrawWithInputConnection();
			for (int i = 0; i < lstStr.length; i++) {
				if (lstStr[i].compareTo(destinationName) == 0) {
					return i;
				}
			}
		}
		return super.getPropertyValue(id);
	}
	
	public boolean isDeleteFile() {
		Integer isDeleteInt = (Integer)getProperty(IS_DELETE);
		return isDeleteInt == null || isDeleteInt == 0? true : false;
	}

	public boolean isAppendFile() {
		Integer isAppendInt = (Integer)getProperty(IS_APPEND);
		return isAppendInt == null || isAppendInt == 1? false : true;
	}

	/**
	 * @return the id of this element.
	 */
	public final String getComponentId() {
		return ID;
	}
	
	private void readObject(ObjectInputStream in)
		throws IOException, ClassNotFoundException {
		
		in.defaultReadObject();
		if (getProperty(FILE_MARSHALLER) == null) {
			setProperty(FILE_MARSHALLER, 0);
		}
		if (getProperty(IS_DELETE) == null) {
			setProperty(IS_DELETE, 0);
		}
		if (getProperty(IS_APPEND) == null) {
			setProperty(IS_APPEND, 1);
		}
		if (getProperty(PERIOD) == null) {
			setProperty(PERIOD, "5000");
		}
		if (getProperty(BINARY_CONTENT_TYPE) == null
				|| "".equals(getProperty(BINARY_CONTENT_TYPE))
			) {
			setProperty(BINARY_CONTENT_TYPE, "text/plain");
		}
	}
}
