package com.bull.cimero.pluginEditor.editors.model.BCModel;

import com.bull.cimero.pluginEditor.utils.log.CimeroLog;

/**
 * This class is the JMS model.
 * It contains the properties of a JMS server
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class JMS extends BindingComponentModel {

	/** serial UID. */
	private static final long serialVersionUID = -1249946678699557300L;
	//
	//CONSTANT
	//
	/** default name. */
	private static final String DEFAULTNAME = "myJMSServer";
	/**The id of this element.*/
	private static final String ID = "Jms";
	//
	//properties
	//
	/**The style id (tpic/queue).*/
	private static final String STYLE = "style";
	/**The style id (tpic/queue).*/
	private static final String PROVIDERDESTINATIONNAME =
		"provider destination name";
	/**The style id (tpic/queue).*/
	private static final String JNDICONNECTIONFACTORYNAME =
		"jndi connection factory name";
	/**The destination id.*/
	private static final String NAMEDEST = "destination";

	/**
	 * default constructor.
	 */
	public JMS() {
		super(DEFAULTNAME);
		if (setInput(false)) {
			setOutput(true);
		}
		setProperty(NAME, DEFAULTNAME);
		setProperty(STYLE, 0);
		setProperty(PROVIDERDESTINATIONNAME, "");
		setProperty(JNDICONNECTIONFACTORYNAME, "");

		setStandard(true);
	}

	/**
	 * default constructor with name in parameter.
	 */
	public JMS(final String name) {
		super(name);
		if (setInput(false)) {
			setOutput(true);
		}
		setProperty(STYLE, 0);
		setProperty(PROVIDERDESTINATIONNAME, "");
		setProperty(JNDICONNECTIONFACTORYNAME, "");

		setStandard(true);
	}

	/**
	 * This function is used to determine if this element is valide.
	 * @return true is this element is valide.
	 * @param showLog true if the log message must to be show.
	 */
	public final boolean isValide(final boolean showLog) {
		String msg = "";
		boolean returnResult = true;
		if (getName().compareTo("") == 0) {
			msg += "\n- Name => the element name cannot be null";
			returnResult = false;
		}
		if (((String) getPropertyValue(PROVIDERDESTINATIONNAME))
				.compareTo("") == 0) {
			msg += "\n- provider destination name => set the name";
			returnResult = false;
		}
		if (((String) getPropertyValue(JNDICONNECTIONFACTORYNAME))
				.compareTo("") == 0) {
			msg += "\n- jndi connection factory name => set the name'";
			returnResult = false;
		}
		if (isOutput()) {
			if (getNameFirstDestination().compareTo("") == 0) {
				msg += "\n- Destination service => the destination of this "
							+ "element cannot be null!";
				returnResult = false;
			}
		}
		if (!returnResult) {
			msg = msg.substring(1);
			msg = "ELEMENT NAME : " + getName() + "\n" + msg;
			if (showLog) {
				CimeroLog.warningLog(msg);
			}
		}
		return returnResult;
	}

	/**
	 * This function is used to create descriptor..
	 */
	protected final void createDescriptor() {
		int indice = 1;
		addTextDescriptor(NAME, indice++ + " - name : ");
		addTextDescriptor(PROVIDERDESTINATIONNAME, indice++
				+ " - provider destination name : ");
		addTextDescriptor(JNDICONNECTIONFACTORYNAME, indice++
				+ " - jndi connection factory name : ");
		addComboBoxDescriptor(STYLE,
	    		indice++ + " - style", new String[]{"topic", "queue"});
		addComboBoxDescriptor(INOUT,
	    		indice++ + " - mode", new String[]{"IN", "OUT"});
		if (isOutput()) {
			String[] lst = getLstObjectDrawWithInputConnection();
			addComboBoxDescriptor(NAMEDEST,
					indice++ + " - destination : ", lst);
		}
	}

	/**
	 * This function is used to set the variable. It is an abstract method
	 * of IpropertySource.
	 * @param id : the id of the object
	 * @param value : the value of this object
	 */
	public final void setPropertyValue(final Object id, final Object value) {
		if (id == INOUT) {
			if (isInput()) {
				if (setInput(false)) {
					setOutput(true);
				}
			} else {
				if (setOutput(false)) {
					setInput(true);
				}
			}
		} else if (id == NAMEDEST) {
			//we retreive the destination name
			int selecNum = Integer.parseInt(value.toString());
			//retreive the list af all the objects name
			String [] lstStr = getLstObjectDrawWithInputConnection();
			//call the function to create a connection beetween
			//this object and the selected object
			if (!refreshDestinationConnection(lstStr[selecNum])) {
				CimeroLog.errorLog("problem when created connection",
						new Exception());
			}
		} else {
			super.setPropertyValue(id, value);
		}
	}

	/**
	 * This function is the implementation of an abstract method of
	 * IPropertySource class.
	 * It is used to return the value of selected element identified by id.
	 * @param id : the id of the element
	 * @return Object : the value of this element
	 */
	public final Object getPropertyValue(final Object id) {
		if (INOUT.equals(id)) {
			if (!isInput()) {
				return 0;
			} else {
				return 1;
			}
		} else if (NAMEDEST.equals(id)) {
			//we retreive the name of the destination object
			String destinationName = getNameFirstDestination();
			//return the num of the name  of the object
			String [] lstStr = getLstObjectDrawWithInputConnection();
			for (int i = 0; i < lstStr.length; i++) {
				if (lstStr[i].compareTo(destinationName) == 0) {
					return i;
				}
			}
		} else {
			return super.getPropertyValue(id);
		}
		return null;
	}

	/**
	 * reset the value of the property.
	 * @param id : the id property
	 */
	public final void resetPropertyValue(final Object id) {

	}

	/**
	 * get the id.
	 * @return the id of this element.
	 */
	public final String getComponentId() {
		return ID;
	}

	/**
	 * @return JMS style (Queue, Topic, ...).
	 */
	public final String getStyle() {
		if ((Integer) getProperty(STYLE) == 0) {
			return "topic";
		} else {
			return "queue";
		}
	}

	/**
	 * set the style property. 0 to topic and 1 to queue.
	 * @param value : the style of the jms component.
	 */
	public final void setStyle(final int value) {
		if ((value == 0) || (value == 1)) {
			setProperty(STYLE, value);
		} else {
			CimeroLog.warningLog("the style value must be have 1 or 0 value");
		}
	}

	/**
	 * @return the provider destination Name.
	 */
	public final String getProviderDestinationName() {
		return (String) getProperty(PROVIDERDESTINATIONNAME);
	}

	/**
	 * set the provider destination name.
	 * @param value : the provider destination name.
	 */
	public final void setProviderDestinationName(final String value) {
		setProperty(PROVIDERDESTINATIONNAME, value);
	}

	/**
	 * @return the jndi destination name.
	 */
	public final String getJndiConnectionFactoryName() {
		return (String) getProperty(JNDICONNECTIONFACTORYNAME);
	}

	/**
	 * set the jndi connection factory name.
	 * @param value : the jndi factory connection name.
	 */
	public final void setJndiConnectionFactoryName(final String value) {
		setProperty(JNDICONNECTIONFACTORYNAME, value);
	}
}
