package com.bull.cimero.pluginEditor.editors.model.BCModel;

import com.bull.cimero.pluginEditor.utils.log.CimeroLog;

/**
 * This class is the Timer model.
 * It contain the properties of a Timer server
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */

public class Timer extends BindingComponentModel {

	/** serialUID. */
	private static final long serialVersionUID = 3182004700564702781L;
	//
	//CONSTANT
	//
	/**The default name of the timer component.*/
	private static final String DEFAULTNAME = "myTimer";
	//
	//properties
	//
	/**The repeat count id.*/
	private static final String REPEATCOUNT = "repeat count";
	/**The repeat interval id.*/
	private static final String REPEATINTERVAL = "repeat interval";
	/**The xml message name id.*/
	private static final String MESSAGENAME = "message name";
	/**The xml message groupe id.*/
	private static final String MESSAGEGROUP = "message group";
	/**The xml message description id.*/
	private static final String MESSAGEDESCRIPTION = "message description";
	/**The destination id.*/
	private static final String NAMEDEST = "destination";
	// variables
	/**The id of this element.*/
	private static final String ID = "Timer";

	/**
	 * The default constructor.
	 */
	public Timer() {
		super(DEFAULTNAME);
		if (setInput(false)) {
			setOutput(true);
		}
		//set the properties
		setProperty(NAME, DEFAULTNAME);
		setProperty(REPEATCOUNT, "-1");
		setProperty(REPEATINTERVAL, "5000");
		setProperty(MESSAGENAME, "");
		setProperty(MESSAGEGROUP, "");
		setProperty(MESSAGEDESCRIPTION, "");
		setProperty(NAMEDEST, "");
	}

	/**
	 * getter REPEATINTERVAL function.
	 * @return int
	 */
	public final int getRepeatInterval() {
		return Integer.parseInt((String) getProperty(REPEATINTERVAL));
	}

	/**
	 * getter REPEATCOUNT function.
	 * @return int
	 */
	public final int getRepeatCount() {
		return Integer.parseInt((String) getProperty(REPEATCOUNT));
	}
	/**
	 * getter MESSAGENAME function.
	 * @return String
	 */
	public final String getMessageName() {
		return (String) getProperty(MESSAGENAME);
	}

	/**
	 * getter MESSAGEGROUP function.
	 * @return String
	 */
	public final String getMessageGROUP() {
		return (String) getProperty(MESSAGEGROUP);
	}

	/**
	 * getter MESSAGEDESCRIPTION function.
	 * @return String
	 */
	public final String getMessageDescription() {
		return (String) getProperty(MESSAGEDESCRIPTION);
	}


	/**
	 * This function is used to determine if this element is valide.
	 * @return true is this element is valide.
	 * @param showLog : true if the log must to be show.
	 */
	public final boolean isValide(final boolean showLog) {
		String msg = "";
		boolean returnResult = true;
		if (getName().compareTo("") == 0) {
			msg += "\n- Name => the element name cannot be null";
			returnResult = false;
		}
		if (((String) getPropertyValue(MESSAGENAME))
				.compareTo("") == 0) {
			msg += "\n- message => set the message name";
			returnResult = false;
		}
		if (((String) getPropertyValue(MESSAGEGROUP)).compareTo("") == 0) {
			msg += "\n- message => set the message group";
			returnResult = false;
		}
		if (getNameFirstDestination().compareTo("") == 0) {
			msg += "\n- destination service => the destination of this "
						+ "element cannot be null!";
			returnResult = false;
		}
		if (!returnResult) {
			msg = msg.substring(1);
			msg = "ELEMENT NAME : " + getName() + "\n" + msg;
			if (showLog) {
				CimeroLog.warningLog(msg);
			}
		}
		return returnResult;
	}

	/**
	 * Create the descriptor to the property views.
	 */
	protected final void createDescriptor() {
		int indice = 1;
		addTextDescriptor(NAME, indice++ + " - name : ");
		addTextDescriptor(REPEATCOUNT, indice++ + " - repeat count : ");
		addTextDescriptor(REPEATINTERVAL, indice++ + " - repeat interval : ");
		addTextDescriptor(MESSAGENAME, indice++
				+ " - name property of the message : ");
		addTextDescriptor(MESSAGEGROUP, indice++
				+ " - group property of the message : ");
		addTextDescriptor(MESSAGEDESCRIPTION, indice++
				+ " - description property of the message : ");
		String[] lst = getLstObjectDrawWithInputConnection();
		addComboBoxDescriptor(NAMEDEST,
				indice++ + " - destination : ", lst);
	}

	/**
	 * This function is used to set the variable. It is an abstract method
	 * of IpropertySource.
	 * @param id : the id of the object
	 * @param value : the value of this object
	 */
	public final void setPropertyValue(final Object id, Object value) {
		if (id == NAMEDEST) {
			//we retreive the destination name
			int selecNum = Integer.parseInt(value.toString());
			//retreive the list af all the objects name
			String [] lstStr = getLstObjectDrawWithInputConnection();
			//call the function to create a connection beetween
			//this object and the selected object
			if (!refreshDestinationConnection(lstStr[selecNum])) {
				CimeroLog.errorLog("problem when created connection",
						new Exception());
			}
		} else if (id == REPEATINTERVAL) {
			String oldValue = (String) getProperty(REPEATINTERVAL);
			try {
				Integer.parseInt((String) value);
			} catch (NumberFormatException e) {
				value = oldValue;
				CimeroLog.errorLog("repeat interval must be a integer value."
						, e);
			}
		} else if (id == REPEATCOUNT) {
			String oldValue = (String) getProperty(REPEATCOUNT);
			try {
				Integer.parseInt((String) value);
			} catch (NumberFormatException e) {
				value = oldValue;
				CimeroLog.errorLog("repeat count must be a integer value."
						, e);
			}
		}
		super.setPropertyValue(id, value);
	}

	/**
	 * This function is the implementation of an abstract method of
	 * IPropertySource class.
	 * It is used to return the value of selected element identified by id.
	 * @param id : the id of the element
	 * @return Object : the value of this element
	 */
	public final Object getPropertyValue(final Object id) {
		if (NAMEDEST.equals(id)) {
			//we retreive the name of the destination object
			String destinationName = getNameFirstDestination();
			//return the num of the name  of the object
			String [] lstStr = getLstObjectDrawWithInputConnection();
			for (int i = 0; i < lstStr.length; i++) {
				if (lstStr[i].compareTo(destinationName) == 0) {
					return i;
				}
			}
		}
		return super.getPropertyValue(id);
	}

	/**
	 * This function is the implementation of an abstract method of
	 * IPropertySource class.
	 * It is used to reset the value of all property.
	 * @param id : the id of the selected element
	 */
	public final void resetPropertyValue(final Object id) {
	}

	/**
	 * get the id.
	 * @return the id of this component.
	 */
	public final String getComponentId() {
		return ID;
	}
}
