package com.bull.cimero.pluginEditor.editors.parts;

import org.eclipse.draw2d.AutomaticRouter;
import org.eclipse.draw2d.BendpointConnectionRouter;
import org.eclipse.draw2d.ConnectionRouter;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.ManhattanConnectionRouter;
import org.eclipse.draw2d.PolygonDecoration;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.ShortestPathConnectionRouter;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.editparts.AbstractConnectionEditPart;
import com.bull.cimero.pluginEditor.editors.policies.ConnectionRemoveEditPolicy;
import com.bull.cimero.pluginEditor.editors.policies.
		ConnectionSelectionEditPolicy;

/**
 * This class is the connection part. It is used to make the bind beetween
 * the connection model and the connection in Eclipse
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class ConnectionPart extends AbstractConnectionEditPart {

	/** width of an unselected arrow. */
	private static final int SCALEARROW_X = 18;
	/** width of an unselected arrow. */
	private static final int SCALEARROW_Y = 6;

	/**
	 * this function is used to configure the connection Figure.
	 * It is possible to add an extremity figure or a label.
	 * @return IFigure : the figure
	 */
	protected final IFigure createFigure() {
		//add an arrow
        PolylineConnection connection = new PolylineConnection();
        PolygonDecoration arrow = new PolygonDecoration();
        arrow.setTemplate(PolygonDecoration.TRIANGLE_TIP);
        arrow.setScale(SCALEARROW_X, SCALEARROW_Y);
        connection.setTargetDecoration(arrow);

        //connection.setConnectionRouter(new ManhattanConnectionRouter());
        connection.setConnectionRouter(new ManhattanConnectionRouter());
        return connection;
    }

	/**
	 * create needed policies.
	 */
	@Override
	protected final void createEditPolicies() {
		installEditPolicy(EditPolicy.SELECTION_FEEDBACK_ROLE,
			            new ConnectionSelectionEditPolicy());
		//ajout de l'edit police permettant de supprimer l'element selectionne
		installEditPolicy(EditPolicy.COMPONENT_ROLE,
				new ConnectionRemoveEditPolicy());
	}
}
