package com.bull.cimero.pluginEditor.editors.parts;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.List;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.FreeformLayer;
import org.eclipse.draw2d.FreeformLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.swt.widgets.Menu;

import com.bull.cimero.pluginEditor.editors.model.Diagram;
import com.bull.cimero.pluginEditor.editors.policies.DiagramLayoutEditPolicy;
import com.bull.cimero.pluginEditor.popupMenu.ContextPopupMenu;

/**
 * class representing the diagram part.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class DiagramPart extends AbstractGraphicalEditPart
				implements PropertyChangeListener {

	/**
	 * the constructor.
	 */
	public DiagramPart() {
		super();
	}

	/**
	 * method to get model children.
	 * @return List : list of cimero objects
	 */
    @Override
	protected final List getModelChildren() {
      return ((Diagram) this.getModel()).getLstCimeroObject();
    }

	/**
	 * method called when activating the part.
	 * add of a listener
	 * @see properties change
	 */
    @Override
    public final void activate() {
      if (isActive()) {
         return;
      }
      super.activate();
      ((Diagram) getModel()).addPropertyChangeListener(this);
    }

    /**
	 * method called when desactivating the part.
	 * remove the listener
	 */
    @Override
    public final void deactivate() {
      if (!isActive()) {
         return;
      }
      super.deactivate();
      ((Diagram) getModel()).removePropertyChangeListener(this);
   }

    /**
	 * method to create a figure.
	 * @return IFigure : the created figure
	 */
    @Override
   protected final IFigure createFigure() {
      Figure figure = new FreeformLayer();
      figure.setLayoutManager(new FreeformLayout());
      //add contextuel menu
      hookContextMenu();
      return figure;
   }

    /**
	 * method to create policies.
	 */
    @Override
   protected final void createEditPolicies() {
      //install a custom layout policy that handles dragging nodes around
      installEditPolicy(EditPolicy.LAYOUT_ROLE,
    		  new DiagramLayoutEditPolicy());
   }

    /**
	 * method to catch properties change.
	 * @param event : event occured
	 */
   public final void propertyChange(final PropertyChangeEvent event) {
      String prop = event.getPropertyName();
      if (Diagram.NODES.equals(prop)) {
         refreshChildren();
      }
   }

   /**
	 * method to perform request.
	 * @param request : request received
	 */
   @Override
  public final void performRequest(final Request request) {
	   System.out.println("PerformRequest sur le diagramPart");
   }

   /**
	 * create a new context menu.
	 *
	 */
	private void hookContextMenu() {
		MenuManager menuMgr = new ContextPopupMenu("#PopupMenu",
				((Diagram) getModel()).getCimeroDiagramName());
		Menu menu = menuMgr.createContextMenu(getViewer().getControl());
		getViewer().getControl().setMenu(menu);
	}
}
