package com.bull.cimero.pluginEditor.editors.parts;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartFactory;

import com.bull.cimero.pluginEditor.editors.DiagramEditor;
import com.bull.cimero.pluginEditor.editors.model.Connection;
import com.bull.cimero.pluginEditor.editors.model.Diagram;
import com.bull.cimero.pluginEditor.editors.model.SEModel.WireTap;

/**
 * an EditPartFactory knows how to make an EditPart from a model object
 * in the context of a parent EditPart.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class PartFactory implements EditPartFactory {

	/** this variable containt the reference of the actual diagram. */
	private Diagram currentDiagram;
	/** reference to the diagramEditor. */
	private DiagramEditor diagramEditor;

	/**
	 * default constructor.
	 * @param diagram : the current diagram
	 * @param diagramEditorParam : current diagrameditor
	 */
	public PartFactory(
			final Diagram diagram, final DiagramEditor diagramEditorParam) {
		this.currentDiagram = diagram;
		this.diagramEditor = diagramEditorParam;
	}

	/**
	 * method to create new edit part.
	 * @param context : the context
	 * @param model : the object to add
	 * @return EditPart
	 */
	public final EditPart createEditPart(
			final EditPart context, final Object model) {
		EditPart part = null;
		boolean bShow;
		//get the bShow variable on the diagramEditor
		bShow = diagramEditor.isBShowErrors();

		if (model instanceof Diagram) {
			//create the diagram part
			part = (DiagramPart) new DiagramPart();
		} else  if (model instanceof Connection) {
			//create the connection part
			part = new ConnectionPart();
		} else if (model instanceof WireTap) {
			part = new WireTapPart(currentDiagram, bShow);
		} else {
			//create the component part.
			part = new GeneriqueCimeroPart(currentDiagram, bShow,
								model.getClass().getName());
		}
		// tell the newly created part about the model object
		part.setModel(model);
		return part;
	}
}
