package com.bull.cimero.pluginEditor.editors.partsOutline;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.List;
import org.eclipse.gef.editparts.AbstractTreeEditPart;
import com.bull.cimero.pluginEditor.editors.model.Diagram;

/**
 * class used to make communication for diagram model and outline view.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class DiagramTreeEditPart extends AbstractTreeEditPart
								implements PropertyChangeListener {

	/**
	 * DiagramTreeEditPart contructor.
	 * @param model : the model to draw
	 */
	public DiagramTreeEditPart(final Object model) {
		super(model);
	}

	/**
	 * activate function.
	 */
	public final void activate() {
		super.activate();
		((Diagram) getModel()).addPropertyChangeListener(this);
	}

	/**
	 * desactivate function.
	 */
	public final void deactivate() {
		((Diagram) getModel()).removePropertyChangeListener(this);
		super.deactivate();
	}

	/**
	 * return all the children that compose the model.
	 * @return the list that contains the children.
	 */
	protected final List getModelChildren() {
		return ((Diagram) getModel()).getLstCimeroObject();
	}

	/**
	 * This function is called when there is an event on the diagram.
	 * @param change : the change event.
	 */
	public final void propertyChange(final PropertyChangeEvent change) {
		refreshChildren();
	}

}
