package com.bull.cimero.pluginEditor.generator.serviceMixGenerator
	.factory.lWFactory.BC;

import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.BCModel.Mail;
import com.bull.cimero.pluginEditor.generator
.serviceMixGenerator.model.global.ActivationSpecClass;
import com.bull.cimero.pluginEditor.generator
.serviceMixGenerator.model.global.LWBeanClass;
import com.bull.cimero.pluginEditor.generator
.serviceMixGenerator.model.global.ComponentClass;
import com.bull.cimero.pluginEditor.generator
.serviceMixGenerator.model.global.ConstructorClass;
import com.bull.cimero.pluginEditor.generator
.serviceMixGenerator.model.global.NameSpaceClass;
import com.bull.cimero.pluginEditor.generator
.serviceMixGenerator.model.global.PropertyClass;

/**
 * This class will be use to add in the ServiceMix model, an activation
 * spec that contain a mail binding component.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class MailFactory extends BCLWFactory {

	/**the xmlns of the mail component.*/
	private static final String XMLNS = "http://xbean.org/schemas/spring/1.0";
	/**the class using by ServiceMix to the Email poller.*/
	public static final String CLASSIN =
		"org.apache.servicemix.components.email.MimeMailPoller";
	/**the class using by serviceMix to the Email sender.*/
	public static final String CLASSOUT =
		"org.apache.servicemix.components.email.MimeMailSender";
	/**The hostname property.*/
	private static final String HOSTNAME = "hostName";
	/**The password property.*/
	private static final String PASSWORD = "password";
	/**The username property.*/
	private static final String USERNAME = "userName";
	/**The name of the sender property.*/
	private static final String SENDER = "sender";
	/**The name of the class using to the MimeMailSender.*/
	private static final String JAVAMAILSENDER =
		"org.springframework.mail.javamail.JavaMailSenderImpl";
	/**The name of the host property (JAVAMAILSENDER property).*/
	private static final String HOST = "host";
	/**The name of the port property (JAVAMAILSENDER property).*/
	private static final String PORT = "port";
	/**The name of the mail marshaler.*/
	private static final String MARSHALERNAME = "marshaler";
	/**The name of the class using to the mail marshaler.*/
	private static final String MIMEMAILMARSHALER =
		"org.apache.servicemix.components.email.MimeMailMarshaler";
	/**The class using to the constant expression.*/
	private static final String CONSTANTEXPRESSION =
		"org.apache.servicemix.expression.ConstantExpression";

	/**
	 * constructor with the mep mode in parameter.
	 * @param mepMode : the mep mode of this component. True if INMEP
	 * @param ref : the reference of the current namespace.
	 */
	public MailFactory(final NameSpaceClass ref, final Boolean mepMode) {
		super(ref, mepMode);
	}

	/**
	 * This function create the mail component.
	 * @param elt : the element model that will be used to generate
	 * 		this activation spec.
	 * @param force : false if the validation must to be do.
	 * @return the activation spec created
	 * @throws Exception : if the is a error during generation
	 */
	public final ActivationSpecClass createXMLModel(
			final GeneriqueCimeroModel elt,
			final Boolean force) throws Exception {
		ActivationSpecClass myActi = null;
		//if not null and valide and instance of mail model
		if ((elt != null) && (elt instanceof Mail)) {
			if ((elt.isValide(false)) || (force)) {
				Mail monMail = (Mail) elt;
				//create the activationSpec
				myActi = new ActivationSpecClass();
				myActi.setNameSpaceRef(getNamespaceRef());
				//set the name
				myActi.setComponentName(monMail.getName());
				//set the service
				myActi.setServiceName(monMail.getName(),
						new NameSpaceClass(NAMESPACENAME , NAMESPACEVALUE));
				//we test if this element is in in mep mode or out mep mode.
				if (isInMepMode()) {
					//it is an INMEP => EMAIL POLLER
					//set the destination service
					setDestinationService(myActi , monMail);

				}
				//create the components
				ComponentClass compo = createComponent(monMail);
				myActi.setComponent(compo);
				return myActi;
			}
		}
		throw new Exception("The generique cimero object is not valide");

	}

	/**
	 * This function creates the component of the servicemix mail element.
	 * @param monMail : the cimero model element
	 * @return the instance of the component class.
	 */
	private ComponentClass createComponent(final Mail monMail) {
		//create the componentclass instance
		ComponentClass monCompo = new ComponentClass(getNamespaceRef());
		//add the bean class.
		LWBeanClass monB = null;
		if (monMail.isInput()) {
			monB = createBeanRouteurOUTMEP(monMail);
		} else {
			monB = createBeanRouteurINMEP(monMail);
		}
		monCompo.setMonBean(monB);
		return monCompo;
	}

	/**
	 * This function creates the bean using by the mail in INMEP in the
	 * serviceMix model.
	 * @param monMail : the mail of the cimero model.
	 * @return the bean routeur.
	 */
	private LWBeanClass createBeanRouteurINMEP(final Mail monMail) {
//		create the bean
		LWBeanClass beanMail = new LWBeanClass(CLASSIN);
		beanMail.setXmlnsValue(XMLNS);
		//add the property hostname property to this bean
		beanMail.addProperty(
				createProperty(HOSTNAME , monMail.getHostNameAdress()) , "");
		//add the password propertu
		beanMail.addProperty(
				createProperty(PASSWORD , monMail.getPassword()) , "");
		//add the username property
		beanMail.addProperty(
				createProperty(USERNAME , monMail.getLogin()) , "");
		return beanMail;
	}

	/**
	 * This function creates the bean using by the mail in OUTMEP in the
	 * serviceMix model.
	 * @param monMail : the cimero element
	 * @return the bean routeur.
	 */
	private LWBeanClass createBeanRouteurOUTMEP(final Mail monMail) {
		LWBeanClass beanRout = new LWBeanClass(CLASSOUT);
		//add the first property which is a another bean.
		beanRout.addProperty(createPropertyMailSender(monMail) , "");
		//add the second property which is another bean.
		beanRout.addProperty(createPropertyMailMarshaler(monMail) , "");
		return beanRout;
	}

	/**
	 * This function creates the property that the name is marshaler.
	 * @param monMail : the cimero element.
	 * @return the property created.
	 */
	private PropertyClass createPropertyMailMarshaler(final Mail monMail) {
		//create the property
		PropertyClass myProp = new PropertyClass(SENDER);
		//create the MIMEMAILSENDER bean to this property.
		LWBeanClass myBean = new LWBeanClass(JAVAMAILSENDER);
		//add the host property
		myBean.addProperty(
				createProperty(HOST , monMail.getHostNameAdress()) , "");
		//add the port property.
		myBean.addProperty(
				createProperty(PORT ,
						String.valueOf(monMail.getCommunicationPort())) , "");
		//add the mimemail bean to the property.
		myProp.setBean(myBean);
		return myProp;
	}

	/**
	 * This function creates the property that the name is sender.
	 * @param monMail : the cimero element.
	 * @return the property created.
	 */
	private PropertyClass createPropertyMailSender(final Mail monMail) {
		//create the property
		PropertyClass myProp = new PropertyClass(MARSHALERNAME);
		//create the bean
		LWBeanClass myBean  = new LWBeanClass(MIMEMAILMARSHALER);
		//create the from property
		PropertyClass fromProperty = new PropertyClass("from");
		LWBeanClass fromBean = new LWBeanClass(CONSTANTEXPRESSION);
		fromBean.setConstructor(new ConstructorClass(monMail.getMailFrom()));
		fromProperty.setBean(fromBean);
		myBean.addProperty(fromProperty , "");

		//create the to property
		PropertyClass toProperty = new PropertyClass("to");
		LWBeanClass toBean = new LWBeanClass(CONSTANTEXPRESSION);
		toBean.setConstructor(new ConstructorClass(monMail.getMailTo()));
		toProperty.setBean(toBean);
		myBean.addProperty(toProperty , "");

		//add the subject property
		if (monMail.getMailSubject().compareTo("") != 0) {
			PropertyClass subjectProperty = new PropertyClass("subject");
			LWBeanClass subjectBean = new LWBeanClass(CONSTANTEXPRESSION);
			subjectBean.setConstructor(
					new ConstructorClass(monMail.getMailSubject()));
			subjectProperty.setBean(subjectBean);
			myBean.addProperty(subjectProperty , "");
		}

		//add the attachement property
		if (monMail.getAttachements().compareTo("") != 0) {
			PropertyClass attachProperty = new PropertyClass("attachments");
			LWBeanClass attachBean = new LWBeanClass(CONSTANTEXPRESSION);
			attachBean.setConstructor(
					new ConstructorClass(monMail.getAttachements()));
			attachProperty.setBean(attachBean);
			myBean.addProperty(attachProperty , "");
		}

		if (monMail.getMailBody().compareTo("") != 0) {
			PropertyClass bodyProperty = new PropertyClass("text");
			LWBeanClass bodyBean = new LWBeanClass(CONSTANTEXPRESSION);
			bodyBean.setConstructor(
					new ConstructorClass(monMail.getMailBody()));
			bodyProperty.setBean(bodyBean);
			myBean.addProperty(bodyProperty , "");
		}

		//add the mimemail bean to the property.
		myProp.setBean(myBean);
		return myProp;
	}

}
