package com.bull.cimero.pluginEditor.generator.serviceMixGenerator
	.factory.lWFactory.BC;

import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.BCModel.ScreenOutput;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ActivationSpecClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ComponentClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.LWBeanClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.NameSpaceClass;

/**
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class ScreenOutputFactory extends BCLWFactory {

	/**The class using by the screen output in ServiceMix.*/
	public static final String CLASS =
		"org.apache.servicemix.components.util.StreamWriterComponent";
	/**The xmlns value.*/
	private static final String XMLNSVALUE =
		"http://xbean.org/schemas/spring/1.0";

	/**constructor with the mep mode in parameter.
	 * @param ref : the reference of the current namespace.
	 * @param mepMode : the mep component mode.
	 * */
	public ScreenOutputFactory(
			final NameSpaceClass ref, final Boolean mepMode) {
		super(ref, mepMode);
	}

	/**
	 * This function create the screen output servicemix model.
	 * @param elt : the element model that will be used to generate
	 * 		this activation spec.
	 * @param force : false if the validation must to be do.
	 * @return the activation spec created
	 * @throws Exception : if the is a error during generation
	 */
	public final ActivationSpecClass createXMLModel(
			final GeneriqueCimeroModel elt,
			final Boolean force)
			throws Exception {
		ActivationSpecClass myActi = null;
		//if not null and valide and instance of mail model
		if ((elt != null) && (elt instanceof ScreenOutput)) {
			if ((elt.isValide(false)) || (force)) {
				ScreenOutput myScreen = (ScreenOutput) elt;
				//create the activationSpec
				myActi = new ActivationSpecClass();
				myActi.setNameSpaceRef(getNamespaceRef());
				//set the name
				myActi.setComponentName(myScreen.getName());
				//set the service
				myActi.setServiceName(myScreen.getName(),
						new NameSpaceClass(NAMESPACENAME , NAMESPACEVALUE));
				//create the components
				ComponentClass compo = createComponent(myScreen);
				myActi.setComponent(compo);
				return myActi;
			}
		}
		throw new Exception("The generique cimero object is not valide");
	}

	/**
	 * This function creates the component of the servicemix screen element.
	 * @param myScreen : the cimero model element
	 * @return the instance of the component class.
	 */
	private ComponentClass createComponent(final ScreenOutput myScreen) {
		//create the componentclass instance
		ComponentClass monCompo = new ComponentClass(getNamespaceRef());
		//add the bean class.
		LWBeanClass monB = null;
		monB = createBeanScreenOutput(myScreen);
		monCompo.setMonBean(monB);
		return monCompo;
	}

	/**
	 * This function creates screen bean in the
	 * serviceMix model.
	 * @param myScreen : the cimero element
	 * @return the bean screenoutput.
	 */
	private LWBeanClass createBeanScreenOutput(
			final ScreenOutput myScreen) {
		LWBeanClass myBean = new LWBeanClass(CLASS);
		//add the xmlns
		myBean.setXmlnsValue(XMLNSVALUE);
		return myBean;
	}

}
