package com.bull.cimero.pluginEditor.generator.serviceMixGenerator
	.factory.lWFactory.BC;

import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.BCModel.Timer;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ActivationSpecClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ComponentClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.LWBeanClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.NameSpaceClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.PropertyClass;

/**
 * The timer servicemix factory.
 * @author Cedric Mouilleron
 * @authoe Pierre Notel
 *@author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class TimerFactory extends BCLWFactory {

	//-----------------------------
	//CONSTANTES
	/**The classusing by servicemix to implement this component.*/
	private static final String CLASSTIMERSM =
		"org.apache.servicemix.components.quartz.QuartzComponent";
	/**XMLNS value.*/
	private static final String XMLNS =
		"http://xbean.org/schemas/spring/1.0";
	/**The triggers property.*/
	private static final String TRIGGERS = "triggers";
	/**The trigger class in servicemix.*/
	private static final String TRIGGERSCLASS = "org.quartz.SimpleTrigger";
	/**The repeat interval property name.*/
	private static final String REPEATINTERVAL = "repeatInterval";
	/**The repeat count property name.*/
	private static final String REPEATCOUNT = "repeatCount";
	/**The job details class.*/
	private static final String JOBDETAILCLASS = "org.quartz.JobDetail";
	/**The job detail name property.*/
	private static final String JOBDETAILNAME = "name";
	/**The job detail group property.*/
	private static final String JOBDETAILGROUP = "group";
	/**The job detail description.*/
	private static final String JOBDETAILDESCR = "description";


	/**
	 * default constructor.
	 * @param nms : the namespace using by this factory.
	 * @param mepMode : the mep mode.
	 */
	public TimerFactory(final NameSpaceClass nms, final Boolean mepMode) {
		super(nms, mepMode);
	}

	/**
	 * This function create the timer servicemix model.
	 * @param elt : the element model that will be used to generate
	 * 		this activation spec.
	 * @param isForce : false if the validation must to be do.
	 * @return the activation spec created
	 * @throws Exception : if the is a error during generation
	 */
	public final ActivationSpecClass createXMLModel(
			final GeneriqueCimeroModel elt,
			final Boolean isForce) throws Exception {
		ActivationSpecClass myActi = null;
		//if not null and valide and instance of mail model
		if ((elt != null) && (elt instanceof Timer)) {
			if ((elt.isValide(false)) || (isForce)) {
				Timer myTimer = (Timer) elt;
				//create the activationSpec
				myActi = new ActivationSpecClass();
				myActi.setNameSpaceRef(getNamespaceRef());
				//set the name
				myActi.setComponentName(myTimer.getName());
				//set the service
				myActi.setServiceName(myTimer.getName(),
						new NameSpaceClass(NAMESPACENAME , NAMESPACEVALUE));
				setDestinationService(myActi , myTimer);
				//create the components
				ComponentClass compo = createComponent(myTimer);
				myActi.setComponent(compo);
				return myActi;
			}
		}
		throw new Exception("The generique cimero object is not valide");
	}

	/**
	 * This function creates the component of the servicemix timer element.
	 * @param myTimer : the cimero model element
	 * @return the instance of the component class.
	 */
	private ComponentClass createComponent(final Timer myTimer) {
		//create the componentclass instance
		ComponentClass monCompo = new ComponentClass(getNamespaceRef());
		//add the bean class.
		LWBeanClass monB = null;
		monB = createBeanTimer(myTimer);
		monCompo.setMonBean(monB);
		return monCompo;
	}

	/**
	 * This function creates timer bean in the
	 * serviceMix model.
	 * @param myTimer : the cimero element
	 * @return the bean timer.
	 */
	private LWBeanClass createBeanTimer(final Timer myTimer) {
		LWBeanClass myB = new LWBeanClass(CLASSTIMERSM);
		myB.setXmlnsValue(XMLNS);
		//create trigger property
		PropertyClass trigger = new PropertyClass(TRIGGERS);
		LWBeanClass beanKey = createBeanKey(myTimer);
		LWBeanClass beanValue = createBeanValue(myTimer);
		trigger.addEntryMap(beanKey, beanValue);
		//add trigger property
		myB.addProperty(trigger, "");
		return myB;
	}

	/**
	 * create the bean used to key in map.
	 * @param myTimer : the cimero timer component.
	 * @return LWBeanClass
	 */
	private LWBeanClass createBeanKey(final Timer myTimer) {
		LWBeanClass beanKey = new LWBeanClass(TRIGGERSCLASS);
		beanKey.addProperty(
			createProperty(REPEATINTERVAL,
					String.valueOf(myTimer.getRepeatInterval())), "");
		beanKey.addProperty(
			createProperty(REPEATCOUNT,
					String.valueOf(myTimer.getRepeatCount())), "");
		return beanKey;
	}

	/**
	 * create the bean used to value in map.
	 * @param myTimer : the cimero timer component.
	 * @return LWBeanClass
	 */
	private LWBeanClass createBeanValue(final Timer myTimer) {
		LWBeanClass beanValue = new LWBeanClass(JOBDETAILCLASS);
		beanValue.addProperty(
				createProperty(JOBDETAILNAME, myTimer.getMessageName()),
				"");
		beanValue.addProperty(
				createProperty(JOBDETAILGROUP, myTimer.getMessageGROUP()),
				"");
		beanValue.addProperty(
				createProperty(JOBDETAILDESCR,
						myTimer.getMessageDescription()), "");
		return beanValue;
	}

}
