package com.bull.cimero.pluginEditor.generator.serviceMixGenerator
	.factory.lWFactory;

import com.bull.cimero.pluginEditor.editors.model.Connection;
import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.SEModel.Routeur;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.GenericServiceMixFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.lWFactory.SE.RouteurFactory;
import com.bull.cimero.pluginEditor.generator
			.serviceMixGenerator.model.global.ActivationSpecClass;
import com.bull.cimero.pluginEditor.generator
			.serviceMixGenerator.model.global.ConstructorClass;
import com.bull.cimero.pluginEditor.generator
			.serviceMixGenerator.model.global.LWBeanClass;
import com.bull.cimero.pluginEditor.generator
			.serviceMixGenerator.model.global.NameSpaceClass;
import com.bull.cimero.pluginEditor.generator
			.serviceMixGenerator.model.global.PropertyClass;

/**
 * This class contains aall the LW components generation functions.
 * This class must to be implement by all the factory.
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public abstract class GeneriqueLWfactory implements GenericServiceMixFactory {

	/**The name of the namespace using by the mail.*/
	public static final String NAMESPACENAME = "foo";
	/**The value of this namespace.*/
	public static final String NAMESPACEVALUE = "http://servicemix.org/cheese";


	/**
	 * This function creates a servicemix property bean.
	 * @param propertyName : the name of the property.
	 * @param propertyValue : the value of this property.
	 * @return the property
	 */
	protected final PropertyClass createProperty(final String propertyName,
			final String propertyValue) {
		PropertyClass myProp = new PropertyClass(propertyName);
		myProp.addValue(propertyValue);
		return myProp;
	}
	
	protected final PropertyClass createPropertyRef(final String propertyName,
			final String propertyRef) {
		PropertyClass myProp = new PropertyClass(propertyName);
		myProp.addRef(propertyRef);
		return myProp;
	}

	/**
	 * This function creates a servicemix property bean.
	 * @param propName : the name of the property
	 * @param lstValueProp : the list of value to this property.
	 * @return the property
	 */
	protected final PropertyClass createProperty(final String propName,
			final String [] lstValueProp) {
		PropertyClass myProp = new PropertyClass(propName);
		for (int i = 0; i < lstValueProp.length; i++) {
			myProp.addValue(lstValueProp[i]);
		}
		return myProp;
	}

	/**
	 * This function create a property that contain a bean constructor.
	 * @param propertyName : the name of the property
	 * @param beanClassDate : the class using by the bean
	 * @param constructorValue : the value of the constructor
	 * @return the property
	 */
	protected final PropertyClass createBeanConstructor(final String
			propertyName, final String beanClassDate, final String
			constructorValue) {
		PropertyClass myProp = new PropertyClass(propertyName);
		LWBeanClass myBean = new LWBeanClass(beanClassDate);
		myBean.setConstructor(new ConstructorClass(constructorValue));
		myProp.setBean(myBean);
		return myProp;
	}

	/**
	 * This function is used to set at the activation spec, the destination
	 * service.
	 * @param myActi : the current activation spec
	 * @param elt : the element of the cimero model.
	 * @throws Exception if the destination service is not find.
	 */
	protected final void setDestinationService(final ActivationSpecClass myActi,
			final GeneriqueCimeroModel elt) throws Exception {
		//we get the destination element
		GeneriqueCimeroModel destElt = getDestination(elt);
		//the destination service has been finded.
		if (destElt != null) {
			//if this is a routeur, the using namespace is not the same
			//as the others
			NameSpaceClass usingNamespace = getNamespace(destElt);

			//we set the destination service of this activation spec
			myActi.setDestinationService(destElt.getName() , usingNamespace);

		} else {
			/*
			throw new Exception("the destination service of the mail component"
					+ "cannot be find !");
			*/
		}
	}

	/**
	 * return the destination element of the element in parameter. If
	 * there is no destination element, this function return false.
	 * @param elt : the reference object.
	 * @return the destination object of the object in parameter
	 */
	public final GeneriqueCimeroModel getDestination(
			final GeneriqueCimeroModel elt) {
		GeneriqueCimeroModel destElt = null;
		if (elt.getOutgoingConnections().size() != 0) {
			Connection modelTmp =
				(Connection) elt.getOutgoingConnections().get(0);
			if (modelTmp != null) {
				destElt = modelTmp.getTarget();
			}
		}
		return destElt;
	}

	/**
	 * return the namespace using by the object in parameter.
	 * @param elt : the object reference.
	 * @return the namespace using by this object
	 */
	public final NameSpaceClass getNamespace(
			final GeneriqueCimeroModel elt) {
		NameSpaceClass usingNamespace = null;
		/*if (elt instanceof Routeur) {
			usingNamespace = new NameSpaceClass(
					RouteurFactory.NAMESPACENAME ,
					RouteurFactory.NAMESPACEVALUE);
		} else {
		*/
			usingNamespace = new NameSpaceClass(
				GeneriqueLWfactory.NAMESPACENAME,
				GeneriqueLWfactory.NAMESPACEVALUE);
		/*}*/
		return usingNamespace;
	}

	/**
	 * @return the namespace using by this factory.
	 */
	public final NameSpaceClass getNamespaceComponent() {
		return new NameSpaceClass(NAMESPACENAME, NAMESPACEVALUE);
	}
}
