package com.bull.cimero.pluginEditor.generator.serviceMixGenerator
	.factory.standardFactory.BC;

import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.BCModel.JMS;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.lWFactory.GeneriqueLWfactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ActivationSpecClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ComponentClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.NameSpaceClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.StandardComponentClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.StandardJBIClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.StandardJMSClass;

/**
 * The JMS factory.
 * @author Cedric Mouilleron
 * @author Pierre Notel
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class JMSFactory extends BCStdFactory {

//	------------------------------------
	//CONSTANTE
	/**default in-out mep mode url.*/
	private static final String DEFINOUTMEP =
		"http://www.w3.org/2004/08/wsdl/in-only";
	/**the consumer role name.*/
	private static final String CONSUMER = "consumer";
	/**The provider role name.*/
	private static final String PROVIDER = "provider";
	/**The http namespace name.*/
	public static final String NAMESPACENAME = "jms";
	/**The http namespace value.*/
	public static final String NAMESPACEVALUE =
		"http://servicemix.apache.org/jms/1.0";

	//----------------------------------------
	// CONSTRUCTOR
	/**
	 * JMSFactory constructor.
	 * @param nms : the namespace using by this component.
	 * @param mepMode : true if the element has an in mep mode.
	 */
	public JMSFactory(final NameSpaceClass nms, final Boolean mepMode) {
		super(nms, mepMode);
	}

	/**
	 * @return the namespace name.
	 */
	public final String getNamespaceName() {
		return NAMESPACENAME;
	}

	/**
	 *@return the namespace value.
	 */
	public final String getNamespaceValue() {
		return NAMESPACEVALUE;
	}

	/**
	 * @see com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.standardFactory.GeneriqueStandardFactory#createXMLJBIElement(
	 * com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel)
	 * @param elt : the cimero jms object.
	 * @param isForce : true is the function not do th validation.
	 * @return the ActivationSpecClass element.
	 * @throws Exception : if there is a problem during the model generation
	 */
	public final ActivationSpecClass createXMLModel(
			final GeneriqueCimeroModel elt,
			final Boolean isForce) throws Exception {
		if (getNamespaceRef() == null) {
			throw new Exception("namespaceclass cannot be null");
		}
		ActivationSpecClass myActi = null;
		//if not null and valide and instance of mail model
		if ((elt != null) && (elt instanceof JMS)) {
			if ((elt.isValide(false)) || (isForce)) {
				JMS monJMS = (JMS) elt;
				//create the activationSpec
				myActi = new ActivationSpecClass();
				myActi.setNameSpaceRef(getNamespaceRef());
				myActi.setComponentName(monJMS.getName());
				myActi.setServiceName(monJMS.getName(),
					new NameSpaceClass(GeneriqueLWfactory.NAMESPACENAME ,
							GeneriqueLWfactory.NAMESPACEVALUE));
				//we test if this element is in in mep mode or out mep mode.
				if (isInMepMode()) {
					//set the destination service
					setDestinationService(myActi , monJMS);
				}
				//create the components
				ComponentClass compo = createComponent(monJMS);
				myActi.setComponent(compo);
				return myActi;
			}
		 } else {
			throw new Exception("The generique cimero object is not valide");
		}
		return null;
	}

	/**
	 * creates the servicemix WS component.
	 * @param monJMS : the cimero model element
	 * @return the instance of the component class.
	 * @throws Exception : if the generation return an error
	 */
	private ComponentClass createComponent(final JMS monJMS)
		throws Exception {
		//create the componentclass instance
		ComponentClass monCompo = new ComponentClass(getNamespaceRef());
		//add the bean class.
		StandardComponentClass monS = null;
		monS = createStandardComponentMEP(monJMS);
		monCompo.setMonComponentStandard(monS);
		return monCompo;
	}

	/**
	 * creates the standard components.
	 * @param monJMS : the cimero WS
	 * @return the standard component class.
	 * @throws Exception : if the generation return an error.
	 */
	private StandardComponentClass createStandardComponentMEP(
			final JMS monJMS) throws Exception {
		StandardComponentClass monStd = new StandardComponentClass(
				new NameSpaceClass(getNamespaceName() , getNamespaceValue()));
		monStd.setMyJBICompo(createJBIElement(monJMS));
		return monStd;
	}

	/**
	 * @see com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.standardFactory.GeneriqueStandardFactory#createJBIElement(
	 * com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel)
	 * @param elt : the cimero jms object.
	 * @return the standard jms element.
	 * @throws Exception : if the generation return an error
	 */
	public final StandardJBIClass createJBIElement(
			final GeneriqueCimeroModel elt)	throws Exception {
		StandardJMSClass myJmsSM = null;
		if (elt instanceof JMS) {
			JMS myJMS = (JMS) elt;
			NameSpaceClass nmsMyJMS = getNamespace(myJMS);
			if (isInMepMode()) {
				//get the destination service.
				GeneriqueCimeroModel destElt = getDestination(elt);
				if (destElt != null) {
					NameSpaceClass destinationNms = getNamespace(destElt);
					myJmsSM = new StandardJMSClass(getNamespaceComponent(),
							myJMS.getName(),
							nmsMyJMS,
							myJMS.getName(),
							destElt.getName(),
							destinationNms,
							CONSUMER,
							DEFINOUTMEP,
							myJMS.getStyle(),
							myJMS.getProviderDestinationName(),
							myJMS.getJndiConnectionFactoryName());
				} else {
					throw new Exception("The destination component of the "
							+ myJMS.getName() + " object doesn't exist");
				}
			} else {
				myJmsSM = new StandardJMSClass(getNamespaceComponent(),
						myJMS.getName(),
						nmsMyJMS,
						myJMS.getName(),
						null,
						null,
						PROVIDER,
						DEFINOUTMEP,
						myJMS.getStyle(),
						myJMS.getProviderDestinationName(),
						myJMS.getJndiConnectionFactoryName());
			}
		}
		return myJmsSM;
	}
}
