package com.bull.cimero.pluginEditor.generator.serviceMixGenerator
	.factory.standardFactory.BC;

import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.BCModel.JdbcBC;
import com.bull.cimero.pluginEditor.editors.model.BCModel.ServiceWeb;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.lWFactory.GeneriqueLWfactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ActivationSpecClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ComponentClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.NameSpaceClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.StandardComponentClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.StandardHttpClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.StandardJBIClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.StandardJDBCClass;

/**
 * f
 */
public class JdbcBCFactory extends BCStdFactory {

	//------------------------------------
	//CONSTANTE
	/**default in-out mep mode url.*/
	private static final String DEFINOUTMEP =
		"http://www.w3.org/2004/08/wsdl/in-out";
	/**default in-only mep mode url.*/
	private static final String DEFINMEP =
		"http://www.w3.org/2004/08/wsdl/in-only";
	
	private static final String DEFOUTMEP = 
		"http://www.w3.org/2004/08/wsdl/out-only";
	
	/**the consumer role name.*/
	private static final String CONSUMER = "consumer";
	/**The provider role name.*/
	private static final String PROVIDER = "provider";

	//-------------------------------------
	//CONSTANTES
	/**The http namespace name.*/
	public static final String NAMESPACENAME = "jdbc";
	/**The http namespace value.*/
	public static final String NAMESPACEVALUE =
		"http://servicemix.apache.org/jdbc/1.0";


	//----------------------------------------------
	//CONSTRUCTOR
	/**
	 * ServiceWebFactory constructor.
	 * @param ref : the namespace using by this component.
	 * @param mepMode : true if the element has an in mep mode.
	 */
	public JdbcBCFactory(final NameSpaceClass ref) {
		super(ref, true);
	}

	/**
	 * @see com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.standardFactory.GeneriqueStandardFactory#createJBIElement(
	 * com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel)
	 * @param elt : the cimero http object.
	 * @return the standard http element.
	 * @throws Exception : if the generation return an error
	 */
	public final StandardJBIClass createJBIElement(
			final GeneriqueCimeroModel elt) throws Exception {
		
		if (elt instanceof JdbcBC) {
			JdbcBC monJdbcBC = (JdbcBC) elt;
			
			
			GeneriqueCimeroModel destElt = getDestination(elt);
			if (destElt != null){
				NameSpaceClass destinationNms = getNamespace(destElt);
				StandardJBIClass myJDBC = new StandardJDBCClass(
							destElt.getName()
							, destinationNms
							, new NameSpaceClass(getNamespaceName() ,
												getNamespaceValue())
							, monJdbcBC.getName()
							, monJdbcBC);
				return myJDBC;			
			}else {
				throw new Exception("The destination component of the "
						+ monJdbcBC.getName() + " object doesn't exist");
			}
		}
		return null;
	}

	/**
	 * @see com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.standardFactory.GeneriqueStandardFactory#createXMLJBIElement(
	 * com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel)
	 * @param elt : the cimero http object.
	 * @param isForce : true is the function not do th validation.
	 * @return the ActivationSpecClass element.
	 * @throws Exception : if there is a problem during the model generation
	 */
	public final ActivationSpecClass createXMLModel(
			final GeneriqueCimeroModel elt, final Boolean isForce)
			throws Exception {
		if (getNamespaceRef() == null) {
			throw new Exception("namespaceclass cannot be null");
		}
		ActivationSpecClass myActi = null;
		//if not null and valide and instance of mail model
		if ((elt != null) && (elt instanceof JdbcBC)) {
			if ((elt.isValide(false)) || (isForce)) {
				JdbcBC monJdbcBC = (JdbcBC) elt;
				//create the activationSpec
				myActi = new ActivationSpecClass();
				myActi.setNameSpaceRef(getNamespaceRef());
				myActi.setComponentName(monJdbcBC.getName());
				myActi.setServiceName(monJdbcBC.getName(),
					new NameSpaceClass(GeneriqueLWfactory.NAMESPACENAME ,
							GeneriqueLWfactory.NAMESPACEVALUE));
				//we test if this element is in in mep mode or out mep mode.
				if (isInMepMode()) {
					//set the destination service
					setDestinationService(myActi , monJdbcBC);
				}
				//create the components
				ComponentClass compo = createComponent(monJdbcBC);
				myActi.setComponent(compo);
				return myActi;
			}
		 } else {
			throw new Exception("The generique cimero object is not valide");
		}
		return null;
	}

	/**
	 * creates the servicemix WS component.
	 * @param monWS : the cimero model element
	 * @return the instance of the component class.
	 * @throws Exception : if the generation return an error
	 */
	private ComponentClass createComponent(final JdbcBC monJdbcBC)
		throws Exception {
		//create the componentclass instance
		ComponentClass monCompo = new ComponentClass(getNamespaceRef());
		//add the bean class.
		StandardComponentClass monS = null;
		monS = createStandardComponentMEP(monJdbcBC);
		monCompo.setMonComponentStandard(monS);
		return monCompo;
	}

	/**
	 * creates the standard components.
	 * @param monWS : the cimero WS
	 * @return the standard component class.
	 * @throws Exception : if the generation return an error.
	 */
	private StandardComponentClass createStandardComponentMEP(
			final JdbcBC monJdbcBC) throws Exception {
		StandardComponentClass monStd = new StandardComponentClass(
				new NameSpaceClass(getNamespaceName() , getNamespaceValue()));
		monStd.setMyJBICompo(createJBIElement(monJdbcBC));
		return monStd;
	}

	/**
	 * @return the namespace name.
	 */
	public final String getNamespaceName() {
		return NAMESPACENAME;
	}

	/**
	 *@return the namespace value.
	 */
	public final String getNamespaceValue() {
		return NAMESPACEVALUE;
	}

}
