package com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.standardFactory.SE;

import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.SEModel.MessageFilter;
import com.bull.cimero.pluginEditor.editors.model.SEModel.Splitter;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.lWFactory.GeneriqueLWfactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ActivationSpecClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.NameSpaceClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.StandardEIPMessageFilter;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.StandardEIPSplitter;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.standardJBIClassImplementation.StandardJBIClass;

/**
 * The MessageFilter factory.
 * @author Cedric Mouilleron
 * @author Pierre Notel
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class MessageFilterFactory extends StandardEIPFactory {

    /**
     * default constructor.
     * @param nms : the namespace using by the servicemix factory.
     */
    public MessageFilterFactory(final NameSpaceClass nms) {
        super(nms);
    }


    /**
     * @see com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory
     * .standardFactory.StandardComponentFactory#createJBIElement(
     * com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel)
     * @param elt : the cimero component.
     * @return the standard JBI class create with the cimero component.
     * @throws Exception if the generation failed.
     */
    public final StandardJBIClass createJBIElement(
            final GeneriqueCimeroModel elt) throws Exception {
        StandardEIPMessageFilter myMessageFilterSM = null;
        if (elt instanceof MessageFilter) {
            MessageFilter myMessageFilterCimero = (MessageFilter) elt;
            //get the destination element.
            GeneriqueCimeroModel destElt = getDestination(elt);
            if (destElt != null) {
                NameSpaceClass destinationNms = getNamespace(destElt);
                myMessageFilterSM = new StandardEIPMessageFilter(
                        new NameSpaceClass(getNamespaceName(),
                                  getNamespaceValue()),
                        myMessageFilterCimero.getName(),
                        new NameSpaceClass(GeneriqueLWfactory.NAMESPACENAME,
                                        GeneriqueLWfactory.NAMESPACEVALUE),
                        destElt.getName(),
                        destinationNms,
                        myMessageFilterCimero.getXpathRule(),
                        myMessageFilterCimero.getName()
                        );
            } else {
                throw new Exception("The destination component of the "
                        + elt.getName() + " object doesn't exist");
            }
        } else {
            throw new Exception("the object : "
                    + elt.getName() + " hasn't a good type for this factory");
        }
        return myMessageFilterSM;
    }


}
