package com.bull.cimero.pluginEditor.generator.serviceMixGenerator.jarGenerator;

import it.eng.components.cimero.DroolsSemanticValidator;
import it.eng.components.cimero.GenericEnricher;
import it.eng.components.cimero.GenericGroovyComponent;
import it.eng.components.cimero.IRequireDataSourceComponent;

import java.util.Iterator;

import com.bull.cimero.pluginEditor.editors.model.Connection;
import com.bull.cimero.pluginEditor.editors.model.Diagram;
import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.BCModel.FileBC;
import com.bull.cimero.pluginEditor.editors.model.BCModel.JMS;
import com.bull.cimero.pluginEditor.editors.model.BCModel.JdbcBC;
import com.bull.cimero.pluginEditor.editors.model.BCModel.ServiceWeb;
import com.bull.cimero.pluginEditor.editors.model.BCModel.TCPBC;
import com.bull.cimero.pluginEditor.editors.model.SEModel.Aggregator;
import com.bull.cimero.pluginEditor.editors.model.SEModel.MessageFilter;
import com.bull.cimero.pluginEditor.editors.model.SEModel.Pipeline;
import com.bull.cimero.pluginEditor.editors.model.SEModel.Routeur;
import com.bull.cimero.pluginEditor.editors.model.SEModel.Splitter;
import com.bull.cimero.pluginEditor.editors.model.SEModel.TCPPipeline;
import com.bull.cimero.pluginEditor.editors.model.SEModel.Transformeur;
import com.bull.cimero.pluginEditor.editors.model.SEModel.WireTap;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.FactoryFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.GenericServiceMixFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.lWFactory.GeneriqueLWFactoryImpl;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.lWFactory.GeneriqueLWfactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.lWFactory.SE.RouteurFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.standardFactory.GeneriqueStandardFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.standardFactory.BC.FileBCFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.standardFactory.BC.JMSFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.standardFactory.BC.JdbcBCFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.standardFactory.BC.ServiceWebFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.standardFactory.BC.TCPBCFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.standardFactory.SE.PipelineFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.standardFactory.SE.StandardEIPFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.standardFactory.SE.TCPPipelineFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.standardFactory.SE.WireTapFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.IModelServiceMix;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ActivationSpecClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.NameSpaceClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.standardModel.ServiceAssemblyClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.standardModel.ServiceUnitClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.standardModel.StandardJBIJARClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.standardModel.StandardLWComponentClass;
import com.bull.cimero.pluginEditor.utils.log.CimeroLog;

/**
 * This class is the factory of the serviceMix model. It is used
 * with a CimeroModel to generate the servicemix model associated. It will
 * be used to generate the jar configuration package.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class JarModelFactory {

	//
	// SU NAME
	//
	/**The HTTP standard SU name.*/
	private static final String HTTPNAME = "http-su";
	/**The EIP standard SU name.*/
	private static final String EIPNAME = "eip-su";
	/**The LW compo name.*/
	private static final String LWNAME = "lw-su";
	/**The JMS compo name.*/
	private static final String JMSNAME = "jms-su";
	/**The Spago compo name */
	private static final String SPAGONAME = "spago-su";
	
	private static final String DROOLSNAME = "drools-su";
	
	
	private static final String JDBCNAME = "jdbc-su";

	private static final String TCPNAME = "tcp-su";
	private static final String FILENAME = "file-su";
	
	//
	// EIP SU DESCRIPTION
	//
	/**The EIP standard SU description.*/
	private static final String EIPDESCR = "Contains eip components";
	/**The EIP standard SU artifact name.*/
	private static final String EIPARTIFACTNAME = "eip-su.zip";
	/**The EIP standard SU component name.*/
	public static final String EIPCOMPONENTNAME = "servicemix-eip";
	
	//
	//SPAGO
	//
	
	private static final String SPAGODESCR = "Contains spago components";
	/**The EIP standard SU artifact name.*/
	private static final String SPAGOARTIFACTNAME = "spago-su.zip";
	/**The EIP standard SU component name.*/
	public static final String SPAGOCOMPONENTNAME = "spago";
	//
	// JMS SU DESCRIPTION
	//
	/**The JMS standard SU description.*/
	private static final String JMSDESCR = "Contains the jms components";
	/**The HTTP standard SU artifact name.*/
	private static final String JMSARTIFACTNAME = "jms-su.zip";
	/**The HTTP standard SU component name.*/
	public static final String JMSCOMPONENTNAME = "servicemix-jms";
	//
	// HTTP SU DESCRIPTION
	//
	/**The HTTP standard SU description.*/
	private static final String HTTPDESCR = "Contains the http components";
	/**The HTTP standard SU artifact name.*/
	private static final String HTTPARTIFACTNAME = "http-su.zip";
	/**The HTTP standard SU component name.*/
	public static final String HTTPCOMPONENTNAME = "servicemix-http";
	
	/**The HTTP standard SU description.*/
	private static final String DROOLSDESCR = "Contains the drools components";
	/**The HTTP standard SU artifact name.*/
	private static final String DROOLSARTIFACTNAME = "drools-su.zip";
	/**The HTTP standard SU component name.*/
	public static final String DROOLSCOMPONENTNAME = "servicemix-drools";
	
	/**The HTTP standard SU description.*/
	private static final String JDBCDECSR = "Contains the jdbc components";
	/**The HTTP standard SU artifact name.*/
	private static final String JDBCDARTIFACTNAME = "jdbc-su.zip";
	/**The HTTP standard SU component name.*/
	public static final String JDBCCOMPONENTNAME = "spagic-jdbc";
	
	/**The TCP standard SU description.*/
	private static final String TCPDECSR = "Contains the tcp components";
	/**The TCP standard SU artifact name.*/
	private static final String TCPDARTIFACTNAME = "tcp-su.zip";
	/**The TCP standard SU component name.*/
	public static final String TCPCOMPONENTNAME = "spagic-tcp";
	
	/**The File standard SU description.*/
	private static final String FILEDESCR = "Contains the file components";
	/**The TCP standard SU artifact name.*/
	private static final String FILEARTIFACTNAME = "file-su.zip";
	/**The TCP standard SU component name.*/
	public static final String FILECOMPONENTNAME = "servicemix-file";
	
	//
	// LW SU DESCRIPTION
	//
	/**The LW component description.*/
	private static final String LWDESCR = "Contains the lightweight components";
	/**The LW artifacts name.*/
	private static final String LWARTIFACTNAME = "lw-su.zip";
	/**The LW component name.*/
	public static final String LWCOMPONENTNAME = "servicemix-lwcontainer";
	//
	// SA DESCRIPTION
	//
	/**The SA name.*/
	public static final String SANAME = "CimeroJar";
	/**The SA description.*/
	private static final String SADESCR = "generated by Cimero Editor";
	/**The default namespace name.*/
	public static final String DEFNAMESPACENAME = "sm";
	/**The default namespace value.*/
	public static final String DEFNAMESPACEVALUE =
		"http://servicemix.apache.org/config/1.0";
	//
	// CLASS VARIABLE
	//
	/**The cimero model.*/
	private Diagram cimeroModel = null;
	/**This namespace is the default namespace using by servicemix.*/
	private NameSpaceClass defaultNamespace = null;
	/**to determine if the validation must to be do.*/
	private boolean isForce = false;
	/**The default name of this component.*/
	private String nameJar = "";

	//-------------------------------------
	// CONSTRUCTOR

	/**
	 * This constructor has an parameter.
	 * @param cimeroD : the diagram which will be use to generate the
	 * @param force : false if the model validation must be do
	 * 	ServiceMix jar model.
	 */
	public JarModelFactory(final Diagram cimeroD, final boolean force) {
		this.cimeroModel = cimeroD;
		this.isForce = force;
		defaultNamespace = new NameSpaceClass(DEFNAMESPACENAME,
				DEFNAMESPACEVALUE);
	}

	//--------------------------------------------
	// SERVICE ASSEMBLY FUNCTION
	/**
	 * this function is used to generate the servicemix model.
	 * The generation is based on the CimeroModel.
	 * @return the servicemix model
	 */
	public final IModelServiceMix generateModelServiceMix() {
		if (cimeroModel != null) {
			//set the name of the file
			setNameJar(cimeroModel.getCimeroDiagramName());
			//create the serviceAssembly
			ServiceAssemblyClass mySA = createServiceAssembly();
			//create all the ServiceUnit Class.
			for (int i = 0; i < cimeroModel.getLstCimeroObject().size(); i++) {
				GeneriqueCimeroModel elt = (GeneriqueCimeroModel)
				cimeroModel.getLstCimeroObject().get(i);
				//
				//we test if the component is valid
				//
				if ((!elt.isValide(false)) && (!isForce)) {
					//not valid
					return null;
				}

				if (elt instanceof ServiceWeb) {
					//HTTP
					addStandardComponentInSUHTTP(elt, mySA);
				} else if (elt instanceof JMS) {
					//JMS
					addStandardComponentInSUJMS(elt, mySA);
				} else if (elt instanceof WireTap) {
					//EIP
					addStandardComponentInSUEIP(elt, mySA);
				} else if (elt instanceof Splitter) {
					//EIP
					addStandardComponentInSUEIP(elt, mySA);
				} else if (elt instanceof Aggregator) {
					//EIP
					addStandardComponentInSUEIP(elt, mySA);
                } else if (elt instanceof MessageFilter) {
                    //EIP
                    addStandardComponentInSUEIP(elt, mySA);
				} else if (elt instanceof Pipeline) {
					//EIP
					Pipeline myPipeline = (Pipeline) elt;
					//create the pipeline eip component
					addStandardComponentInSUEIP(elt, mySA);
					//create the http component
					ServiceWeb wsTmp = new ServiceWeb();
					wsTmp.setInput(true);
					wsTmp.setOutput(false);
					wsTmp.setName(elt.getName()
							+ PipelineFactory.HTTPSUFFIXE);
					wsTmp.setHostNameAdress(myPipeline.getLocationURI());
					wsTmp.setSoapRequest(myPipeline.isSoapRequest());
					wsTmp.setSoapVersion(myPipeline.getSoapVersion());
					wsTmp.setSoapAction(myPipeline.getSoapAction());
					addStandardComponentInSUHTTP(wsTmp, mySA);
				}else if (elt instanceof GenericEnricher ) {
					//EIP
					GenericEnricher myPipeline = (GenericEnricher) elt;
					//create the pipeline eip component
					addStandardComponentInSUEIP(elt, mySA);
					//create the http component
					ServiceWeb wsTmp = new ServiceWeb();
					wsTmp.setInput(true);
					wsTmp.setOutput(false);
					wsTmp.setName(elt.getName()
							+ PipelineFactory.HTTPSUFFIXE);
					wsTmp.setHostNameAdress(myPipeline.getLocationURI());
					wsTmp.setSoapRequest(myPipeline.isSoapRequest());
					wsTmp.setSoapVersion(myPipeline.getSoapVersion());
					if (!"".equals(myPipeline.getSoapActionFromCombo())) {
						wsTmp.setSoapAction(myPipeline.getSoapActionFromCombo());
					} else {
						wsTmp.setSoapAction(myPipeline.getSoapAction());
					}
					wsTmp.setCreatedByPipeline(true);
					addStandardComponentInSUHTTP(wsTmp, mySA);
				// -- (BEGIN) ANDREA ZOPPELLO - DROOLS AS STANDARD COMPONENT
				}
				else if (elt instanceof DroolsSemanticValidator) {
						DroolsSemanticValidator droolsSemnaticValidator = (DroolsSemanticValidator) elt;
						addStandardComponentInSUDROOLS(elt, mySA);
				// -- (END)
				} else if (elt instanceof Routeur) {
					//the routeur is little different. We cannot create it
					//with the generique calls function.
					Routeur myRout = (Routeur) elt;
					addStandardComponentInSUDROOLS(elt, mySA);
		
					// -- (END)
				}else if (elt instanceof JdbcBC) {
					//HTTP
					addStandardComponentInSUJDBC(elt, mySA);
				}else if (elt instanceof TCPBC) {
					//TCP
					addStandardComponentInSUTCP(elt, mySA);
				} else if (elt instanceof TCPPipeline) {
					//EIP
					TCPPipeline myPipeline = (TCPPipeline) elt;
					//create the pipeline eip component
					addStandardComponentInSUEIP(elt, mySA);
					//create the tcp component
					TCPBC bcModel = new TCPBC();
					copyModelProperties(myPipeline, bcModel);
					bcModel.setInput(true);
					bcModel.setOutput(false);
					bcModel.setCustomComponent(true);
					bcModel.setCustomComponentId("TCPBC");
					bcModel.setName(myPipeline.getName()+
							TCPPipelineFactory.TCPSUFFIXE);
					bcModel.setCreatedByPipeline(true);
					addStandardComponentInSUTCP(bcModel, mySA);
				}else if(elt instanceof FileBC)
	                    addStandardComponentInSUFile(elt, mySA);

				else {
					//it is a LW component.
					addComponentInLWSU(elt , mySA);
				}
			}
			return mySA;
		} else {
			return null;
		}
	}
	
	private void copyModelProperties(GeneriqueCimeroModel src,
		GeneriqueCimeroModel dest) {

		Iterator<String> iter = src.getPropertyKeys().iterator();
		
		String key = "";
		while (iter.hasNext()) {
			key = iter.next();
			dest.setProperty(key, src.getProperty(key));
		}
	}

	//-------------------------------------------
	// SU FUNCTION

	
	/**
	 * This function is used to add eip component in the SA.
	 * @param elt : the GeneriqueCimero component.
	 * @param mySA : the SA where this component will be add.
	 */
	private void addStandardComponentInSUEIP(final GeneriqueCimeroModel elt,
			final ServiceAssemblyClass mySA) {
		//search if a EIP SU already exist.
		StandardJBIJARClass myStd = getExistingSU(mySA, getNameJar() + EIPNAME);

		if (myStd == null) {
			//the eip name doesn't exist. We create it.
			myStd = new StandardJBIJARClass();
			//add the default namespace
			myStd.addNamespace(this.defaultNamespace);
			//add the eip namespace
			NameSpaceClass eipNms = new NameSpaceClass(
					StandardEIPFactory.NAMESPACENAME ,
					StandardEIPFactory.NAMESPACEVALUE);
			myStd.addNamespace(eipNms);
			//add the SU that contain the LW compo
			ServiceUnitClass mySU = new ServiceUnitClass(
					getNameJar() + EIPNAME , EIPDESCR ,
					getNameJar() + EIPARTIFACTNAME ,
					EIPCOMPONENTNAME);
			mySU.setMonStandard(myStd);
			mySA.addSU(mySU);
		}



		if (elt instanceof WireTap) {
			//the copy destination service cannot be specify with
			//the generique function.
			WireTap myWT = (WireTap) elt;
			//add wiretap component
			addWireTapComponent(myWT , myStd);
		} else if (elt instanceof Pipeline) {
			Pipeline myPipeline = (Pipeline) elt;
			//add pipeline component.
			addPipelineComponent(myPipeline, myStd);
		} else if (elt instanceof TCPPipeline) {
			TCPPipeline myPipeline = (TCPPipeline) elt;
			//add pipeline component.
			addTCPPipelineComponent(myPipeline, myStd);
		} else {
			//
			//create the factory
			//
			StandardEIPFactory myFact = null;
			try {
				myFact = (StandardEIPFactory) FactoryFactory.createFactory(
						elt, this.defaultNamespace, false);
			} catch (Exception e) {
				CimeroLog.errorLog("eip factory creation error", e);
			}
			//
			//call the create jbi generation function
			//
			try {
				myStd.addStandardJBIElement(myFact.createJBIElement(elt));
			} catch (Exception e) {
				CimeroLog.errorLog("eip factory generation error", e);
				return;
			}

		}
		//add the namespace of this elt and the destination namespace
		addNamespaceInStandardJBICompo(elt , myStd);

	}

	/**
	 * This function is used to add a http component in the SA.
	 * @param elt : the http cimero component.
	 * @param mySA : the SA where the HHTP component will be add.
	 */
	private void addStandardComponentInSUJMS(final GeneriqueCimeroModel elt,
			final ServiceAssemblyClass mySA) {
		if (elt == null) {
			CimeroLog.errorLog("the component cannot be null",
					new Exception());
		}
		StandardJBIJARClass myStd = getExistingSU(mySA, getNameJar() + JMSNAME);

		if (myStd == null) {
			myStd = new StandardJBIJARClass();
			myStd.addNamespace(this.defaultNamespace);
			NameSpaceClass jmsNms = new NameSpaceClass(
					JMSFactory.NAMESPACENAME ,
					JMSFactory.NAMESPACEVALUE);
			myStd.addNamespace(jmsNms);
			ServiceUnitClass mySU = new ServiceUnitClass(
					getNameJar() + JMSNAME , JMSDESCR,
					getNameJar() + JMSARTIFACTNAME ,
					JMSCOMPONENTNAME);
			mySU.setMonStandard(myStd);
			mySA.addSU(mySU);
		}
		//
		//create the factory
		//
		GeneriqueStandardFactory myFact = null;
		try {
			myFact = (GeneriqueStandardFactory) FactoryFactory
				.createFactory(elt, this.defaultNamespace, !elt.isInput());
		} catch (Exception e) {
			CimeroLog.errorLog("jms factory unknown", e);
			return;
		}
		//
		//call the create jbi generation function
		//
		try {
			myStd.addStandardJBIElement(myFact.createJBIElement(elt));
		} catch (Exception e) {
			CimeroLog.errorLog("jms factory generation exception", e);
			return;
		}
		//
		//add the namespace of this elt and the destination namespace
		//
		addNamespaceInStandardJBICompo(elt , myStd);
	}

	
	private void addStandardComponentInSUDROOLS(final GeneriqueCimeroModel elt,
			final ServiceAssemblyClass mySA){
		if (elt == null) {
			CimeroLog.errorLog("the component cannot be null",
					new Exception());
		}
		
		StandardJBIJARClass myStd = getExistingSU(mySA, getNameJar() + DROOLSNAME);
		if (myStd == null) {
			//we create this SU
			myStd = new StandardJBIJARClass();
			//if the default namespace is not specify
			if (!myStd.nameSpaceExist(this.defaultNamespace)) {
				//we add this namespace
				myStd.addNamespace(this.defaultNamespace);
			}
			
			NameSpaceClass nms = new NameSpaceClass(
					com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.standardFactory.SE.RouteurFactory.NAMESPACENAME ,
					com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.standardFactory.SE.RouteurFactory.NAMESPACEVALUE);
			if (!myStd.nameSpaceExist(nms)) {
				//we add this namespace
				myStd.addNamespace(nms);
			}
			//add the SU that contain the LW compo
			ServiceUnitClass mySU = new ServiceUnitClass(
					getNameJar() + DROOLSNAME , DROOLSDESCR ,
					getNameJar() + DROOLSARTIFACTNAME ,
					DROOLSCOMPONENTNAME);
			mySU.setMonStandard(myStd);
			mySA.addSU(mySU);
		}
		
		//
		//create the factory
		//
		GeneriqueStandardFactory myFact = null;
		try {
			System.out.println("elt.getClass()" + elt.getClass().getName());
			elt.setStandard(true);
			myFact = (GeneriqueStandardFactory) FactoryFactory
				.createFactory(elt, this.defaultNamespace, !elt.isInput());
		} catch (Exception e) {
			CimeroLog.errorLog("http factory unknown", e);
			return;
		}
		//
		//call the create jbi generation function
		//
		try {
			myStd.addStandardJBIElement(myFact.createJBIElement(elt));
		} catch (Exception e) {
			CimeroLog.errorLog("http factory generation exception", e);
			return;
		}
		
		 if (elt instanceof DroolsSemanticValidator) {
			 myStd.addIncludeFilesInZip(((DroolsSemanticValidator)elt).getRuleBaseResourcePath());
		}
		//
		//add the namespace of this elt and the destination namespace
		//
		addNamespaceInStandardJBICompoForDroolsSU(elt , myStd);
		
	}
	/**
	 * This function is used to add a http component in the SA.
	 * @param elt : the http cimero component.
	 * @param mySA : the SA where the HHTP component will be add.
	 */
	private void addStandardComponentInSUHTTP(final GeneriqueCimeroModel elt,
			final ServiceAssemblyClass mySA) {
		if (elt == null) {
			CimeroLog.errorLog("the component cannot be null",
					new Exception());
		}
		StandardJBIJARClass myStd = getExistingSU(mySA, getNameJar() + HTTPNAME);

		if (myStd == null) {
			//we create this SU
			myStd = new StandardJBIJARClass();
			//if the default namespace is not specify
			if (!myStd.nameSpaceExist(this.defaultNamespace)) {
				//we add this namespace
				myStd.addNamespace(this.defaultNamespace);
			}
			//if the http namespace is not specify
			NameSpaceClass nms = new NameSpaceClass(
					ServiceWebFactory.NAMESPACENAME ,
					ServiceWebFactory.NAMESPACEVALUE);
			if (!myStd.nameSpaceExist(nms)) {
				//we add this namespace
				myStd.addNamespace(nms);
			}
			//add the SU that contain the LW compo
			ServiceUnitClass mySU = new ServiceUnitClass(
					getNameJar() + HTTPNAME , HTTPDESCR ,
					getNameJar() + HTTPARTIFACTNAME ,
					HTTPCOMPONENTNAME);
			mySU.setMonStandard(myStd);
			mySA.addSU(mySU);
		}
		//
		//create the factory
		//
		GeneriqueStandardFactory myFact = null;
		try {
			System.out.println("elt.getClass()" + elt.getClass().getName());
			myFact = (GeneriqueStandardFactory) FactoryFactory
				.createFactory(elt, this.defaultNamespace, !elt.isInput());
		} catch (Exception e) {
			CimeroLog.errorLog("http factory unknown", e);
			return;
		}
		//
		//call the create jbi generation function
		//
		try {
			myStd.addStandardJBIElement(myFact.createJBIElement(elt));
		} catch (Exception e) {
			CimeroLog.errorLog("http factory generation exception", e);
			return;
		}
		//
		//add the namespace of this elt and the destination namespace
		//
		addNamespaceInStandardJBICompo(elt , myStd);

	}
	
	private void addStandardComponentInSUJDBC(final GeneriqueCimeroModel elt,
			final ServiceAssemblyClass mySA) {
		if (elt == null) {
			CimeroLog.errorLog("the component cannot be null",
					new Exception());
		}
		StandardJBIJARClass myStd = getExistingSU(mySA, getNameJar() + HTTPNAME);

			if (myStd == null) {
				//we create this SU
				myStd = new StandardJBIJARClass();
				//if the default namespace is not specify
				if (!myStd.nameSpaceExist(this.defaultNamespace)) {
					//we add this namespace
					myStd.addNamespace(this.defaultNamespace);
				}
				//if the http namespace is not specify
				NameSpaceClass nms = new NameSpaceClass(
						JdbcBCFactory.NAMESPACENAME ,
						JdbcBCFactory.NAMESPACEVALUE);
				if (!myStd.nameSpaceExist(nms)) {
					//we add this namespace
					myStd.addNamespace(nms);
				}
				//add the SU that contain the LW compo
				ServiceUnitClass mySU = new ServiceUnitClass(
						getNameJar() + JDBCNAME , JDBCDECSR ,
						getNameJar() + JDBCDARTIFACTNAME ,
						JDBCCOMPONENTNAME);
				mySU.setMonStandard(myStd);
				mySA.addSU(mySU);
			}
			//
			//create the factory
			//
			GeneriqueStandardFactory myFact = null;
			try {
				System.out.println("elt.getClass()" + elt.getClass().getName());
				myFact = (GeneriqueStandardFactory) FactoryFactory
					.createFactory(elt, this.defaultNamespace, !elt.isInput());
			} catch (Exception e) {
				CimeroLog.errorLog("http factory unknown", e);
				return;
			}
			//
			//call the create jbi generation function
			//
			try {
				myStd.addStandardJBIElement(myFact.createJBIElement(elt));
			} catch (Exception e) {
				CimeroLog.errorLog("jdbc factory generation exception", e);
				return;
			}
			//
			//add the namespace of this elt and the destination namespace
			//
			addNamespaceInStandardJBICompo(elt , myStd);

	}

	private void addStandardComponentInSUTCP(final GeneriqueCimeroModel elt,
			final ServiceAssemblyClass mySA) {
		
		if (elt == null) {
			CimeroLog.errorLog("the component cannot be null",
					new Exception());
		}
		StandardJBIJARClass myStd = getExistingSU(mySA, getNameJar() + TCPNAME);
		
		if (myStd == null) {
			//we create this SU
			myStd = new StandardJBIJARClass();
			//if the default namespace is not specify
			if (!myStd.nameSpaceExist(this.defaultNamespace)) {
				//we add this namespace
				myStd.addNamespace(this.defaultNamespace);
			}
			//if the http namespace is not specify
			NameSpaceClass nms = new NameSpaceClass(
					TCPBCFactory.NAMESPACENAME ,
					TCPBCFactory.NAMESPACEVALUE);
			if (!myStd.nameSpaceExist(nms)) {
				//we add this namespace
				myStd.addNamespace(nms);
			}
			//add the SU that contain the LW compo
			ServiceUnitClass mySU = new ServiceUnitClass(
					getNameJar() + TCPNAME , TCPDECSR ,
					getNameJar() + TCPDARTIFACTNAME ,
					TCPCOMPONENTNAME);
			mySU.setMonStandard(myStd);
			mySA.addSU(mySU);
		}
		//
		//create the factory
		//
		GeneriqueStandardFactory myFact = null;
		try {
			System.out.println("elt.getClass()" + elt.getClass().getName());
			myFact = (GeneriqueStandardFactory) FactoryFactory
				.createFactory(elt, this.defaultNamespace, !elt.isInput());
		} catch (Exception e) {
			CimeroLog.errorLog("tcp factory unknown", e);
			return;
		}
		//
		//call the create jbi generation function
		//
		try {
			myStd.addStandardJBIElement(myFact.createJBIElement(elt));
		} catch (Exception e) {
			CimeroLog.errorLog("TCP factory generation exception", e);
			return;
		}
		//
		//add the namespace of this elt and the destination namespace
		//
		addNamespaceInStandardJBICompo(elt , myStd);

	}
	
    private void addStandardComponentInSUFile(GeneriqueCimeroModel elt, ServiceAssemblyClass mySA)
    {
		if (elt == null) {
			CimeroLog.errorLog("the component cannot be null",
					new Exception());
		}
		StandardJBIJARClass myStd = getExistingSU(mySA, getNameJar() + FILENAME);
        
		if (myStd == null) {
			//we create this SU
			myStd = new StandardJBIJARClass();
			//if the default namespace is not specify
			if (!myStd.nameSpaceExist(this.defaultNamespace)) {
				//we add this namespace
				myStd.addNamespace(this.defaultNamespace);
			}
			//if the http namespace is not specify
            NameSpaceClass nms = new NameSpaceClass(FileBCFactory.NAMESPACENAME, FileBCFactory.NAMESPACEVALUE);
            if (!myStd.nameSpaceExist(nms)) {
				//we add this namespace
				myStd.addNamespace(nms);
			}
			//add the SU that contain the LW compo            
            ServiceUnitClass mySU = new ServiceUnitClass(
					getNameJar() + FILENAME , FILEDESCR ,
					getNameJar() + FILEARTIFACTNAME ,
					FILECOMPONENTNAME);
			mySU.setMonStandard(myStd);
			mySA.addSU(mySU);
        }
        GeneriqueStandardFactory myFact = null;
        try {
            System.out.println((new StringBuilder("elt.getClass()")).append(elt.getClass().getName()).toString());
            myFact = (GeneriqueStandardFactory)FactoryFactory.createFactory(elt, defaultNamespace, Boolean.valueOf(!elt.isInput()));
        }catch(Exception e) {
            CimeroLog.errorLog("http factory unknown", e);
            return;
        }
        try {
            myStd.addStandardJBIElement(myFact.createJBIElement(elt));
        }catch(Exception e) {
            CimeroLog.errorLog("File factory generation exception", e);
            return;
        }
        addNamespaceInStandardJBICompo(elt, myStd);
    }

	/**
	 * this function add a standard LW component in the SA. If this
	 * stadard LW component are not created, this function create it.
	 * @param elt : the element to add in the std LW component.
	 * @param mySA : the sa where this element will be add.
	 */
	private void addComponentInLWSU(final GeneriqueCimeroModel elt,
			final ServiceAssemblyClass mySA) {
		//we test if a LW stadard compo has been already created.
		boolean lwExist = false;
		StandardLWComponentClass myStdLW = null;
		for (int i = 0; i < mySA.getLstSU().size(); i++) {
			if (!mySA.getLstSU().get(i).isStandard()) {
				//we get the LW component.
				myStdLW = mySA.getLstSU().get(i).getLW();
				lwExist = true;
			}
		}
		if (!lwExist) {
			//we create the standard LW component
			myStdLW = new StandardLWComponentClass(this.defaultNamespace);
			myStdLW.addNamespace(this.defaultNamespace);
			//add the SU that contain the LW compo
			ServiceUnitClass mySrc = new ServiceUnitClass(
					getNameJar() + LWNAME , LWDESCR ,
					getNameJar() + LWARTIFACTNAME ,
					LWCOMPONENTNAME);
			mySrc.addLW(myStdLW);
			mySA.addSU(mySrc);
		}

		//we create the element
		if (elt instanceof Routeur) {
			//the routeur is little different. We cannot create it
			//with the generique calls function.
			Routeur myRout = (Routeur) elt;
			addRouteur(myRout , myStdLW);
		} else {
			//
			//create the factory
			//
			
			GenericServiceMixFactory myFact = null;
			try {
				myFact = FactoryFactory.createFactory(
						elt, this.defaultNamespace, !elt.isInput());
			} catch (Exception e) {
				CimeroLog.errorLog("Lw factory creation error", e);
			}
			//
			//call the createXMLModel function
			//
			
			if (elt instanceof Transformeur){
				myStdLW.addIncludeFilesInZip(((Transformeur)elt).getXsltPath());
			}
			if (elt instanceof GenericGroovyComponent){
				myStdLW.addIncludeFilesInZip(((GenericGroovyComponent)elt).getGroovyPath());
			}
			if (elt instanceof IRequireDataSourceComponent) {
				String requiredDs = ((IRequireDataSourceComponent)elt).getDataSource();
				if (!myStdLW.containsDataSource(requiredDs)){
					myStdLW.addDataSourceBean(requiredDs);
				}
			}
			try {
				myStdLW.addActivationSpec(
						myFact.createXMLModel(elt, isForce));
			} catch (Exception e) {
			   CimeroLog.errorLog("exception during XML generation "
					+ "of the factory :" + myFact.getClass().getName(),
					e);
			}
		}
		//add the namespace of this element and the namespace of
		//this destination component.
		addNamespaceInStandardLWCompo(elt , myStdLW);

	}

	//-------------------------------------------------
	// SPECIFIC FUNCTION

	//-----------------------------------------
	// SPECIFIC FUNCTION

	/**
	 * add in a standard JBI Jar class a pipeline component.
	 * @param myPipeline : the standard EIP pipeline component.
	 * @param myStd : the standard JBI Jar class where this component will
	 * be add.
	 */
	private void addPipelineComponent(final Pipeline myPipeline,
			final StandardJBIJARClass myStd) {
		GeneriqueStandardFactory myFact = null;
		myFact = new PipelineFactory(this.defaultNamespace);
		try {
			myStd.addStandardJBIElement(myFact.createJBIElement(myPipeline));

			//add the destination namespace
			NameSpaceClass nmsDest =
				myFact.getNamespace(myFact.getDestination(myPipeline));
			if (!myStd.nameSpaceExist(nmsDest)) {
				myStd.addNamespace(nmsDest);
			}

			//add the http destination namespace
			nmsDest = new NameSpaceClass(ServiceWebFactory.NAMESPACENAME,
					ServiceWebFactory.NAMESPACEVALUE);
			if (!myStd.nameSpaceExist(nmsDest)) {
				myStd.addNamespace(nmsDest);
			}
		} catch (Exception e) {
			CimeroLog.errorLog("exception during the tracer model generation",
					e);
		}

	}

	private void addTCPPipelineComponent(final TCPPipeline myPipeline,
			final StandardJBIJARClass myStd) {
		GeneriqueStandardFactory myFact = null;
		myFact = new TCPPipelineFactory(this.defaultNamespace);
		try {
			myStd.addStandardJBIElement(myFact.createJBIElement(myPipeline));

			//add the destination namespace
			NameSpaceClass nmsDest =
				myFact.getNamespace(myFact.getDestination(myPipeline));
			if (!myStd.nameSpaceExist(nmsDest)) {
				myStd.addNamespace(nmsDest);
			}

			//add the tcp destination namespace
			nmsDest = new NameSpaceClass(TCPBCFactory.NAMESPACENAME,
					TCPBCFactory.NAMESPACEVALUE);
			if (!myStd.nameSpaceExist(nmsDest)) {
				myStd.addNamespace(nmsDest);
			}
		} catch (Exception e) {
			CimeroLog.errorLog("exception during the tracer model generation",
					e);
		}

	}
	/**
	 * add in a standard JBI Jar class a wiretap component.
	 * @param myWT : the wiretap component.
	 * @param myStd : the JBI Jar class where this component will be add.
	 */
	private void addWireTapComponent(final WireTap myWT,
			final StandardJBIJARClass myStd) {
		GeneriqueStandardFactory myFact = null;
		myFact = new WireTapFactory(this.defaultNamespace);
		try {
			myStd.addStandardJBIElement(myFact.createJBIElement(myWT));

			//add the destination namespace
			GeneriqueCimeroModel destElt =
				((Connection) myWT.getConn(false)).getTarget();
			NameSpaceClass nmsDest = myFact.getNamespace(destElt);
			if (!myStd.nameSpaceExist(nmsDest)) {
				myStd.addNamespace(nmsDest);
			}

			//add the copy destination namespace
			GeneriqueCimeroModel destCopy =
				((Connection) myWT.getConn(true)).getTarget();
			nmsDest = myFact.getNamespace(destCopy);
			if (!myStd.nameSpaceExist(nmsDest)) {
				myStd.addNamespace(nmsDest);
			}
		} catch (Exception e) {
			CimeroLog.errorLog("exception during the tracer model generation",
					e);
		}
	}



	/**
	 * This function return the existing SU if it exists.
	 * @param mySA : the SA that contains SU
	 * @param name : the name of the SU to search.
	 * @return null if the SU doesn't exist or the SU if it exists.
	 */
	private StandardJBIJARClass getExistingSU(final ServiceAssemblyClass mySA,
			final String name) {
		StandardJBIJARClass myStd = null;
		for (int i = 0; i < mySA.getLstSU().size(); i++) {
			if (mySA.getLstSU().get(i).isStandard()) {
				//test if this is a HTTP Standard element.
				if (mySA.getLstSU().get(i).getSuName().equals(name)) {
					myStd = mySA.getLstSU().get(i).getMonStandard();
					return myStd;
				}
			}
		}
		return null;
	}



	/**
	 * add in the LW component the namespace of the element and the namespace
	 * of it destination.
	 * @param elt : the element.
	 * @param myStdLW : the LW Compo where the namespace will be add.
	 */
	private void addNamespaceInStandardLWCompo(final GeneriqueCimeroModel elt,
			final StandardLWComponentClass myStdLW) {
		GeneriqueLWfactory myFact = new GeneriqueLWFactoryImpl();
		//test if the namespace of this element has been saved
		if (!myStdLW.nameSpaceExist(myFact.getNamespace(elt))) {
			//we add this namespace
			myStdLW.addNamespace(myFact.getNamespace(elt));
		}
		//add the destination namespace
		if (!elt.isInput()) {
			NameSpaceClass nmsDest =
				myFact.getNamespace(myFact.getDestination(elt));
			if (!myStdLW.nameSpaceExist(nmsDest)) {
				//we add this namespace
				myStdLW.addNamespace(nmsDest);
			}
		}
	}

	/**
	 * add in the jbi component the namespace of the element and the namespace
	 * of it destination.
	 * @param elt : the element.
	 * @param myStd : the jbi Compo where the namespace will be add.
	 */
	private void addNamespaceInStandardJBICompo(final GeneriqueCimeroModel elt,
			final StandardJBIJARClass myStd) {
		GeneriqueLWfactory myFact = new GeneriqueLWFactoryImpl();
		//test if the namespace of this element has been saved
		if (!myStd.nameSpaceExist(myFact.getNamespace(elt))) {
			//we add this namespace
			myStd.addNamespace(myFact.getNamespace(elt));
		}
		//add the destination namespace
		if (!elt.isInput()) {
			NameSpaceClass nmsDest =
				myFact.getNamespace(myFact.getDestination(elt));
			if (!myStd.nameSpaceExist(nmsDest)) {
				//we add this namespace
				myStd.addNamespace(nmsDest);
			}
		}
	}
	
	/**
	 * add in the jbi component the namespace of the element and the namespace
	 * of it destination.
	 * @param elt : the element.
	 * @param myStd : the jbi Compo where the namespace will be add.
	 */
	private void addNamespaceInStandardJBICompoForDroolsSU(final GeneriqueCimeroModel elt,
			final StandardJBIJARClass myStd) {
		GeneriqueLWfactory myFact = new GeneriqueLWFactoryImpl();
		
		NameSpaceClass usingNs = new NameSpaceClass(
				GeneriqueLWfactory.NAMESPACENAME,
				GeneriqueLWfactory.NAMESPACEVALUE);
		
		if (!myStd.nameSpaceExist(usingNs)) {
			//we add this namespace
			myStd.addNamespace(usingNs);
		}
		//add the destination namespace
		if (!elt.isInput()) {
			NameSpaceClass nmsDest =
				myFact.getNamespace(myFact.getDestination(elt));
			if (!myStd.nameSpaceExist(nmsDest)) {
				//we add this namespace
				myStd.addNamespace(nmsDest);
			}
		}
	}

	/**
	 * create a routeur in the StdLWcomponent.
	 * @param elt : the model to generate the routeur.
	 * @param myLW : the standardLWcomponent where this routeur will be add.
	 */
	private void addRouteur(final GeneriqueCimeroModel elt,
			final StandardLWComponentClass myLW) {
		if (elt instanceof Routeur) {
			Routeur myRout = (Routeur) elt;
			//we create the Activationspec of this routeur
			RouteurFactory myFact = new RouteurFactory(
					new NameSpaceClass(DEFNAMESPACENAME , DEFNAMESPACEVALUE));
			try {
				myFact.setPrefixeJar("deploy/" + getNameJar() + "droolsFile_");
				ActivationSpecClass myActi =
					myFact.createXMLModel(myRout, isForce);
				myLW.addActivationSpec(myActi);
			} catch (Exception e) {
				CimeroLog.errorLog("exception during the routeur model "
						+ "generation", e);
			}
		}
	}


	/**
	 * create the serviceassembly class.
	 * @return the SA created.
	 */
	private ServiceAssemblyClass createServiceAssembly() {
		ServiceAssemblyClass mySA = new ServiceAssemblyClass(
				getNameJar(), SADESCR);
		return mySA;
	}

	/**
	 * get the name of the jar.
	 * @return the nameJar
	 */
	public final String getNameJar() {
		String[] tmp = nameJar.split("\\.");
		return tmp[0];
	}

	/**
	 * set the name of the jar.
	 * @param nameJarParam the nameJar to set
	 */
	public final void setNameJar(final String nameJarParam) {
		this.nameJar = nameJarParam;
	}
}
