package com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global;

/**
 * This class represents the ComponentClass of the serviceMix model.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 *@author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class ComponentClass {
	/**The namespace reference using by this component.*/
	private NameSpaceClass namespaceComponentRef = null;
	/**a boolean to determine if this component
	 * contain a LWBeanClass or a standardComponentClass.*/
	private boolean beanClass = false;
	/**Reference to the beanClass.*/
	private LWBeanClass monBean = null;
	/**reference to the standardClass.*/
	private StandardComponentClass monCompoStandard = null;

	/**
	 * Constructor with the namespace in parameter.
	 * @param namespaceComponentRefParam : the namespace using by this
	 * 			component
	 */
	public ComponentClass(final NameSpaceClass namespaceComponentRefParam) {
		super();
		this.namespaceComponentRef = namespaceComponentRefParam;
	}

	//-------------------------------------------------
	//SETTERS FUNCTION
	/**
	 * This function is used to set the StandradComponentClass.
	 * @param st : the standardComponent class.
	 */
	public final void setMonComponentStandard(final StandardComponentClass st) {
		if (st != null) {
			this.beanClass = false;
			this.monCompoStandard = st;
		}
	}

	/**
	 * This function is used to set the LWBeanClass.
	 * @param monBeanParam : the beanclass
	 */
	public final void setMonBean(final LWBeanClass monBeanParam) {
		if (monBeanParam != null) {
			this.beanClass = true;
			this.monBean = monBeanParam;
		}
	}

	//----------------------------------------------------
	// GETTER METHOD
	/**
	 * @return true if this component has a reference to a beanClass.
	 */
	public final boolean isBeanClass() {
		return beanClass;
	}

	/**
	 * @return the beanClass reference.
	 */
	public final LWBeanClass getMonBean() {
		return monBean;
	}

	/**
	 * @return the standardComponentClass reference.
	 */
	public final StandardComponentClass getMonCompoStandard() {
		return monCompoStandard;
	}

	/**
	 * @return the namespace using by this component.
	 */
	public final NameSpaceClass getNamespaceComponentRef() {
		return namespaceComponentRef;
	}


}
