package com.bull.cimero.pluginEditor.generator.serviceMixGenerator
	.model.standardModel;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.gef.palette.ToolEntry;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import cimeroEditor.CimeroEditorPlugin;

import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ActivationSpecClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.DatasourceBean;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.NameSpaceClass;
import com.bull.cimero.pluginEditor.utils.ressources.Ressources;

/**
 * This class represents the Ligthweight component using in
 * a standard serviceMix package.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 *@author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class StandardLWComponentClass {
	/**a list of namespace using by this component.*/
	private List < NameSpaceClass > namespaceLst = null;
	/**a list of activation spec class.*/
	private List < ActivationSpecClass > activationSpecLst = null;
	/**The current namespace using by this component.*/
	private NameSpaceClass currentNms = null;
	
	private List < DatasourceBean > lstDataSources  = null;
	
	private List < String > includedFiles  = null;

	/**
	 * default constructor.
	 * @param nms : the namespace using by this component.
	 */
	public StandardLWComponentClass(final NameSpaceClass nms) {
		namespaceLst = new ArrayList < NameSpaceClass >();
		activationSpecLst = new ArrayList < ActivationSpecClass >();
		this.currentNms = nms;
		this.lstDataSources = new ArrayList<DatasourceBean>();
		this.includedFiles = new ArrayList<String>();
	}

	/**
	 * @return Returns the activationSpecLst.
	 */
	public final List < ActivationSpecClass > getActivationSpecLst() {
		return activationSpecLst;
	}

	/**
	 * @return Returns the namespaceLst.
	 */
	public final List < NameSpaceClass > getNamespaceLst() {
		return namespaceLst;
	}

	public final void addIncludeFilesInZip(String absoluteFilePath){
		this.includedFiles.add(absoluteFilePath);
	}
	
	/**
	 * to add an activation spec in the list.
	 * @param myActi : the activation spec to add in the list.
	 */
	public final void addActivationSpec(final ActivationSpecClass myActi) {
		activationSpecLst.add(myActi);
	}

	/**
	 * to add a namespace in the list of namespace.
	 * @param myNms : the namespace to add in the list.
	 */
	public final void addNamespace(final NameSpaceClass myNms) {
		namespaceLst.add(myNms);
	}

	/**
	 * to determine if a namespace is in the namespace list.
	 * @param myNms : the namespace to search in the list.
	 * @return true if the namespace already exist.
	 */
	public final boolean nameSpaceExist(final NameSpaceClass myNms) {
		for (int i = 0; i < namespaceLst.size(); i++) {
			NameSpaceClass nms = namespaceLst.get(i);
			if (nms.getNameSpaceName().compareTo(
					myNms.getNameSpaceName()) == 0) {
				return true;
			}
		}
		return false;
	}

	/**
	 * @return the currentNms
	 */
	public final NameSpaceClass getCurrentNms() {
		return currentNms;
	}

	public final void addDataSourceBean(String datasourceId){
		
		Ressources ressource = new Ressources();
		String fullPath = CimeroEditorPlugin.getRessourcePath("/ressources/DataSources.xml");
		if (fullPath != null) {
			
			if (fullPath.substring(0, "file:".length()).compareTo("file:") == 0) {
				fullPath = fullPath.substring("file:".length());
			}
		
			ressource.setDocument(fullPath);
			
			
			NodeList nodeList = ressource.getXPathNodeSet("/DATASOURCES/DATASOURCE");
			Node node = null;
			NamedNodeMap attributes = null;
			String idTemp = null;
			for (int i = 0; i < nodeList.getLength(); i++) {
				node = nodeList.item(i);				
				attributes = node.getAttributes();
				idTemp = attributes.getNamedItem("id").getNodeValue();
				if (idTemp.equalsIgnoreCase(datasourceId)){
					DatasourceBean dsBean = new DatasourceBean(datasourceId, attributes.getNamedItem("jndiName").getNodeValue());
					this.lstDataSources.add(dsBean);
				}
			}
		}
	}

	public List<DatasourceBean> getLstDataSources() {
		return lstDataSources;
	}

	public void setLstDataSources(List<DatasourceBean> lstDataSources) {
		this.lstDataSources = lstDataSources;
	}
	
	public boolean containsDataSource(String dsId){
		
		for (DatasourceBean dsBean : this.lstDataSources){
			if (dsBean.getId().equalsIgnoreCase(dsId)){
				return true;
			}
		}
		return false;
	}

	public List<String> getIncludedFiles() {
		return includedFiles;
	}

	public void setIncludedFiles(List<String> includedFiles) {
		this.includedFiles = includedFiles;
	}
	

}
