package it.eng.cimero.plugin.actions;

import java.io.BufferedReader;
import java.io.FileReader;

import javax.xml.registry.infomodel.Organization;

import it.eng.cimero.plugin.dialogs.ListRulesDialog;
import it.eng.cimero.plugin.dialogs.ListXmlNamespacesDialog;
import it.eng.serviceregistry.IServiceRegistry;
import it.eng.serviceregistry.JAXRServiceRegistryImpl;
import it.eng.utils.DialogUtils;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

import com.bull.cimero.pluginEditor.utils.log.CimeroLog;

import cimeroEditor.CimeroEditorPlugin;

/**
 * Our sample action implements workbench action delegate.
 * The action proxy will be created by the workbench and
 * shown in the UI. When the user tries to use the action,
 * this delegate will be created and execution will be 
 * delegated to it.
 * @see IWorkbenchWindowActionDelegate
 */
public class LoadBusinessRegistryOrganization implements IWorkbenchWindowActionDelegate {
	private IWorkbenchWindow window;
	/**
	 * The constructor.
	 */
	public LoadBusinessRegistryOrganization() {
	}

	/**
	 * The action has been activated. The argument of the
	 * method represents the 'real' action sitting
	 * in the workbench UI.
	 * @see IWorkbenchWindowActionDelegate#run
	 */
	public void run(IAction action) {
		String fileName = openFileDialog();
		if (fileName != null){
			try{
				loadOrganizations(fileName);
				DialogUtils.information(window.getShell(), "Organizations Loaded Successfully");
			}catch(Exception e){
				CimeroLog.errorLog("Error in Loading Organizations", e);
				DialogUtils.error(window.getShell(), "Error in Loading Organizations");
			}
			
		}
		
	}

	/**
	 * Selection in the workbench has been changed. We 
	 * can change the state of the 'real' action here
	 * if we want, but this can only happen after 
	 * the delegate has been created.
	 * @see IWorkbenchWindowActionDelegate#selectionChanged
	 */
	public void selectionChanged(IAction action, ISelection selection) {
	}

	/**
	 * We can use this method to dispose of any system
	 * resources we previously allocated.
	 * @see IWorkbenchWindowActionDelegate#dispose
	 */
	public void dispose() {
	}

	/**
	 * We will cache window object in order to
	 * be able to provide parent shell for the message dialog.
	 * @see IWorkbenchWindowActionDelegate#init
	 */
	public void init(IWorkbenchWindow window) {
		this.window = window;
	}
	
	public void loadOrganizations(String fileName) throws Exception  {
		BufferedReader br  = null;
		IServiceRegistry serviceRegistry = null;
		try{
			serviceRegistry = CimeroEditorPlugin.getDefault().getServiceRegistry();
			serviceRegistry.initRegistry();

			br = new BufferedReader(new FileReader(fileName));
			String[] tokens = null;
			String line = null;
			
			while ((line = br.readLine()) != null) {
				if (!line.startsWith("#")){
					tokens = line.split(";");
					if (serviceRegistry.getOrganizationByName(tokens[0]) != null ){
						DialogUtils.information(window.getShell(), "Organization "+tokens[0]+ " already registered ");
					}else{
						Organization insOrg = serviceRegistry.newOrganization(tokens[0], tokens[1]);
						serviceRegistry.publishOrganization(insOrg);
						CimeroLog.infoLog("-> Loaded organization ["+tokens[0]+"] In Business registry ");
					}
				}
			}
		}catch (Exception e) {
			e.printStackTrace();
			throw e;
		}finally{
			try{
				br.close();
			}catch (Exception ioe) {
				ioe.printStackTrace();
			}
			serviceRegistry.finalizeRegistry();
		}
	}
	
	protected String openFileDialog(){
		 FileDialog fd = new FileDialog(window.getShell(), SWT.OPEN);
	     fd.setText("Open a valid organizations file ");
	     
	     String[] filterExt = { "*.org" };
	     fd.setFilterExtensions(filterExt);
	     String selected = fd.open();
	     System.out.println(selected);
	     return selected;
	}
}