package it.eng.cimero.plugin.dialogs;

import it.eng.spagosoa.metadb.api.IMetadbApi;
import it.eng.spagosoa.metadb.model.Attribute;
import it.eng.spagosoa.metadb.model.Endpoint;
import it.eng.spagosoa.metadb.model.Rule;
import it.eng.utils.DialogUtils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.vafada.swtcalendar.SWTCalendarEvent;
import org.vafada.swtcalendar.SWTCalendarListener;

import demo.SWTCalendarDialog;

public class DetailRuleDialog extends TitleAreaDialog {

	// - 0: IN (si applica a tutti i MEP)
	// - 1: OUT (si applica ai MEP IN-OUT e IN-OPTIONAL-OUT)
	// - 2: IN-OUT
    
	public static final Integer MEP_APPLY_IN = 0;
	public static final Integer MEP_APPLY_OUT = 1;
	public static final Integer MEP_APPLY_IN_OUT = 2;
	protected final SimpleDateFormat formatter = new SimpleDateFormat("dd/MM/yyyy");
	
//	Scopo della regola.
//	Valori possibili:
//	- 0: estrazione dati
//	- 1: determinazione tipo iter
	
	public static final Integer USAGE_EXTRACT_DATA = 0;
	public static final Integer USAGE_ITER_TYPE_DETERMINATION = 1;
	
	private Composite aComposite = null;
    private CLabel ruleDescriptionLabel = null;
    private Text ruleDescriptionTextField = null;
    
    private CLabel conditionLabel = null;
    private Text conditionTextArea = null;
    
    private CLabel ruleExpressionLabel = null;
    private Text ruleExpressionTextArea = null;
    
    
	private Rule rule = null;
	
    private SashForm internalSash  = null; 

    
    private Group ruleRequireConditionGroup = null;
    private Button ruleSubjectToConditionYesButton = null;
    private Button ruleSubjectToConditionNoButton = null;
    
    
    
    
    
    private Group applyGroup = null;
    private Button applyToIncomingMessage = null;
    private Button applyToOutcomingMessage = null;
    private Button applyToBothMessage = null;
    
    private CLabel startDateLabel = null;
    private Text   startDateTextArea = null;
    
    private CLabel endDateLabel = null;
    private Text   endDateTextArea = null;
    
    private CLabel destinationAttributeLabel = null;
    private Text destinationAttributeTextField = null;
	
    private IMetadbApi metadb = null;
    private Endpoint endpoint = null;
	public DetailRuleDialog(Shell parentShell, Rule rule, IMetadbApi metadb) {
		
		super(parentShell);
		this.metadb = metadb;
		this.rule = rule;
		
	}
	
	
	public DetailRuleDialog(Shell parentShell, Rule rule, Endpoint ep, IMetadbApi metadb) {
		
		this(parentShell,rule, metadb);
		this.endpoint = ep;
		
	}
				

	/**
	   * @see org.eclipse.jface.window.Window#create() We complete the dialog with
	   *      a title and a message
	   */
	  public void create() {
		super.create();
	    setTitle(" Rule Dialog ");
	   
	  }

	 
	  

	@Override
	protected Control createContents(Composite parent) {
		Control ctrl = super.createContents(parent);
		return ctrl;
	}

	protected Control createDialogArea(Composite parent) {
		
        this.internalSash = new SashForm(parent,  SWT.VERTICAL | SWT.FLAT);
        this.internalSash.setLayoutData(new GridData(GridData.FILL_BOTH));
       
		aComposite = new Composite(internalSash, SWT.NONE);
		final GridLayout layout = new GridLayout();
    	layout.marginWidth = 15;
    	layout.marginHeight = 10;
    	layout.numColumns = 10;
    	aComposite.setLayout(layout);
    	
    	aComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    	GridData data = new GridData(GridData.FILL_HORIZONTAL);
    	data.widthHint = 200;
    	data.horizontalSpan = 9;

    	//    	 Riga 1
    	ruleDescriptionLabel = new CLabel(aComposite, SWT.RIGHT);
    	ruleDescriptionLabel.setText(" Rule Description ");
    	ruleDescriptionTextField = new Text(aComposite, SWT.BORDER | SWT.FLAT );
    	ruleDescriptionTextField.setLayoutData(data);
    	ruleDescriptionTextField.setEditable(true);
    	ruleDescriptionTextField.setText((rule != null ? rule.getDescription() : ""));
    	ruleDescriptionTextField.setLayoutData(data);
    	
    	//
    	// -- VALIDITY DATE
    	//
    	
    	startDateLabel = new CLabel(aComposite, SWT.RIGHT);
    	startDateLabel.setText(" Start Date ");
    	
    	startDateTextArea = new Text(aComposite, SWT.BORDER | SWT.FLAT );
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.horizontalSpan = 5;
    	startDateTextArea.setEditable(false);
    	//startDateTextArea.setText(rule != null ? rule.getStartDate().toString() : "");
    	startDateTextArea.setText("");
    	startDateTextArea.setLayoutData(data);
    	
    	
//    	 Gestione della data
    	final Button bChangeDateStartDate = new Button(aComposite, SWT.PUSH);
    	bChangeDateStartDate.setText("Change");
        final DetailRuleDialog theDialog = this;
        bChangeDateStartDate.addListener(SWT.Selection, new Listener() {

              public void handleEvent(Event event) {
            	  
            	  Point parentLocation = theDialog.getShell().getLocation();
            	  Point buttonLocation = bChangeDateStartDate.getLocation();
            	  
            	  Point dialogCalendarLocation  = new Point(parentLocation.x+ buttonLocation.x+100, parentLocation.y + buttonLocation.y+90);
            	  
                  final SWTCalendarDialog cal = new SWTCalendarDialog(Display.getCurrent(), dialogCalendarLocation);

            	  //final CalendarDialog cal = new CalendarDialog(getShell());
                  
            	  cal.addDateChangedListener(new SWTCalendarListener() {

                      public void dateChanged(SWTCalendarEvent calendarEvent) {

                    	  startDateTextArea.setText(formatter.format(calendarEvent.getCalendar().getTime()));
                         
                      }

                  });
                

                  if (startDateTextArea.getText() != null && startDateTextArea.getText().length() > 0) {

                      try {

                          Date d = formatter.parse(startDateTextArea.getText());

                          cal.setDate(d);

                      } catch (ParseException pe) {
                      }

                  }

                  cal.open();
              }

        });
        data = new GridData(GridData.FILL_HORIZONTAL);
    	data.horizontalSpan = 2;
    	bChangeDateStartDate.setLayoutData(data);
    	
    	final Button bClearStartDate = new Button(aComposite, SWT.PUSH);
    	bClearStartDate.setText("Clear");
    	bClearStartDate.addSelectionListener(new SelectionListener(){

			public void widgetDefaultSelected(SelectionEvent e) {
				startDateTextArea.setText("");
			}

			public void widgetSelected(SelectionEvent e) {
				startDateTextArea.setText("");
				
			}});
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.horizontalSpan = 2;
    	bClearStartDate.setLayoutData(data);
    	
    	endDateLabel = new CLabel(aComposite, SWT.RIGHT);
    	endDateLabel.setText(" End Date ");
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.horizontalSpan = 5;
    	endDateTextArea = new Text(aComposite, SWT.BORDER | SWT.FLAT );
    	endDateTextArea.setLayoutData(data);
    	endDateTextArea.setEditable(false);
    	endDateTextArea.setText("");
    	endDateTextArea.setLayoutData(data);
    	
    	// 
//   	 Gestione della data
    	final Button bChangeDateEndDate = new Button(aComposite, SWT.PUSH);
    	bChangeDateEndDate.setText("Change");
        
    	bChangeDateEndDate.addListener(SWT.Selection, new Listener() {

              public void handleEvent(Event event) {
            	  
            	  Point parentLocation = theDialog.getShell().getLocation();
            	  Point buttonLocation = bChangeDateEndDate.getLocation();
            	  
            	  Point dialogCalendarLocation  = new Point(parentLocation.x+ buttonLocation.x+100, parentLocation.y + buttonLocation.y+90);
            	  
                  final SWTCalendarDialog cal = new SWTCalendarDialog(Display.getCurrent(), dialogCalendarLocation);

            	  //final CalendarDialog cal = new CalendarDialog(getShell());
                  
            	  cal.addDateChangedListener(new SWTCalendarListener() {

                      public void dateChanged(SWTCalendarEvent calendarEvent) {

                    	  endDateTextArea.setText(formatter.format(calendarEvent.getCalendar().getTime()));
                         
                      }

                  });
                

                  if (endDateTextArea.getText() != null && endDateTextArea.getText().length() > 0) {

                      try {

                          Date d = formatter.parse(endDateTextArea.getText());

                          cal.setDate(d);

                      } catch (ParseException pe) {
                      }

                  }

                  cal.open();
              }

        });
        data = new GridData(GridData.FILL_HORIZONTAL);
    	data.horizontalSpan = 2;
    	bChangeDateEndDate.setLayoutData(data);
    	
    	final Button bClearEndDate = new Button(aComposite, SWT.PUSH);
    	bClearEndDate.setText("Clear");
    	bClearEndDate.addSelectionListener(new SelectionListener(){

			public void widgetDefaultSelected(SelectionEvent e) {
				endDateTextArea.setText("");
			}

			public void widgetSelected(SelectionEvent e) {
				endDateTextArea.setText("");
				
			}});
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.horizontalSpan = 2;
    	bClearEndDate.setLayoutData(data);
    	//
    	
    	
    	applyGroup = new Group(aComposite, SWT.SHADOW_ETCHED_IN);
    	applyGroup.setText(" Apply To Message "); 
    	applyToIncomingMessage = new Button(applyGroup, SWT.RADIO);
    	applyToIncomingMessage.setText("Incoming ");
    	applyToIncomingMessage.setBounds(5, 10, 70, 30);
    	applyToOutcomingMessage = new Button(applyGroup, SWT.RADIO);
    	applyToOutcomingMessage.setText("Outcoming");
    	applyToOutcomingMessage.setBounds(100, 10, 70, 30);
    	
    	applyToBothMessage = new Button(applyGroup, SWT.RADIO);
    	applyToBothMessage.setText("Both");
    	applyToBothMessage.setBounds(195, 10, 70, 30);
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.horizontalSpan = 10;
    	data.heightHint = 30;
    	applyGroup.setLayoutData(data);
    	
    	
    	ruleRequireConditionGroup = new Group(aComposite, SWT.SHADOW_ETCHED_IN);
    	ruleRequireConditionGroup.setText(" Precondition For Rule "); 
    	ruleSubjectToConditionYesButton = new Button(ruleRequireConditionGroup, SWT.RADIO);
    	ruleSubjectToConditionYesButton.setText("Yes");
    	ruleSubjectToConditionYesButton.setBounds(5, 10, 70, 30);
    	ruleSubjectToConditionNoButton = new Button(ruleRequireConditionGroup, SWT.RADIO);
    	ruleSubjectToConditionNoButton.setText("No");
    	ruleSubjectToConditionNoButton.setBounds(100, 10, 70, 30);
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.horizontalSpan = 10;
    	data.heightHint = 30;
    	ruleRequireConditionGroup.setLayoutData(data);
    	
    	ruleSubjectToConditionYesButton.addSelectionListener(new SelectionListener(){

			public void widgetDefaultSelected(SelectionEvent e) {
				 conditionTextArea.setEditable(true);
				
			}

			public void widgetSelected(SelectionEvent e) {
				conditionTextArea.setEditable(true);
				
			}
    		
    	});
    	
    	ruleSubjectToConditionNoButton.addSelectionListener(new SelectionListener(){

			public void widgetDefaultSelected(SelectionEvent e) {
				 conditionTextArea.setEditable(false);
				
			}

			public void widgetSelected(SelectionEvent e) {
				conditionTextArea.setEditable(false);
				
			}
    		
    	});
    	
    	//
    	// -- CONDITION COMPOSITE
    	//
    	Composite conditionComposite = new Composite(internalSash, SWT.NONE);
    	layout.marginWidth = 15;
    	layout.marginHeight = 10;
    	layout.numColumns = 10;
    	conditionComposite.setLayout(layout);
    	
    	
    	conditionLabel = new CLabel(conditionComposite, SWT.CENTER);
    	conditionLabel.setText(" Rule Precondition (XPath) ");
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.widthHint = 200;
    	data.horizontalSpan = 10;
    	conditionLabel.setLayoutData(data);
    	
    	conditionTextArea = new Text(conditionComposite, SWT.MULTI| SWT.WRAP | SWT.BORDER);
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.widthHint = 200;
    	data.heightHint = 50;
    	data.horizontalSpan = 10;
    	conditionTextArea.setLayoutData(data);

    	
    	Composite ruleExpressionComposite = new Composite(internalSash, SWT.NONE);
    	layout.marginWidth = 15;
    	layout.marginHeight = 10;
    	layout.numColumns = 10;
    	ruleExpressionComposite.setLayout(layout);
    	//  	 Rule Expression
    	ruleExpressionLabel = new CLabel(ruleExpressionComposite, SWT.CENTER);
    	ruleExpressionLabel.setText(" Rule Expression (XPath) ");
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.widthHint = 200;
    	data.horizontalSpan = 10;
    	ruleExpressionLabel.setLayoutData(data);
    	
    	
    	ruleExpressionTextArea = new Text(ruleExpressionComposite, SWT.MULTI| SWT.WRAP | SWT.BORDER);
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.widthHint = 200;
    	data.heightHint = 50;
    	data.horizontalSpan = 10;
    	ruleExpressionTextArea.setLayoutData(data);
    	
    	destinationAttributeLabel = new CLabel(ruleExpressionComposite, SWT.RIGHT);
    	destinationAttributeLabel.setText(" Dest. Attribute");
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.widthHint = 200;
    	data.horizontalSpan = 7;
    	destinationAttributeTextField = new Text(ruleExpressionComposite, SWT.BORDER | SWT.FLAT );
    	destinationAttributeTextField.setLayoutData(data);
    	destinationAttributeTextField.setEditable(true);
    	
    	destinationAttributeTextField.setText("");
    	destinationAttributeTextField.setLayoutData(data);
    	
    	final Button bChooseAttribute = new Button(ruleExpressionComposite, SWT.PUSH);
    	bChooseAttribute.setText(" Choose Existing ");
    	bChooseAttribute.addSelectionListener(new SelectionListener(){

			public void widgetDefaultSelected(SelectionEvent e) {
				chooseAttributesDialog();
			}

			public void widgetSelected(SelectionEvent e) {
				chooseAttributesDialog();
			}
			
			public void chooseAttributesDialog(){
				ListAttributesDialog listAttributesDialog = new ListAttributesDialog(getShell(), metadb, "Choose the destination Attribute for Rule");
				listAttributesDialog.open();
				Attribute selectedAttribute = listAttributesDialog.getSelectedAttribute();
				if (selectedAttribute != null){
					destinationAttributeTextField.setText(selectedAttribute.getName());
				}
			}
    	});
    	
    	if (rule != null){
    		bChooseAttribute.setEnabled(false);
    	}
    	
    		
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.horizontalSpan = 2;
    	bClearEndDate.setLayoutData(data);
    	 // ---------------------------------------
    	
    	if (rule == null){
    		
    		ruleSubjectToConditionYesButton.setSelection(true);
    		applyToIncomingMessage.setSelection(true);
    		destinationAttributeTextField.setEditable(true);	
    		
    	}else{
    		ruleDescriptionTextField.setText(rule.getDescription());
    		startDateTextArea.setText(rule.getStartDate() != null ? rule.getStartDate().toString() : "");
    		endDateTextArea.setText(rule.getEndDate() != null ? rule.getStartDate().toString() : "");
    		
    		Integer ruleMep = rule.getMepState();
    		if (ruleMep == MEP_APPLY_IN)
    			applyToIncomingMessage.setSelection(true);
    		if (ruleMep == MEP_APPLY_OUT)
    			applyToOutcomingMessage.setSelection(true);
    		if (ruleMep == MEP_APPLY_IN_OUT)
    			applyToBothMessage.setSelection(true);
    		
    		if (rule.getConditionExpr() != null){
    			ruleSubjectToConditionYesButton.setSelection(true);
    			conditionTextArea.setText(rule.getConditionExpr());
    		}else{
    			ruleSubjectToConditionNoButton.setSelection(true);
    			conditionTextArea.setText("");
    			conditionTextArea.setEditable(false);
    		}
    		
    		if (rule.getExpr() != null){
    			ruleExpressionTextArea.setText(rule.getExpr());
    		}
    		
    		if (rule.getAttribute() != null){
    			metadb.open();
    			Rule reReadRule = (Rule)metadb.getObjectById(Rule.class, rule.getIdRule());
    			Attribute attr = reReadRule.getAttribute();
    			String destAttr = attr.getName();
    			metadb.close();
    			destinationAttributeTextField.setText(destAttr);
    			destinationAttributeTextField.setEditable(false);
    		}
    		
    		
    		
    	}
    	
    	
    	// Weight Settings on Sash
    	internalSash.setWeights(new int[] {30, 20, 25});
        return parent; 
	
	}
	
	protected void okPressed() {
	
		if (rule == null){
			newRule();
		}else{
			updateRule();
		}
	}
	
	
	public void updateRule(){
		String ruleDescriptionText = ruleDescriptionTextField.getText();
		
		if (ruleDescriptionText.trim().length() == 0){
			MessageDialog.openError(
					getShell(),
					"CimeroEditor Plug-in",
					"The rule description is required");
			return;
		}
		String ruleExprText = ruleExpressionTextArea.getText().trim();
		if (ruleExprText.length() == 0){
			MessageDialog.openError(
					getShell(),
					"CimeroEditor Plug-in",
					"The rule description is required");
			return;
		}
		String conditionText = conditionTextArea.getText();
		if (ruleSubjectToConditionYesButton.getSelection() && conditionText.trim().length() == 0){
			MessageDialog.openError(
					getShell(),
					"CimeroEditor Plug-in",
					"If the rule require a precondition the relative field must be not empty");
			return;
		}
		
		
		
		Integer mepState = null;
		if (applyToIncomingMessage.getSelection()){
			mepState = MEP_APPLY_IN;
		}
		if (applyToOutcomingMessage.getSelection()){
			mepState = MEP_APPLY_OUT;
		}
		if (applyToBothMessage.getSelection()){
			mepState = MEP_APPLY_IN_OUT;
		}
		
		try{
			metadb.openTx();
		
			Rule aRule = (Rule)metadb.getObjectById(Rule.class, rule.getIdRule());
			aRule.setDescription(ruleDescriptionText);
			
			aRule.setMepState(mepState);
		
			if (ruleSubjectToConditionYesButton.getSelection())
				aRule.setConditionExpr(conditionText);
			else
				aRule.setConditionExpr(null);
			
			aRule.setExpr(ruleExprText);
			metadb.storeRule(aRule);
		
			metadb.commitAndClose();
			this.rule = aRule;
			close();
		}catch (Throwable e) {
			e.printStackTrace();
			metadb.rollBackAndClose();
			MessageDialog.openError(
					getShell(),
					"CimeroEditor Plug-in",
					"An error is occurred ["+e.getMessage()+"]");
			return;
		}
		
	}
	public void newRule(){
		String ruleDescriptionText = ruleDescriptionTextField.getText();
		
		if (ruleDescriptionText.trim().length() == 0){
			DialogUtils.error(
					getShell(),
					"The rule description is required");
			return;
		}
		String ruleExprText = ruleExpressionTextArea.getText().trim();
		if (ruleExprText.length() == 0){
			DialogUtils.error(
					getShell(),
					"The rule description is required");
			return;
		}
		String conditionText = conditionTextArea.getText();
		if (ruleSubjectToConditionYesButton.getSelection() && conditionText.trim().length() == 0){
			DialogUtils.error(
					getShell(),
					"If the rule require a precondition the relative field must be not empty");
			return;
		}
		String destAttributeName = destinationAttributeTextField.getText().trim();
		if (destAttributeName.length() == 0){
			DialogUtils.error(
					getShell(),
					"If the rule produce an attribute the destination attribute cannot be null");
			return;
		}
		
		
		Integer mepState = null;
		if (applyToIncomingMessage.getSelection()){
			mepState = MEP_APPLY_IN;
		}
		if (applyToOutcomingMessage.getSelection()){
			mepState = MEP_APPLY_OUT;
		}
		if (applyToBothMessage.getSelection()){
			mepState = MEP_APPLY_IN_OUT;
		}
		Integer usage  = USAGE_EXTRACT_DATA;
		
		
		try{
			metadb.openTx();
		
			Rule aRule = new Rule();
			aRule.setDescription(ruleDescriptionText);
			aRule.setRuleUsage(usage);
			aRule.setMepState(mepState);
		
			if (ruleSubjectToConditionYesButton.getSelection())
				aRule.setConditionExpr(conditionText);
			else
				aRule.setConditionExpr(null);
			
			Attribute anAttribute = getAttributeOrCreateNew(destAttributeName,metadb);
			
			aRule.setAttribute(anAttribute);
			
			aRule.setExpr(ruleExprText);
			metadb.storeRule(aRule);
		
			if (endpoint != null){
				metadb.linkRuleToEndpoint(aRule, endpoint);
			}
			metadb.commitAndClose();
			close();
		}catch (Throwable e) {
			e.printStackTrace();
			metadb.rollBackAndClose();
			DialogUtils.error(getShell(), 
					"An error is occurred ["+e.getMessage()+"]");
			return;
		}
		
	}
	
	@Override
	protected void buttonPressed(int buttonId) {
		if (buttonId == IDialogConstants.CANCEL_ID){
			close();
		}
		super.buttonPressed(buttonId);
	}



	public Rule getRule() {
		return rule;
	}



	public void setRule(Rule rule) {
		this.rule = rule;
	}
		
	public Attribute getAttributeOrCreateNew(String attributeName, IMetadbApi metadb){
		Attribute anAttribute = metadb.getAttributeByName(attributeName);
		if (anAttribute == null){
			anAttribute = new Attribute();
			
			anAttribute.setName(attributeName);
			anAttribute.setType(0);
			
			anAttribute = metadb.storeAttribute(anAttribute);
		}
		return anAttribute;
	}
	
	
	

	
	
	
	
	

}
