package it.eng.cimero.plugin.dialogs;

import it.eng.cimero.plugin.preferences.PreferenceConstants;
import it.eng.cimero.plugin.providers.DataSourceBeanContentProvider;
import it.eng.cimero.plugin.providers.DataSourceBeanLabelProvider;
import it.eng.utils.DialogUtils;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.apache.commons.dbcp.BasicDataSource;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.RequestEntity;
import org.apache.commons.httpclient.methods.StringRequestEntity;
import org.dom4j.Branch;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.Node;
import org.dom4j.XPath;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.SAXReader;
import org.dom4j.io.XMLWriter;
import org.dom4j.xpath.DefaultXPath;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

import cimeroEditor.CimeroEditorPlugin;

import com.bull.cimero.pluginEditor.utils.log.CimeroLog;

public class ListDatasourcesDialog extends TitleAreaDialog {
	
    private Composite aComposite = null;
	
    private SashForm internalSash  = null; 
    
    private Table tableDatasources = null;
    private TableViewer tbvDatasources = null; 
    private List<DataSourceBeanDialog> theDatasources = null;
    private Document dsDocument = null;
    private String fullPath = null;
    private HashMap<String, String>smxJndiNamespaces = null;
		
	public ListDatasourcesDialog(Shell parentShell) {
		
		super(parentShell);
		initDataSources();
		initNamespacesJndi();
		this.theDatasources = new ArrayList<DataSourceBeanDialog>();
		

	}
	
	private void initNamespacesJndi(){
		this.smxJndiNamespaces = new HashMap<String, String>();
		this.smxJndiNamespaces.put("jencks","http://jencks.org/2.0"); 
		this.smxJndiNamespaces.put("amq","http://activemq.org/config/1.0");
		this.smxJndiNamespaces.put("amqra","http://activemq.org/ra/1.0");
		this.smxJndiNamespaces.put("util","http://www.springframework.org/schema/util");
		this.smxJndiNamespaces.put("soapenv","http://schemas.xmlsoap.org/soap/envelope/");
		this.smxJndiNamespaces.put("xsd","http://www.w3.org/2001/XMLSchema"); 
		this.smxJndiNamespaces.put("xsi","http://www.w3.org/2001/XMLSchema-instance");
	}
				

	/**
	   * @see org.eclipse.jface.window.Window#create() We complete the dialog with
	   *      a title and a message
	   */
	  public void create() {
		super.create();
		setTitle("Spagic DataSources ");
	    setMessage("Spagic DataSources ");
	   
	  }

	 
	  

	@Override
	protected Control createContents(Composite parent) {
		Control ctrl = super.createContents(parent);
		return ctrl;
	}

	protected Control createDialogArea(Composite parent) {
		
        this.internalSash = new SashForm(parent,  SWT.VERTICAL | SWT.FLAT);
        this.internalSash.setLayoutData(new GridData(GridData.FILL_BOTH));
       	
    	final ListDatasourcesDialog theDialog = this;
    	
    	this.theDatasources = getDataSources(); 
    	
        this.tableDatasources = new Table(internalSash, SWT.MULTI | SWT.FULL_SELECTION | SWT.BORDER);
        this.tableDatasources.setLinesVisible(true);
        this.tableDatasources.setHeaderVisible(true);
        TableColumn column = null;
		column = new TableColumn(tableDatasources, SWT.LEFT);
		column.setText("Name");
		column.setWidth(200);
		column = new TableColumn(tableDatasources, SWT.LEFT);
		column.setText("Driver");
		column.setWidth(200);
		column = new TableColumn(tableDatasources, SWT.LEFT);
		column.setText("url");
		column.setWidth(200);
		column = new TableColumn(tableDatasources, SWT.LEFT);
		column.setText("user");
		column.setWidth(200);
        this.tbvDatasources = new TableViewer(tableDatasources);
        tbvDatasources.setContentProvider(new DataSourceBeanContentProvider());
        tbvDatasources.setLabelProvider(new DataSourceBeanLabelProvider());
        tbvDatasources.setInput(this.theDatasources);    
    	
        this.tableDatasources.addKeyListener(new KeyListener(){
        	public void keyPressed(org.eclipse.swt.events.KeyEvent e) {
        		//System.out.println("KeyCode =" +  e.keyCode);
        		if (e.keyCode == 127){
        			TableItem[] tis = theDialog.tableDatasources.getSelection();
        			DataSourceBeanDialog selected = (DataSourceBeanDialog)tis[0].getData();
        			boolean confirm = MessageDialog.openConfirm(
    						getShell(),
    						"Spagic Studio",
    						"This will remove datasource "+ selected.getName()+ "from spagic studio are you sure ? ");
        			if (confirm){
        				deleteDatasources(selected);
        				theDialog.tbvDatasources.setInput(getDataSources());
        				theDialog.tbvDatasources.refresh();
        			}
        			
        		}
        	};
        	public void keyReleased(org.eclipse.swt.events.KeyEvent e) {
        		
        	};
        	
        });
       
       
        
        aComposite = new Composite(internalSash, SWT.NONE);
		final GridLayout layout = new GridLayout();
    	layout.marginWidth = 15;
    	layout.marginHeight = 10;
    	layout.numColumns = 12;
    	aComposite.setLayout(layout);
    	aComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    	
    	String imgAddDatasourcePath = "icons" + File.separator + "database_add.png";
    	Image addDatasorceImage = CimeroEditorPlugin.getImageDescriptor(
    			imgAddDatasourcePath).createImage();
    	
    	String imgPathImpSmx = "icons" + File.separator + "database_imp_smx.png";
    	Image impSmxImage = CimeroEditorPlugin.getImageDescriptor(
    			imgPathImpSmx).createImage();
    	
    	String imgPathImpFromFile = "icons" + File.separator + "database_imp_file.png";
    	Image impFromFileImage = CimeroEditorPlugin.getImageDescriptor(
    			imgPathImpFromFile).createImage();
    	
    	String imgPathExportJNDI = "icons" + File.separator + "database_export.png";
    	Image exportJNDIImage = CimeroEditorPlugin.getImageDescriptor(
    			imgPathExportJNDI).createImage();
    	
    	Button btnAddSpagicDatasource = new Button(aComposite, SWT.FLAT);
    	btnAddSpagicDatasource.setImage(addDatasorceImage);
    	btnAddSpagicDatasource.setText(" Add Spagic Datasource ");
    	GridData data = new GridData(GridData.FILL_HORIZONTAL);
    	data.widthHint = 200;
    	data.horizontalSpan = 3;
    	btnAddSpagicDatasource.setLayoutData(data);
    	
    	Button btnImportFromSMX = new Button(aComposite, SWT.FLAT);
    	btnImportFromSMX.setImage(impSmxImage);
    	btnImportFromSMX.setText(" Import from ServiceMix ");
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.widthHint = 200;
    	data.horizontalSpan = 3;
    	btnImportFromSMX.setLayoutData(data);
    	
    	Button btnImportFromJndiXmlFile = new Button(aComposite, SWT.FLAT);
    	btnImportFromJndiXmlFile.setImage(impFromFileImage);
    	btnImportFromJndiXmlFile.setText(" Import from xml file ");
    	
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.widthHint = 200;
    	data.horizontalSpan = 3;
    	btnImportFromJndiXmlFile.setLayoutData(data);
    	
    	
    	Button btnExportJndiXmlFile = new Button(aComposite, SWT.FLAT);
    	btnExportJndiXmlFile.setImage(exportJNDIImage);
    	btnExportJndiXmlFile.setText(" Generate xml for jndi.xml ");
    	
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.widthHint = 200;
    	data.horizontalSpan = 3;
    	btnExportJndiXmlFile.setLayoutData(data);
    	
    	
    	btnExportJndiXmlFile.addListener(SWT.Selection,new Listener() {
        	public void handleEvent(Event event) {
        		ViewJndiXMLDialog viewJndiXMLDialog = new ViewJndiXMLDialog(getShell(), getJNDIFragment());
        		viewJndiXMLDialog.open();
        	}
    	});
    	
    	btnAddSpagicDatasource.addListener(SWT.Selection,new Listener() {
        	public void handleEvent(Event event) {
        		DetailDatasourceDialog dsDialog = new DetailDatasourceDialog(getShell());
        		dsDialog.open();
        		DataSourceBeanDialog dsBean = dsDialog.getDatasourceBean();
        		if (dsBean != null){
        			addDataSources(dsBean);
        			theDialog.tbvDatasources.setInput(getDataSources());
    				theDialog.tbvDatasources.refresh();
        		}
        	}
        });
        
    	btnImportFromSMX.addListener(SWT.Selection, 
    		new Listener() {
            	public void handleEvent(Event event) {
            		try{
            			List<DataSourceBeanDialog> dsListFromSmx = getDataSourcesListFromRunningSmx();

                		if (dsListFromSmx == null || dsListFromSmx.size() == 0){
                			DialogUtils.warning(getShell(), " Cannot get Datasources from runnig ServiceMix, Ensure that is running and you've getResources SA deployed ");
                			return;
                		}
                		
            			SelectDataSourcesDialog selectionDsDialog = new SelectDataSourcesDialog(getShell(), "Datasources List from ServiceMix",dsListFromSmx);
            			selectionDsDialog.open();
            			List<DataSourceBeanDialog> selectedFromJNDI = selectionDsDialog.getSelectedDatasources();
                		if (selectedFromJNDI != null){
                			
                			for (DataSourceBeanDialog dsInJndi : selectedFromJNDI){
                				if (exist(dsInJndi.getName())){
                					if (DialogUtils.confirm(getShell(), "Datasource ["+dsInJndi.getName()+"] is already defined in Spagic Studio, Update with configurations found in running smx ?")){
                						deleteDatasources(dsInJndi);
                						addDataSources(dsInJndi);
                					}
                				}else{
                					theDialog.addDataSources(dsInJndi);
                				}
                			}
                			theDialog.tbvDatasources.setInput(getDataSources());
            				theDialog.tbvDatasources.refresh();
                		}
            		}catch (Exception e) {
						DialogUtils.error(getShell(),"Cannot contact smx server to get datasourceList");
					}
            	}
            });
    	
    	btnImportFromJndiXmlFile.addListener(SWT.Selection, 
    		new Listener() {
    		public void handleEvent(Event event) {
        		String fileName = openFileDialog();
        		List<DataSourceBeanDialog> dsListFromJNDIFile = getDataSourcesListFromJndiXMLFile(fileName);
        		
        		if (dsListFromJNDIFile == null || dsListFromJNDIFile.size() == 0){
        			DialogUtils.warning(getShell(), " Cannot get Datasources from ["+fileName+"] ensure this is the jndi.xml file of servicemix");
        			return;
        		}
        		
        		SelectDataSourcesDialog selectionDsDialog = new SelectDataSourcesDialog(getShell(), "Datasources JNDI File ",dsListFromJNDIFile);
        		
        		selectionDsDialog.open();
        		List<DataSourceBeanDialog> selectedFromJNDI = selectionDsDialog.getSelectedDatasources();
        		if (selectedFromJNDI != null){
        			
        			for (DataSourceBeanDialog dsInJndi : selectedFromJNDI){
        				if (exist(dsInJndi.getName())){
        					if (DialogUtils.confirm(getShell(), "Datasource ["+dsInJndi.getName()+"] is already defined in Spagic Studio, Update with configurations found in jndi file ?")){
        						deleteDatasources(dsInJndi);
        						addDataSources(dsInJndi);
        					}
        				}else{
        					theDialog.addDataSources(dsInJndi);
        				}
        			}
        			theDialog.tbvDatasources.setInput(getDataSources());
    				theDialog.tbvDatasources.refresh();
        		}
        	}	
        });
    	
    	
    	internalSash.setWeights(new int[] {60,40});
        return parent;
	}
	
	protected void okPressed() {
		super.okPressed();
		
	}
	



	protected void createButtonsForButtonBar(Composite parent) {
		
		
	}
	protected String getJNDIFragment(){
		StringBuffer sBuffer = new StringBuffer();
		List<DataSourceBeanDialog> datasorces = getDataSources();
		for (DataSourceBeanDialog ds : datasorces){
			sBuffer.append(ds.toJndiConfiguration());
			sBuffer.append("\n");
			sBuffer.append("\n");
		}
		return sBuffer.toString();
	}
	
	protected List<DataSourceBeanDialog> getDataSources(){
		List<DataSourceBeanDialog> listDsBean = new ArrayList<DataSourceBeanDialog>();
		List <Node> dsNodes = dsDocument.selectNodes("/DATASOURCES/DATASOURCE");
		DataSourceBeanDialog dsBean = null;
		for (Node ds : dsNodes){
			dsBean = new DataSourceBeanDialog(ds.valueOf("@id"),
									ds.valueOf("@jdbcDriver"),
									ds.valueOf("@jdbcUrl"),
									ds.valueOf("@jdbcUser"),
									ds.valueOf("@jdbcPassword"));
			listDsBean.add(dsBean);
		}
		return listDsBean;
	}
	
	protected void  initDataSources(){	
		fullPath = CimeroEditorPlugin.getRessourcePath("/ressources/DataSources.xml");
		if (fullPath != null) {
			if (fullPath.substring(0, "file:".length()).compareTo("file:") == 0) {
				fullPath = fullPath.substring("file:".length());
			}
		}
		try{
			SAXReader reader = new SAXReader();
			dsDocument = reader.read(fullPath);
		}catch (Exception e) {
			CimeroLog.errorLog("Error in parsing datasource file ", e);
		}
	}
	
	protected void addDataSources(DataSourceBeanDialog dsBean){
		
			try{
				Node datasourcesNode = dsDocument.selectSingleNode("/DATASOURCES");
		
				((Element)datasourcesNode).addElement("DATASOURCE" )
				.addAttribute( "id", dsBean.getName())
				.addAttribute( "jndiName", "java:comp/env/jdbc/" + dsBean.getName() )
				.addAttribute( "jdbcDriver", dsBean.getDriverClassName() )
				.addAttribute( "jdbcUrl", dsBean.getJdbcUrl() )
				.addAttribute( "jdbcUser",dsBean.getUser() )
				.addAttribute( "jdbcPassword", dsBean.getPassword());
        
		
				serializeDatasources();
			
			
			}catch(Exception e){
				CimeroLog.errorLog("Error in writing ds File", e);
			}
		
	
	}
	
	protected void deleteDatasources(DataSourceBeanDialog dsSelected){
		
		try{
			Node datasourcesNode = dsDocument.selectSingleNode("/DATASOURCES");
		
			Node selectedDsNode = dsDocument.selectSingleNode("/DATASOURCES/DATASOURCE[@id='"+dsSelected.getName()+"']");
			
			if (selectedDsNode != null){
				((Branch)datasourcesNode).remove(selectedDsNode);
			}
			serializeDatasources();
			
		}catch(Exception e){
			CimeroLog.errorLog("Error in writing ds File", e);
		}
	}
	
	protected void serializeDatasources(){
		try{
		//	 Pretty print the document to System.out
			OutputFormat format = OutputFormat.createPrettyPrint();
			XMLWriter writer = new XMLWriter(  new FileWriter( fullPath ), format );
			writer.write( dsDocument );
			writer.flush();
			writer.close();
		}catch(Exception e){
			CimeroLog.errorLog("Error in writing ds File", e);
		}
	}
	
	protected boolean exist(String dsName) {
		Node selectedDsNode = dsDocument.selectSingleNode("/DATASOURCES/DATASOURCE[@id='"+dsName+"']");
		return selectedDsNode != null;
	}		
	
	@Override
	public boolean close() {
		
		this.dsDocument = null;
		this.fullPath  = null;
		return super.close();
	}


	protected List<DataSourceBeanDialog> getDataSourcesListFromRunningSmx() {
		List<DataSourceBeanDialog> listDsBean = new ArrayList<DataSourceBeanDialog>();
		String strURL = CimeroEditorPlugin.getDefault().getPluginPreferences().getString(PreferenceConstants.P_SMX_GET_RESOURCES);
		//strURL = "http://localhost:8080/getResources/";
		PostMethod post = new PostMethod(strURL);
		String soapBodyContent = "<REQUESTED_RESOURCES><ITEM id='DATASOURCES'/></REQUESTED_RESOURCES>";
		
        RequestEntity entity = new StringRequestEntity(getSoapRequestForXmlString(soapBodyContent));
        post.setRequestEntity(entity);
        // consult documentation for your web service
        post.setRequestHeader("SOAPAction", "");
        // Get HTTP client
        HttpClient httpclient = new HttpClient();
        // Execute request
        try {
            int result = httpclient.executeMethod(post);
            // Display status code
            System.out.println("Response status code: " + result);
            // Display response
            System.out.println("Response body: ");
            String soapResponse = post.getResponseBodyAsString();
            
            Document dom = DocumentHelper.parseText(soapResponse);
            
            XPath xPathExpression = new DefaultXPath("/soapenv:Envelope/soapenv:Body/DATASOURCES/DATASOURCE");
			
			xPathExpression.setNamespaceURIs(this.smxJndiNamespaces);
			
			List<Node> dsNodes = xPathExpression.selectNodes(dom);
           
            
            DataSourceBeanDialog dsBeanDialog = null;
          
            for (Node dsNode : dsNodes){
            	dsBeanDialog = new DataSourceBeanDialog(dsNode.valueOf("@name"), dsNode.valueOf("@driver"),dsNode.valueOf("@url") , dsNode.valueOf("@user") , dsNode.valueOf("@password") );
				listDsBean.add(dsBeanDialog);
            	  
            }
        }catch (Exception e) {
			CimeroLog.errorLog("Exception", e);
        } finally {
            // Release current connection to the connection pool once you are done
            post.releaseConnection();
        }
        return listDsBean;
	}
	public static String getSoapRequestForXmlString(String xmlString){
		StringBuffer sb = new StringBuffer();
		sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
		sb.append("<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">");
		sb.append("<soapenv:Body>");
		sb.append(xmlString);
		sb.append("</soapenv:Body>");
		sb.append("</soapenv:Envelope>");
		return sb.toString();
	}

	protected String openFileDialog(){
		 FileDialog fd = new FileDialog(this.getShell(), SWT.OPEN);
	     fd.setText("Open a valid jndi.xml servicemix file ");
	     
	     String[] filterExt = { "*.xml" };
	     fd.setFilterExtensions(filterExt);
	     String selected = fd.open();
	     System.out.println(selected);
	     return selected;
	}

	protected List<DataSourceBeanDialog> getDataSourcesListFromJndiXMLFile(String jndiFileName) {
		List<DataSourceBeanDialog> listDsBean = new ArrayList<DataSourceBeanDialog>();	
		try{
			SAXReader reader = new SAXReader(false);
			Document jndiDocument = reader.read(jndiFileName);
			
//			<bean id="mysql-ds" 
//                class="org.apache.commons.dbcp.BasicDataSource" 
//                destroy-method="close"> 
//                <property name="driverClassName" value="com.mysql.jdbc.Driver"/> 
//                <property name="url" value="jdbc:mysql://athos:3306/smx2"/> 
//                <property name="username" value="smx2"/> 
//                <property name="password" value="smx2"/> 
//          </bean> 
			
			XPath xPathExpression = new DefaultXPath("/beans/util:map[@id='jndiEntries']/entry");
			
			xPathExpression.setNamespaceURIs(this.smxJndiNamespaces);
			List<Node> entryNodes = xPathExpression.selectNodes(jndiDocument);
			
			Node propNode = null;
			for (Node en : entryNodes){
				String nodeKey = en.valueOf("@key");
				System.out.println("entry" + nodeKey);
				if (nodeKey.startsWith("java:comp/env/jdbc/")){
					//System.out.println("entry" + nodeKey +" IS A DS");
					String name = nodeKey.substring("java:comp/env/jdbc/".length());
//					<property name="driverClassName" value="com.mysql.jdbc.Driver"/> 
//	                  <property name="url" value="jdbc:mysql://athos:3306/smx2"/> 
//	                  <property name="username" value="smx2"/> 
//	                  <property name="password" value="smx2"/> 
					propNode = en.selectSingleNode("bean/property[@name='driverClassName']");
					String driverClassName = propNode.valueOf("@value");
					propNode = en.selectSingleNode("bean/property[@name='url']");
					String jdbcUrl = propNode.valueOf("@value");
					propNode = en.selectSingleNode("bean/property[@name='username']");
					String user = propNode.valueOf("@value");
					propNode = en.selectSingleNode("bean/property[@name='password']");
					String password = propNode.valueOf("@value");
					
					DataSourceBeanDialog dsbean = new DataSourceBeanDialog(name, driverClassName,jdbcUrl, user, password);
					listDsBean.add(dsbean);
					
					
				}else{
					//System.out.println("entry" + nodeKey + " IS NOT A DS ");
				}
			}
			}catch (Exception e) {
				CimeroLog.errorLog("Error in getting resources from Jndi", e);
			}
			return listDsBean;
		}




	
	
}
