package it.eng.cimero.plugin.dialogs;

import it.eng.cimero.plugin.providers.RulesContentProvider;
import it.eng.cimero.plugin.providers.RulesLabelProvider;
import it.eng.spagosoa.metadb.api.IMetadbApi;
import it.eng.spagosoa.metadb.model.AttributeCatalog;
import it.eng.spagosoa.metadb.model.Endpoint;
import it.eng.spagosoa.metadb.model.IterType;
import it.eng.spagosoa.metadb.model.Rule;
import it.eng.utils.DialogUtils;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

public class ListRulesForEndpointDialog extends TitleAreaDialog {
	
    private Composite aComposite = null;
	
    private SashForm internalSash  = null; 
    
    private Table tableRules = null;
    private TableViewer tbvRules = null; 
    private List<Rule> theRules = null;
    private Endpoint ep = null;
   
    private IMetadbApi metadb = null;
    
	public ListRulesForEndpointDialog(Shell parentShell, Endpoint ep, IMetadbApi metadb) {
		
		super(parentShell);
		this.theRules = new ArrayList<Rule>();
		
		this.ep = ep;
		this.metadb = metadb;
	}
	
	
				

	/**
	   * @see org.eclipse.jface.window.Window#create() We complete the dialog with
	   *      a title and a message
	   */
	  public void create() {
		super.create();
		setTitle("Rules Configuration For ["+ep.getName()+"]");
	    setMessage("Rules Configuration For ["+ep.getName()+"]");
	   
	  }

	 
	  

	@Override
	protected Control createContents(Composite parent) {
		Control ctrl = super.createContents(parent);
		return ctrl;
	}

	protected Control createDialogArea(Composite parent) {
		
        this.internalSash = new SashForm(parent,  SWT.VERTICAL | SWT.FLAT);
        this.internalSash.setLayoutData(new GridData(GridData.FILL_BOTH));
       	
    	final ListRulesForEndpointDialog theDialog = this;
    	
    	this.theRules = getRulesForEndpoint(); 
    	
    	Composite aCompositeTitleTable = new Composite(internalSash, SWT.NONE);
		final GridLayout layout = new GridLayout();
    	layout.marginWidth = 15;
    	layout.marginHeight = 10;
    	
    	layout.numColumns = 10;
    	aCompositeTitleTable.setLayout(layout);
    	
    	aCompositeTitleTable.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    	GridData data = new GridData(GridData.FILL_HORIZONTAL);
    	data.heightHint = 50;
    	data.widthHint = 200;
    	data.horizontalSpan = 10;

    	//    	 Riga 1
    	CLabel labelForTable = new CLabel(aCompositeTitleTable, SWT.CENTER);
    	labelForTable.setText(" Rules Associated to this endpoint ");
    	
    	labelForTable.setLayoutData(data);
    	
        this.tableRules = new Table(internalSash, SWT.MULTI | SWT.FULL_SELECTION | SWT.BORDER);
        this.tableRules.setLinesVisible(true);
        this.tableRules.setHeaderVisible(true);
        TableColumn column = null;
		column = new  TableColumn(tableRules, SWT.LEFT);
		column.setText("Rule Description");
		column.setWidth(450);
		this.tableRules.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
        this.tbvRules = new TableViewer(tableRules);
        tbvRules.setContentProvider(new RulesContentProvider());
        tbvRules.setLabelProvider(new RulesLabelProvider());
        
    	tbvRules.setInput(theRules);
        
        this.tableRules.addKeyListener(new KeyListener(){
        	public void keyPressed(org.eclipse.swt.events.KeyEvent e) {
        		System.out.println("KeyCode =" +  e.keyCode);
        		
        		if (e.keyCode == 127){
        			TableItem[] tis = theDialog.tableRules.getSelection();
        			Rule selectedRules = (Rule)tis[0].getData();
        			
        			boolean confirm = MessageDialog.openConfirm(
    						getShell(),
    						"Spagic Studio",
    						"This will remove the association bewteen " + selectedRules.getDescription() + " and the endpoint are you sure ? ");
        			
        			if (confirm){
        				unlinkRuleToEndpoint(selectedRules, ep);
        				theDialog.tbvRules.setInput(getRulesForEndpoint());
        				theDialog.tbvRules.refresh();
        			}
        			
        		}
        	};
        	public void keyReleased(org.eclipse.swt.events.KeyEvent e) {
        		
        	};
        	
        });
        
        tbvRules.addDoubleClickListener(new IDoubleClickListener(){
			public void doubleClick(DoubleClickEvent event) {
				TableItem[] tis = tableRules.getSelection();
    			if (tis.length > 0){
    				Rule rule = (Rule)tis[0].getData();
    				DetailRuleDialog detailRuleDialog = new DetailRuleDialog(getShell(),rule,metadb);
    				detailRuleDialog.open();
    				theDialog.tbvRules.setInput(getRulesForEndpoint());
    				theDialog.tbvRules.refresh();
    			}
			}});
        
        aComposite = new Composite(internalSash, SWT.NONE);
    	layout.marginWidth = 15;
    	layout.marginHeight = 10;
    	layout.numColumns = 6;
    	aComposite.setLayout(layout);
    	aComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    	
    	
    	
    	
    	
    	Button btnAddRule = new Button(aComposite, SWT.FLAT);
    	btnAddRule.setText(" Create New Rule For Endpoint");
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.widthHint = 200;
    	data.horizontalSpan = 3;
    	btnAddRule.setLayoutData(data);
    	
    	
    	
    	Button btnLinkExisting = new Button(aComposite, SWT.FLAT);
    	btnLinkExisting.setText(" Link Existing Rule To Endpoint ");
    	data = new GridData(GridData.FILL_HORIZONTAL);
    	data.widthHint = 200;
    	data.horizontalSpan = 3;
    	btnLinkExisting.setLayoutData(data);
    	
    	btnLinkExisting.addListener(SWT.Selection, 
        		new Listener() {
                	public void handleEvent(Event event) {
                		
                 		ListRulesDialog listRulesDialog = new ListRulesDialog(getShell(),metadb, true, "Select Rule For ["+ep.getName()+"]");
                 		listRulesDialog.open();
                 		
                 		if (listRulesDialog.getSelectedRule() != null){
                 			linkRuleToEndpoint(listRulesDialog.getSelectedRule() , ep);
                 			theDialog.tbvRules.setInput(getRulesForEndpoint());
                 			theDialog.tbvRules.refresh();
                 		}
                }});
         
    	btnAddRule.addListener(SWT.Selection, 
    		new Listener() {
            	public void handleEvent(Event event) {
            		
             		DetailRuleDialog detailRuleDialog = new DetailRuleDialog(getShell(),null,theDialog.ep,metadb);
             		detailRuleDialog.open();
             		
            		theDialog.tbvRules.setInput(getRulesForEndpoint());
    				theDialog.tbvRules.refresh();
            }});
    	
    	/*
    	btnModifyRule.addListener(SWT.Selection, 
        		new Listener() {
                	public void handleEvent(Event event) {
                		TableItem[] tis = theDialog.tableRules.getSelection();
            			if (tis.length > 0){
            				Rule rule = (Rule)tis[0].getData();
            				DetailRuleDialog detailRuleDialog = new DetailRuleDialog(getShell(),rule,metadb);
            				detailRuleDialog.open();
            				theDialog.tbvRules.setInput(getRulesForEndpoint());
            				theDialog.tbvRules.refresh();
            			}
                }});
        */
    	
    	internalSash.setWeights(new int[] {30,40,30});
        return parent; 
	
	}
	
	protected void okPressed() {
		super.okPressed();
		
	}
	
	@Override
	protected void buttonPressed(int buttonId) {
	
		super.buttonPressed(buttonId);
	}


	protected void createButtonsForButtonBar(Composite parent) {
		// create OK and Cancel buttons by default
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.CLOSE_LABEL,
				true);
		
	}

	public List<Rule> getRulesForEndpoint(){
		metadb.open();
		List<Rule> result =  metadb.getRulesForEndpoint(ep);
		metadb.close();
		return result;
	}	
	
	private void linkRuleToEndpoint(Rule rule, Endpoint ep1){
		try {
			metadb.openTx();
			metadb.linkRuleToEndpoint(rule, ep1);
			metadb.commitAndClose();
		} catch (Throwable t) {
			metadb.rollBackAndClose();
		
		}
	}
	
	private void unlinkRuleToEndpoint(Rule rule, Endpoint ep1){
		try {
			metadb.openTx();
			metadb.unlinkRuleToEndpoint(rule, ep1);
			metadb.commitAndClose();
		} catch (Throwable t) {
			metadb.rollBackAndClose();
		
		}
		
	}
	
	




	
	
}
