package it.eng.components.cimero;

import java.util.Iterator;
import java.util.List;

import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.lWFactory.SE.SELWFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ActivationSpecClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ComponentClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.LWBeanClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.NameSpaceClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.PropertyClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.PropertyClassListBean;

/**
 */
public class GenericJdbcAdvancedQueryFactory extends SELWFactory {

	/**the class using by ServiceMix to the transformer.*/
	public static final String QUERY_CLASS =
		"it.eng.spagosoa.smx.components.jdbcquery.JDBCAdvancedQueryComponent";
	public static final String STORE_PROCEDURE_CLASS =
		"it.eng.spagosoa.smx.components.jdbcstore.JDBCStoreProcedureComponent";
	public static final String CONFIG_CLASS =
		"it.eng.spagosoa.smx.components.jdbcquery.JDBCQueryConfig";
	public static final String CONN_CONFIG_CLASS =
		"it.eng.spagosoa.smx.components.jdbcquery.JDBCConnectionConfig";
		

	/**
	 * default constructor.
	 * @param ref : the reference of the current namespace.
	 */
	public GenericJdbcAdvancedQueryFactory(final NameSpaceClass ref) {
		super(ref);
	}
	
	/**
	 * this function will create an activation spec that represents
	 * a transformeur in serviceMix.
	 * @param elt : the element using to create this activation spec.
	 * @param force : false if the validation must to be do.
	 * @return the activation spec create.
	 * @throws Exception
	 * @throws Exception : if the is a error during generation
	 */
	public final ActivationSpecClass createXMLModel(
			final GeneriqueCimeroModel elt,
			final Boolean force) throws Exception {
		ActivationSpecClass myActi = null;
		//if not null and valide and instance of transformer
		if ((elt != null) && (elt instanceof GenericJdbcAdvancedQuery)) {
			if ((elt.isValide(false)) || (force)) {
				GenericJdbcAdvancedQuery genJdbcAdv = (GenericJdbcAdvancedQuery) elt;
				myActi = new ActivationSpecClass();
				myActi.setNameSpaceRef(getNamespaceRef());
				myActi.setComponentName(genJdbcAdv.getName());
				myActi.setServiceName(genJdbcAdv.getName(),
						new NameSpaceClass(NAMESPACENAME , NAMESPACEVALUE));
				setDestinationService(myActi , elt);
				//add the component
				ComponentClass myCompo = createComponent(genJdbcAdv);
				myActi.setComponent(myCompo);
				return myActi;
			}
		}
		throw new Exception("The generique cimero object is not valide");

	}

	/**
	 * This function return the servicemix component of a Transformeur.
	 * @param monTrfs : the cimero transformeur
	 * @return the servicemix component.
	 */
	private ComponentClass createComponent(final GenericJdbcAdvancedQuery monTrfs) 
		throws Exception {
		//create the componentclass instance
		ComponentClass monCompo = new ComponentClass(getNamespaceRef());
		//add the bean class.
		LWBeanClass monB = createBeanTransformeur(monTrfs);
		monCompo.setMonBean(monB);
		return monCompo;
	}

	
	/**
	 * This function creates the bean using by the transformeur in the
	 * serviceMix model.
	 * @param jdbcAQuery : the cimero element
	 * @return the bean transformer.
	 */
	private LWBeanClass createBeanTransformeur(final GenericJdbcAdvancedQuery jdbcAQuery) {
		LWBeanClass myBean = new LWBeanClass(
			jdbcAQuery.isStoreProcedure()? STORE_PROCEDURE_CLASS : QUERY_CLASS
		);
		
		// query configuration
		PropertyClass config = createProperty("queryConfig", "");
		LWBeanClass configBean = new LWBeanClass(CONFIG_CLASS);

				configBean.addProperty(createProperty("query",
						(String)jdbcAQuery.getPropertyValue(GenericJdbcAdvancedQuery.QUERY)),
					"");
				configBean.addProperty(createProperty("enrichMessage",
						(String)jdbcAQuery.getIsEnrich()),
					"");
				configBean.addProperty(createProperty("xmlEnvelope",
						(String)jdbcAQuery.getPropertyValue(GenericJdbcAdvancedQuery.XML_ENVELOPE)),
					"");
				configBean.addProperty(createProperty("rowsXmlEnvelope",
						(String)jdbcAQuery.getPropertyValue(GenericJdbcAdvancedQuery.ROWS_XML_ENVELOPE)),
					"");
				configBean.addProperty(createProperty("rowXmlEnvelope",
						(String)jdbcAQuery.getPropertyValue(GenericJdbcAdvancedQuery.ROW_XML_ENVELOPE)),
					"");
				configBean.addProperty(createProperty("faultManagement",
						(String)jdbcAQuery.getFaultManagement()),
					"");
				ParameterList queryParamConfig = new ParameterList("queryParams");
				List<String> queryParamNames = jdbcAQuery.getQueryParameterNames();
				for (Iterator<String> iter = queryParamNames.iterator();
					queryParamNames != null && iter.hasNext();) {
					String name = iter.next();
					System.out.println("Adding query param: "+name);
					queryParamConfig.addQueryParamConfig(
						name,
						jdbcAQuery.getParamIsOutput(name),
						jdbcAQuery.getParamXpath(name),
						jdbcAQuery.getParamType(name));
				}
				if (!queryParamNames.isEmpty()) {
					configBean.addProperty(queryParamConfig, "");
				}
				config.setBean(configBean);
				
		// connection config
		PropertyClass connConfig = createProperty("connConfig", "");
		LWBeanClass connConfigBean = new LWBeanClass(CONN_CONFIG_CLASS);
			if (jdbcAQuery.isUseDatasources()) {
				connConfigBean.addProperty(createPropertyRef("datasource" ,
						jdbcAQuery.getDataSource()), "");
			} else {
				connConfigBean.addProperty(createProperty("driver",
						(String)jdbcAQuery.getPropertyValue(GenericJdbcAdvancedQuery.DRIVER)),
					"");
				connConfigBean.addProperty(createProperty("jdbcURL",
						(String)jdbcAQuery.getPropertyValue(GenericJdbcAdvancedQuery.JDBC_URL)),
					"");
				connConfigBean.addProperty(createProperty("userName",
						(String)jdbcAQuery.getPropertyValue(GenericJdbcAdvancedQuery.USER_NAME)),
					"");
				connConfigBean.addProperty(createProperty("userPassword",
						(String)jdbcAQuery.getPropertyValue(GenericJdbcAdvancedQuery.USER_PASSWORD)),
					"");
				connConfigBean.addProperty(createProperty("databaseVendor",
						(String)jdbcAQuery.getDatabaseVendor()),
					"");
			}
			connConfig.setBean(connConfigBean);
		

		myBean.addProperty(connConfig, "");
		myBean.addProperty(config, "");
		return myBean;
	}

}

class ParameterList extends PropertyClass implements PropertyClassListBean {
	public static final String QUERY_PARAM_CONFIG_CLASS = 
		"it.eng.spagosoa.smx.components.jdbcquery.QueryParameterConfig";

	public ParameterList(String classNameParam) {
		super(classNameParam);
	}
	
	public void addQueryParamConfig(String placeHolder, String isOutput,
		String xpath, String type) {
		
		StringBuffer xmlBean = new StringBuffer("<bean class=\"")
			.append(QUERY_PARAM_CONFIG_CLASS).append("\" >")
			.append("<property name=\"placeHolder\" value=\"")
				.append(placeHolder).append("\" />")
			.append("<property name=\"outputParam\" value=\"")
				.append(isOutput).append("\" />")
			.append("<property name=\"xpath\" value=\"")
				.append(xpath).append("\" />")
			.append("<property name=\"paramType\" value=\"")
				.append(type).append("\" />")
			.append("</bean>");
		addValue(xmlBean.toString());
	}
	
}
