package it.eng.components.cimero;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.gef.palette.ToolEntry;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import cimeroEditor.CimeroEditorPlugin;

import com.bull.cimero.pluginEditor.editors.model.SEModel.ServiceEngineModel;
import com.bull.cimero.pluginEditor.utils.log.CimeroLog;
import com.bull.cimero.pluginEditor.utils.ressources.FileUtils;
import com.bull.cimero.pluginEditor.utils.ressources.Ressources;


 /**
 * The Generic JDBC model class.
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */

public class GenericTalendComponent extends ServiceEngineModel {

	/**serialization number.*/
	private static final long serialVersionUID = 1L;
	/**The location uri of the WS transformeur path.*/
	//private static final String LOCATIONURI = "Location URI";
	/**The name of the destination service property.*/
	private static final String NAMEDEST = "destination service";
	/**The name of the mep mode property.*/
	private static final String INOUTMEP = "in/out MEP";
	/**The default transformer name.*/
	private static final String DEFAULTNAME = "myTalend";
	
	private static final String TALEND_JOB_CLASS_NAME = "talendJobClassName";
	/**the id of this element.*/
	private static final String ID = "TALENDCOMPONENT";
	
	
	/**
	 * The default constructor.
	 */
	public GenericTalendComponent() {
		super(DEFAULTNAME, 1 , DEFAULTOUTIN);
		//add the properties
		setProperty(NAME, DEFAULTNAME);
		setProperty(NAMEDEST, "");
		setProperty(INOUTMEP, 0);
		setProperty(TALEND_JOB_CLASS_NAME, "");
	}
	
	public void configureCustomComponent(){
		/*
		Ressources ressource = new Ressources();
		String fullPath = CimeroEditorPlugin.getRessourcePath("/ressources/CustomComponents.xml");
		if (fullPath != null) {
			
			if (fullPath.substring(0, "file:".length()).compareTo("file:") == 0) {
				fullPath = fullPath.substring("file:".length());
			}
		
			ressource.setDocument(fullPath);
			
			
			NodeList nodeList = ressource.getXPathNodeSet("/COMPONENTS/COMPONENT");
			Node node = null;
			NamedNodeMap attributes = null;
			String tmpId  = null;
			for (int i = 0; i < nodeList.getLength(); i++) {
				node = nodeList.item(i);				
				attributes = node.getAttributes();
				
				tmpId = attributes.getNamedItem("id").getNodeValue();
				if (tmpId.equalsIgnoreCase(getCustomComponentId())){
					setLocationURI(attributes.getNamedItem("talendURI").getNodeValue());
					
				}
			}
		}
		*/
	}
	
	
	

	
	public String getJobClassName(){
		return (String) getProperty(TALEND_JOB_CLASS_NAME);
	}
	/**
	 * inoutMep getter function.
	 * @return true if this component is an inoutmep
	 */
	public final boolean isInOutMep() {
		return ((Integer) getProperty(INOUTMEP) == 0);
	}

	/**
	 * path setter function.
	 * @param inOutMepParam : in or out
	 */
	public final void setInOutMep(final boolean inOutMepParam) {
		if (inOutMepParam) {
			setProperty(INOUTMEP, 0);
		} else {
			setProperty(INOUTMEP, 1);
		}
	}

	/**
	 * The validate function. It is used to determine if the instance class
	 * is valide or not.
	 * @param showLog : true if the log must to be show.
	 * @return boolean
	 */
	public final boolean isValide(final boolean showLog) {
		return true;
	}

	/**
	 * IPropertySource abstract method.
	 * @param value : the value of the property
	 * @param id : the id of the property
	 */
	public final void setPropertyValue(final Object id, final Object value) {
		if (id == NAME){
			System.out.println(id + "==> ["+value+"]");
		}
		if (id == NAMEDEST) {
			//we retreive the destination name
			int selecNum = Integer.parseInt(value.toString());
			//retreive the list af all the objects name
			String [] lstStr = getLstObjectDrawWithInputConnection();
			//call the function to create a connection beetween
			//this object and the selected object
			if (!refreshDestinationConnection(lstStr[selecNum])) {
				CimeroLog.errorLog("problem when created connection",
						new Exception());
			}
		}
		
		
		super.setPropertyValue(id, value);
	}

	/**
	 * IPropertySource abstract method.
	 * @return Object
	 * @param id : the id of the property
	 */
	public final Object getPropertyValue(final Object id) {
		if (NAMEDEST.equals(id)) {
			//we retreive the name of the destination object
			String destinationName = getNameFirstDestination();
			//return the num of the name  of the object
			String [] lstStr = getLstObjectDrawWithInputConnection();
			for (int i = 0; i < lstStr.length; i++) {
				if (lstStr[i].compareTo(destinationName) == 0) {
					return i;
				}
			}
			return 0;
		}else {
			return super.getProperty((String) id);
		}
	}

	/**
	 * IPropertySource abstract method.
	 * @param id : the id of the property
	 */
	public void resetPropertyValue(final Object id) {
		// do nothing
	}

	/**
	 * Abstract method of GeneriqueCimeroModel. It is used to
	 * create the descriptor
	 */
	@Override
	protected final void createDescriptor() {
		int indice = 1;
		addTextDescriptor(NAME, indice++ + " - name");
		addTextDescriptor(TALEND_JOB_CLASS_NAME, indice++ + " - Talend Job Full Class Name");
		//addTextDescriptor(LOCATIONURI, indice++
		//		+ " - select the location URI of the WS");
		//addComboBoxDescriptor(SOAPREQUEST, indice++ + " - soap request ? :",
		//				new String[]{"true", "false"});
		String [] lstStr = getLstObjectDrawWithInputConnection();
		addComboBoxDescriptor(NAMEDEST, indice++
				+ " - select the destination service", lstStr);

	}

	/**
	 * return the id of this element.
	 * @return the id of this element.
	 */
	public final String getComponentId() {
		return ID;
	}
	
	
}
