package it.eng.components.cimero;

import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.SEModel.Transformeur;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.lWFactory.SE.SELWFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.lWFactory.SE.TransformeurFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ActivationSpecClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ComponentClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.LWBeanClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.NameSpaceClass;

/**
 * This class will be use to add in the ServiceMix model, an activation
 * spec that contain a JDBC 
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class GenericTalendFactory  extends SELWFactory {

	/**the class using by ServiceMix to the transformer.*/
	public static final String CLASS =
		"it.eng.spagosoa.smx.components.talend.TalendCaller";

	private static final String TALEND_JOB_CLASS_NAME = "talendJobClassName";

	/**
	 * default constructor.
	 * @param ref : the reference of the current namespace.
	 */
	public GenericTalendFactory(final NameSpaceClass ref) {
		super(ref);
	}
	
	/**
	 * this function will create an activation spec that represents
	 * a transformeur in serviceMix.
	 * @param elt : the element using to create this activation spec.
	 * @param force : false if the validation must to be do.
	 * @return the activation spec create.
	 * @throws Exception
	 * @throws Exception : if the is a error during generation
	 */
	public final ActivationSpecClass createXMLModel(
			final GeneriqueCimeroModel elt,
			final Boolean force) throws Exception {
		ActivationSpecClass myActi = null;
		//if not null and valide and instance of transformer
		if ((elt != null) && (elt instanceof GenericTalendComponent)) {
			if ((elt.isValide(false)) || (force)) {
				GenericTalendComponent monTalendComponent = (GenericTalendComponent) elt;
				myActi = new ActivationSpecClass();
				myActi.setNameSpaceRef(getNamespaceRef());
				myActi.setComponentName(monTalendComponent.getName());
				myActi.setServiceName(monTalendComponent.getName(),
						new NameSpaceClass(NAMESPACENAME , NAMESPACEVALUE));
				setDestinationService(myActi , elt);
				//add the component
				ComponentClass myCompo = createComponent(monTalendComponent);
				myActi.setComponent(myCompo);
				return myActi;
			}
		}
		throw new Exception("The generique cimero object is not valide");

	}

	/**
	 * This function return the servicemix component of a Transformeur.
	 * @param monTrfs : the cimero transformeur
	 * @return the servicemix component.
	 */
	private ComponentClass createComponent(final GenericTalendComponent monTalend) {
		//create the componentclass instance
		ComponentClass monCompo = new ComponentClass(getNamespaceRef());
		//add the bean class.
		LWBeanClass monB = createBeanTalend(monTalend);
		monCompo.setMonBean(monB);
		return monCompo;
	}

	
	private LWBeanClass createBeanTalend(final GenericTalendComponent monTalend) {
		LWBeanClass myBean = new LWBeanClass(CLASS);
		myBean.addProperty(createProperty(TALEND_JOB_CLASS_NAME , monTalend.getJobClassName()), "");
		return myBean;
	}
}

