package it.eng.components.cimero;

import org.eclipse.gef.palette.ToolEntry;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import cimeroEditor.CimeroEditorPlugin;

import com.bull.cimero.pluginEditor.editors.model.SEModel.ServiceEngineModel;
import com.bull.cimero.pluginEditor.utils.log.CimeroLog;
import com.bull.cimero.pluginEditor.utils.ressources.FileUtils;
import com.bull.cimero.pluginEditor.utils.ressources.Ressources;

/**
 * The Generic Transformer
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class GenericTransformer extends ServiceEngineModel {

	/**serialization number.*/
	private static final long serialVersionUID = 1L;
	/**The xsl transformeur path.*/
	private static final String XSLTPATH = "Xslt file Path";
	/**The name of the destination service property.*/
	private static final String NAMEDEST = "destination service";
	/**The name of the mep mode property.*/
	private static final String INOUTMEP = "in/out MEP";
	/**The default transformer name.*/
	private static final String DEFAULTNAME = "myGenericTransformeur";
	/**The size of the xslpath prefixe.*/
	private static final int XSLPREFIXESIZE = 5;
	/**the id of this element.*/
	private static final String ID = "TRANSFORMER";

	/**
	 * The default constructor.
	 */
	public GenericTransformer() {
		super(DEFAULTNAME, 1 , DEFAULTOUTIN);
		//add the properties
		setProperty(NAME, DEFAULTNAME);
		setProperty(NAMEDEST, "");
		setProperty(INOUTMEP, 0);
		
	}
	
	public void configureCustomComponent(){
		Ressources ressource = new Ressources();
		String fullPath = CimeroEditorPlugin.getRessourcePath("/ressources/CustomComponents.xml");
		if (fullPath != null) {
			
			if (fullPath.substring(0, "file:".length()).compareTo("file:") == 0) {
				fullPath = fullPath.substring("file:".length());
			}
		
			ressource.setDocument(fullPath);
			
			
			NodeList nodeList = ressource.getXPathNodeSet("/COMPONENTS/COMPONENT");
			Node node = null;
			NamedNodeMap attributes = null;
			String tmpId  = null;
			Class clazz = null;
			ToolEntry  tool = null;
			for (int i = 0; i < nodeList.getLength(); i++) {
				node = nodeList.item(i);				
				attributes = node.getAttributes();
				
				tmpId = attributes.getNamedItem("id").getNodeValue();
				if (tmpId.equalsIgnoreCase(getCustomComponentId())){
					setXsltPath(attributes.getNamedItem("xsltUri").getNodeValue());
				}
			}
		}
	}
	

	/**
	 * Path getter function.
	 * @return String
	 */
	public final String getXsltPath() {
		return (String) getProperty(XSLTPATH);
	}

	/**
	 * path setter function.
	 * @param pathParam : the path of the xslt file
	 */
	public final void setXsltPath(final String pathParam) {
		setProperty(XSLTPATH, pathParam);
	}

	/**
	 * inoutMep getter function.
	 * @return true if this component is an inoutmep
	 */
	public final boolean isInOutMep() {
		return ((Integer) getProperty(INOUTMEP) == 0);
	}

	/**
	 * path setter function.
	 * @param inOutMepParam : in or out
	 */
	public final void setInOutMep(final boolean inOutMepParam) {
		if (inOutMepParam) {
			setProperty(INOUTMEP, 0);
		} else {
			setProperty(INOUTMEP, 1);
		}
	}

	/**
	 * The validate function. It is used to determine if the instance class
	 * is valide or not.
	 * @param showLog : true if the log must to be show.
	 * @return boolean
	 */
	public final boolean isValide(final boolean showLog) {
		String msg = "";
		boolean returnResult = true;
		//name verification
		if (getName().compareTo("") == 0) {
			msg += "\n1 - Name => the transformer name cannot be null";
			returnResult = false;
		}
		//xsl path verification
		if (getXsltPath().compareTo("") == 0) {
			msg += "\n2 - Xslt file path => set the xslt file path";
			returnResult = false;
		} else if (getXsltPath().substring(0 , XSLPREFIXESIZE)
							.compareTo("file:") != 0) {
			msg += "\n2 - Xslt file path => "
				+ " the path must to begin with \"file:\"";
			returnResult = false;
		} else {
			//test if the xslt file exists
			String subStr = getXsltPath().substring(XSLPREFIXESIZE ,
					getXsltPath().length());
			if (!FileUtils.getDefault().isFile((subStr))) {
				msg += "\n2 - Xslt file path => file doesn't exist";
				returnResult = false;
			}
		}
		/*
		if (getNameFirstDestination().compareTo("") == 0) {
			msg += "\n3 - Destination service => the destination of this "
						+ "element cannot be null!";
			returnResult = false;
		}
		*/
		if (!returnResult) {
			msg = msg.substring(1);
			msg = "ELEMENT NAME : " + getName() + " \n" + msg;
			if (showLog) {
				CimeroLog.warningLog(msg);
			}
		}
		return returnResult;
	}

	/**
	 * IPropertySource abstract method.
	 * @param value : the value of the property
	 * @param id : the id of the property
	 */
	public final void setPropertyValue(final Object id, final Object value) {
		if (id == NAME){
			System.out.println(id + "==> ["+value+"]");
		}
		if (id == NAMEDEST) {
			//we retreive the destination name
			int selecNum = Integer.parseInt(value.toString());
			//retreive the list af all the objects name
			String [] lstStr = getLstObjectDrawWithInputConnection();
			//call the function to create a connection beetween
			//this object and the selected object
			if (!refreshDestinationConnection(lstStr[selecNum])) {
				CimeroLog.errorLog("problem when created connection",
						new Exception());
			}
		}
		super.setPropertyValue(id, value);
	}

	/**
	 * IPropertySource abstract method.
	 * @return Object
	 * @param id : the id of the property
	 */
	public final Object getPropertyValue(final Object id) {
		if (NAMEDEST.equals(id)) {
			//we retreive the name of the destination object
			String destinationName = getNameFirstDestination();
			//return the num of the name  of the object
			String [] lstStr = getLstObjectDrawWithInputConnection();
			for (int i = 0; i < lstStr.length; i++) {
				if (lstStr[i].compareTo(destinationName) == 0) {
					return i;
				}
			}
			return 0;
		} else {
			return super.getProperty((String) id);
		}
	}

	/**
	 * IPropertySource abstract method.
	 * @param id : the id of the property
	 */
	public void resetPropertyValue(final Object id) {
		// do nothing
	}

	/**
	 * Abstract method of GeneriqueCimeroModel. It is used to
	 * create the descriptor
	 */
	protected  void createDescriptor() {
		int indice = 1;
		String [] lstStr = getLstObjectDrawWithInputConnection();
		addTextDescriptor(NAME, indice++ + " - name");
		addTextDescriptor(XSLTPATH, indice++ + " - select the xslt file path");

		addComboBoxDescriptor(NAMEDEST, indice++
				+ " - select the destination service", lstStr);
	}

	/**
	 * return the id of this element.
	 * @return the id of this element.
	 */
	public final String getComponentId() {
		return ID;
	}
}
