package it.eng.components.citizendb;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.gef.palette.ToolEntry;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import cimeroEditor.CimeroEditorPlugin;

import com.bull.cimero.pluginEditor.editors.model.SEModel.ServiceEngineModel;
import com.bull.cimero.pluginEditor.utils.log.CimeroLog;
import com.bull.cimero.pluginEditor.utils.ressources.FileUtils;
import com.bull.cimero.pluginEditor.utils.ressources.Ressources;


 /**
 * The Generic JDBC model class.
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */

public class CDB10_EstrazioneAltreRelazioniPerSoggetto extends ServiceEngineModel implements ICitizenDBComponent {

	/**serialization number.*/
	protected static final long serialVersionUID = 1L;
	/**The xsl transformeur path.*/
	protected static final String DATASOURCE = "Datasource";
	/**The name of the destination service property.*/
	protected static final String NAMEDEST = "destination service";
	/**The name of the mep mode property.*/
	protected static final String INOUTMEP = "in/out MEP";
	
	
	private Map<String, Integer> dataSourceIds = null;
	private Map<Integer,String > dataSourcePositions = null;
	/**
	 * The default constructor.
	 */
	/**The default transformer name.*/
	private static final String DEFAULTNAME = "myEstrazioneAltreRelazioniPerSoggetto";
	
	/**the id of this element.*/
	private static final String ID = "CDB10_EstrazioneAltreRelazioniPerSoggetto";
	
	public CDB10_EstrazioneAltreRelazioniPerSoggetto() {
		super(DEFAULTNAME, 1 , DEFAULTOUTIN);
		//add the properties
		setProperty(NAME, DEFAULTNAME);
		setProperty(NAMEDEST, "");
		setProperty(INOUTMEP, 0);
	}
	
	public void configureCustomComponent(){
		this.dataSourceIds = new HashMap<String, Integer>();
		this.dataSourcePositions = new HashMap<Integer, String>();
		
		Ressources ressource = new Ressources();
		String fullPath = CimeroEditorPlugin.getRessourcePath("/ressources/DataSources.xml");
		String first = "";
		if (fullPath != null) {
			
			if (fullPath.substring(0, "file:".length()).compareTo("file:") == 0) {
				fullPath = fullPath.substring("file:".length());
			}
		
			ressource.setDocument(fullPath);
			
			
			NodeList nodeList = ressource.getXPathNodeSet("/DATASOURCES/DATASOURCE");
			Node node = null;
			NamedNodeMap attributes = null;
			String tmpId  = null;
			Class clazz = null;

			for (int i = 0; i < nodeList.getLength(); i++) {
				
				node = nodeList.item(i);				
				attributes = node.getAttributes();
				dataSourceIds.put(attributes.getNamedItem("id").getNodeValue(), i);
				if (i == 0)
					first = attributes.getNamedItem("id").getNodeValue();
				dataSourcePositions.put(i, attributes.getNamedItem("id").getNodeValue());
			}
		}
		setProperty(DATASOURCE, first);
	}
	

	

	

	/**
	 * inoutMep getter function.
	 * @return true if this component is an inoutmep
	 */
	public final boolean isInOutMep() {
		return ((Integer) getProperty(INOUTMEP) == 0);
	}

	/**
	 * path setter function.
	 * @param inOutMepParam : in or out
	 */
	public final void setInOutMep(final boolean inOutMepParam) {
		if (inOutMepParam) {
			setProperty(INOUTMEP, 0);
		} else {
			setProperty(INOUTMEP, 1);
		}
	}

	/**
	 * The validate function. It is used to determine if the instance class
	 * is valide or not.
	 * @param showLog : true if the log must to be show.
	 * @return boolean
	 */
	public final boolean isValide(final boolean showLog) {
		String msg = "";
		boolean returnResult = true;
		//name verification
		if (getName().compareTo("") == 0) {
			msg += "\n1 - Name => the transformer name cannot be null";
			returnResult = false;
		}
		

		
		if (!returnResult) {
			msg = msg.substring(1);
			msg = "ELEMENT NAME : " + getName() + " \n" + msg;
			if (showLog) {
				CimeroLog.warningLog(msg);
			}
		}
		return returnResult;
	}

	/**
	 * IPropertySource abstract method.
	 * @param value : the value of the property
	 * @param id : the id of the property
	 */
	public final void setPropertyValue(final Object id, final Object value) {
		if (id == NAME){
			System.out.println(id + "==> ["+value+"]");
		}
		if (id == NAMEDEST) {
			//we retreive the destination name
			int selecNum = Integer.parseInt(value.toString());
			//retreive the list af all the objects name
			String [] lstStr = getLstObjectDrawWithInputConnection();
			//call the function to create a connection beetween
			//this object and the selected object
			if (!refreshDestinationConnection(lstStr[selecNum])) {
				CimeroLog.errorLog("problem when created connection",
						new Exception());
			}
		}
		
		if (id == DATASOURCE){
			int intValue = ((Integer)value).intValue();
			setProperty(DATASOURCE, dataSourcePositions.get(intValue));
			return;
		}
		super.setPropertyValue(id, value);
	}

	/**
	 * IPropertySource abstract method.
	 * @return Object
	 * @param id : the id of the property
	 */
	public final Object getPropertyValue(final Object id) {
		if (NAMEDEST.equals(id)) {
			//we retreive the name of the destination object
			String destinationName = getNameFirstDestination();
			//return the num of the name  of the object
			String [] lstStr = getLstObjectDrawWithInputConnection();
			for (int i = 0; i < lstStr.length; i++) {
				if (lstStr[i].compareTo(destinationName) == 0) {
					return i;
				}
			}
			return 0;
		} else if (DATASOURCE.equals(id)) {
			String propertyDs = (String)getProperty(DATASOURCE);
			if (propertyDs == null || propertyDs.trim().length() == 0){
				return 0;
			}else{
				return dataSourceIds.get(propertyDs);
			}
		}else {
			return super.getProperty((String) id);
		}
	}

	/**
	 * IPropertySource abstract method.
	 * @param id : the id of the property
	 */
	public void resetPropertyValue(final Object id) {
		// do nothing
	}

	/**
	 * Abstract method of GeneriqueCimeroModel. It is used to
	 * create the descriptor
	 */
	protected  void createDescriptor() {
		int indice = 1;
		String [] lstStr = getLstObjectDrawWithInputConnection();
		addTextDescriptor(NAME, indice++ + " - name");
		
		Ressources ressource = new Ressources();
		String fullPath = CimeroEditorPlugin.getRessourcePath("/ressources/DataSources.xml");
		String first = "";
		String[] comboDs = null;
		if (fullPath != null) {
			
			if (fullPath.substring(0, "file:".length()).compareTo("file:") == 0) {
				fullPath = fullPath.substring("file:".length());
			}
		
			ressource.setDocument(fullPath);
			
			
			NodeList nodeList = ressource.getXPathNodeSet("/DATASOURCES/DATASOURCE");
			Node node = null;
			NamedNodeMap attributes = null;
			
			comboDs = new String[nodeList.getLength()];
			for (int i = 0; i < nodeList.getLength(); i++) {
				node = nodeList.item(i);				
				attributes = node.getAttributes();
				comboDs[i] = (String)attributes.getNamedItem("id").getNodeValue();
			}
		}
		
		
		addComboBoxDescriptor(DATASOURCE, indice++ +" - dataSource", comboDs);

		addComboBoxDescriptor(NAMEDEST, indice++
				+ " - select the destination service", lstStr);
	}

	/**
	 * return the id of this element.
	 * @return the id of this element.
	 */
	public final String getComponentId() {
		return ID;
	}
	
	public final String getDataSource() {
		return (String) getProperty(DATASOURCE);
	}

	public String getCitizenDBService() {
		return ID;
	}
}
