package it.eng.components.citizendb;

import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;
import com.bull.cimero.pluginEditor.editors.model.SEModel.Transformeur;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.lWFactory.SE.SELWFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.factory.lWFactory.SE.TransformeurFactory;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ActivationSpecClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.ComponentClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.LWBeanClass;
import com.bull.cimero.pluginEditor.generator.serviceMixGenerator.model.global.NameSpaceClass;

/**
 * This class will be use to add in the ServiceMix model, an activation
 * spec that contain a JDBC 
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class CitizendDBFactory  extends SELWFactory {

	/**the class using by ServiceMix to the transformer.*/
	public static final String CLASS =
		"it.eng.crm.services.citizendb.GenericCitizenDBComponent";
	
	/**The property name to set the xslt file.*/
	private static final String DATASOURCE = "dataSource";
	
	private static final String CITIZEN_DB_SERVICE = "citizenDBService";

	/**
	 * default constructor.
	 * @param ref : the reference of the current namespace.
	 */
	public CitizendDBFactory(final NameSpaceClass ref) {
		super(ref);
	}
	
	/**
	 * this function will create an activation spec that represents
	 * a transformeur in serviceMix.
	 * @param elt : the element using to create this activation spec.
	 * @param force : false if the validation must to be do.
	 * @return the activation spec create.
	 * @throws Exception
	 * @throws Exception : if the is a error during generation
	 */
	public final ActivationSpecClass createXMLModel(
			final GeneriqueCimeroModel elt,
			final Boolean force) throws Exception {
		ActivationSpecClass myActi = null;
		//if not null and valide and instance of transformer
		if ((elt != null)) {
			if ((elt.isValide(false)) || (force)) {
				ICitizenDBComponent monCitizenDBComponent = (ICitizenDBComponent) elt;
				myActi = new ActivationSpecClass();
				myActi.setNameSpaceRef(getNamespaceRef());
				myActi.setComponentName(monCitizenDBComponent.getName());
				myActi.setServiceName(monCitizenDBComponent.getName(),
						new NameSpaceClass(NAMESPACENAME , NAMESPACEVALUE));
				setDestinationService(myActi , elt);
				//add the component
				ComponentClass myCompo = createComponent(monCitizenDBComponent);
				myActi.setComponent(myCompo);
				return myActi;
			}
		}
		throw new Exception("The generique cimero object is not valide");

	}

	/**
	 * This function return the servicemix component of a Transformeur.
	 * @param monTrfs : the cimero transformeur
	 * @return the servicemix component.
	 */
	private ComponentClass createComponent(final ICitizenDBComponent monCitizenDBComponent) {
		//create the componentclass instance
		ComponentClass monCompo = new ComponentClass(getNamespaceRef());
		//add the bean class.
		LWBeanClass monB = createBeanCitizenDB(monCitizenDBComponent);
		monCompo.setMonBean(monB);
		return monCompo;
	}

	
	private LWBeanClass createBeanCitizenDB(final ICitizenDBComponent monCitizenDBComponent) {
		LWBeanClass myBean = new LWBeanClass(CLASS);
		myBean.addProperty(createPropertyRef(DATASOURCE , monCitizenDBComponent.getDataSource()), "");
		myBean.addProperty(createProperty(CITIZEN_DB_SERVICE , monCitizenDBComponent.getCitizenDBService()), "");
		return myBean;
	}
}

